package org.cn3.morpho.analysis;

import java.io.File;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.SWCReader;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.utils.BinaryTreeUtils;
import org.cn3.morpho.utils.SwcDataUtils;

/**
 * 
 * @author gillette
 * 
 * Determines quantiles for the distribution of bifurcation angles in the
 * SWC dataset provided. The intention was to provide cutoff values on which to 
 * base the encoding of bifurcations beyond topology.
 *  
 * This program is mostly hard-coded and requires work to generalize.
 */
public class RemoteAngleDistributionAnalyzer {

	public static void main(String args[]){
		try{
			File swcDirectory = new File("C:/swcData/NM/");
			File[] swcFiles, archiveDirs = swcDirectory.listFiles();
			List<BinaryTreeNode> trees, nodes;
			DistributionBean angles = new DistributionBean();
			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(2);
			double angle;
			
			StringBuffer sb = new StringBuffer();
			sb.append("ArborType,Angle\n");
			
			/* Objects for multifurcation cases - incomplete and currently being ignored
			Map<BinaryTreeNode,Double> nodeAngleMap = new HashMap<BinaryTreeNode,Double>(4);
			double minDist; */
			// Go through each archive
			for (File archiveDir : archiveDirs){
				if (archiveDir.isDirectory()){
					archiveDir = new File(archiveDir.getAbsoluteFile()+"/CNG version/");
					swcFiles = archiveDir.listFiles();
					// Go through each neuron in the archive
					for (File swcFile : swcFiles){
						if (swcFile.getName().endsWith(".swc")){
							// Get each individual tree of the neuron rooted from the soma
							System.out.println(swcFile.getName());
							trees = SWCReader.convertSwcToBinaryTreeList(swcFile);
							
							for (BinaryTreeNode tree : trees){
								// Calculate bifurcation angle for each bifurcation
								nodes = BinaryTreeUtils.createBifurcationList(tree);
								for (BinaryTreeNode node : nodes){
									/* Code for determining angle in a multifurcation; skipping since multifurcation are very rare
									minDist = 0;
									// Get child's children if it is too close and thus unreliable and possibly from a multifurcation 
									double leftDist = SwcDataUtils.getDistance(node.getSwcData(), node.getLeft().getSwcData());
									double rightDist = SwcDataUtils.getDistance(node.getSwcData(), node.getRight().getSwcData());
									if (SwcDataUtils.getDistance(node1, node2))
									*/
									
									// Calculate angle between bifurcation and its child nodes
									angle = SwcDataUtils.getAngle(
											node.getSwcData(), 
											node.getLeft().getSwcData(), 
											node.getRight().getSwcData());
									angles.addData(angle);
									sb.append(node.getSwcData().getType()+","+nf.format(angle)+"\n");
								}
							}
						}
					}
				}
			}
			
			// Write angles to file
			DataWriter.writeStringToFile("C:/Documents and Settings/gillette/My Documents/research/NMOBifurcationAngles.csv", sb.toString());
			
			// Calculate quartiles and quintiles
			System.out.println("Quartiles: "+nf.format(angles.getPercentile(.25))+", "+nf.format(angles.getPercentile(.5))+", "+nf.format(angles.getPercentile(.75)));
			System.out.println("Quintiles: "+nf.format(angles.getPercentile(.2))+", "+nf.format(angles.getPercentile(.4))+", "+nf.format(angles.getPercentile(.6))+", "+nf.format(angles.getPercentile(.8)));
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
}
