package org.cn3.morpho.data;

import java.io.File;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.resources.MorphoDataProperties;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class FindNewCells {

	private static final String PARAM_SWC_BASE_DIR = "swc-base-dir";
	private static final String PARAM_SWC_DATASET = "swc-dataset";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			File dir = getSwcDirectory(config);
			
			Pattern p = Pattern.compile("(.*)\\.CNG.swc");
			Matcher m;
			
			File[] labDirs = dir.listFiles();
			File cngDir;
			File[] cellFiles;
			Map<String,NeuronMetadata> metaData = MetaDataManager.getCellMetaData();
			int total = 0;
			for (int i = 0; i < labDirs.length; i++){
				if (labDirs[i].isDirectory()){
					cngDir = new File(labDirs[i].getAbsoluteFile()+"/CNG version");
					if (cngDir.exists() && cngDir.isDirectory()){
						cellFiles = cngDir.listFiles();
						for (int j = 0; j < cellFiles.length; j++){
							m = p.matcher(cellFiles[j].getName());
							if (m.matches()){
								if (!metaData.containsKey(m.group(1))){
									System.out.println(labDirs[i].getName()+": "+m.group(1));
									total++;
								}
							}
							else{
								throw new Exception(cellFiles[j].getName()+" does not match pattern");
							}
						}
					}
				}
			}
			System.out.println("Total: "+total);
			
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static File getSwcDirectory(JSAPResult config) throws Exception{
		File dir = new File(config.getString(PARAM_SWC_BASE_DIR));
		String dataset = config.getString(PARAM_SWC_DATASET);
		if (dir.equals("")){
			dir = new File(MorphoDataProperties.getInstance().getSwcDir(dataset));
		}
		if (!dir.exists()){
			throw new Exception("Directory "+dir.getAbsolutePath()+" does not exist");
		}
		if (!dir.isDirectory()){
			throw new Exception(dir.getAbsolutePath()+" is not a directory");
		}
		return(dir);
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
		FlaggedOption opt;
		
        opt = new FlaggedOption(PARAM_SWC_BASE_DIR)
           .setStringParser(JSAP.STRING_PARSER)
           .setRequired(false)
           .setDefault("")
           .setShortFlag('s') 
           .setLongFlag(PARAM_SWC_BASE_DIR);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_SWC_DATASET)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(false)
        .setDefault("NM")
        .setShortFlag('d') 
        .setLongFlag(PARAM_SWC_DATASET);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + FindNewCells.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}
	
}
