package org.cn3.morpho.data;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.persistence.NeuronDAO;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.domain.NeuronMetadataComplete;
import org.cn3.morpho.resources.MorphoDataProperties;
import org.cn3.morpho.resources.MorphoProjectProperties;


/**
 * 
 * @author gillette
 * Handles various types of meta-data. Manager layer in case method
 * of handling meta data (files, database, etc) changes.
 * IN PROGRESS
 *
 */
public class MetaDataManager {

	private static String source = null;
	private static Map<String,NeuronMetadata> cellMetaData = null;
	private static Map<String,NeuronMetadataComplete> extendedCellMetaData = null;
	private static Map<String,Set<String>> cellMetaDataSets = null;
	
	public static final String CELL_TYPE_LIST = "cellTypeList";

	public static Map<String,Set<String>> getCellMetaDataSets() throws Exception{
		if (cellMetaDataSets == null){
			cellMetaDataSets = new HashMap<String,Set<String>>();
			Map<String,?> cellMetaData = getCellMetaData();
			Set<String> set;
			NeuronMetadata neuron;
			String cellType, brainRegion, animal;
			for (String neuronName : cellMetaData.keySet()){
				if (!neuronName.equals(CELL_TYPE_LIST)){
					neuron = (NeuronMetadata)cellMetaData.get(neuronName);
					cellType = neuron.getCellType().toLowerCase();
					brainRegion = neuron.getBrainRegion().toLowerCase();
					animal = neuron.getAnimal().toLowerCase();
					
					if (!cellMetaDataSets.containsKey(cellType)){
						cellMetaDataSets.put(cellType, new HashSet<String>());
					}
					set = cellMetaDataSets.get(cellType);
					set.add(neuronName);
	
					if (!cellMetaDataSets.containsKey(brainRegion)){
						cellMetaDataSets.put(brainRegion, new HashSet<String>());
					}
					set = cellMetaDataSets.get(brainRegion);
					set.add(neuronName);
					
					if (!cellMetaDataSets.containsKey(animal)){
						cellMetaDataSets.put(animal, new HashSet<String>());
					}
					set = cellMetaDataSets.get(animal);
					set.add(neuronName);
				}
			}
		}
		return cellMetaDataSets;
	}
	
	public static Map<String,NeuronMetadataComplete> getExtendedCellMetaData() throws Exception{
		if (extendedCellMetaData == null){
			source = MorphoProjectProperties.getInstance().getDataSource();
			if (source.equals(MorphoProjectProperties.DATASOURCE_DB)){
				extendedCellMetaData = getExtendedCellMetaDataFromDB();
			}
			else {
				extendedCellMetaData = getExtendedCellMetaDataFromFile();
			}
		}
		return extendedCellMetaData;
	}
	public static Map<String,NeuronMetadata> getCellMetaData() throws Exception{
		if (cellMetaData == null){
			source = MorphoProjectProperties.getInstance().getDataSource();
			if (source.equals(MorphoProjectProperties.DATASOURCE_DB)){
				try{
					cellMetaData = getCellMetaDataFromDB();
				} catch (Exception e){
					System.err.println("Failed to get metadata from database, trying to retrieve from file");
					cellMetaData = getCellMetaDataFromFile(MorphoDataProperties.getInstance().getMetaDataFilename());
				}
			}
			else {
				cellMetaData = getCellMetaDataFromFile(MorphoDataProperties.getInstance().getMetaDataFilename());
			}
		}
		System.out.println(cellMetaData.size());
		return cellMetaData;
	}
	
	public static HashMap<String,NeuronMetadata> getCellMetaDataFromDB() throws Exception{
		NeuronDAO neuronDAO = NeuronDAO.getInstance();
		List<NeuronMetadata> neurons = neuronDAO.getNeurons();
		HashMap<String,NeuronMetadata> dataMap = new HashMap<String,NeuronMetadata>();
		//Map<String,Integer> cellTypeMap = new HashMap<String,Integer>();

		NeuronMetadata neuron;
		for (int i = 0; i < neurons.size(); i++){
			neuron = neurons.get(i);
			dataMap.put(neuron.getName(), neuron);
		}
	
		return dataMap;
	}
	public static HashMap<String,NeuronMetadataComplete> getExtendedCellMetaDataFromDB() throws Exception{
		NeuronDAO neuronDAO = NeuronDAO.getInstance();
		List<NeuronMetadataComplete> neurons = neuronDAO.getNeuronInfos();
		HashMap<String,NeuronMetadataComplete> dataMap = new HashMap<String,NeuronMetadataComplete>();
		//Map<String,Integer> cellTypeMap = new HashMap<String,Integer>();

		NeuronMetadataComplete neuron;
		for (int i = 0; i < neurons.size(); i++){
			neuron = neurons.get(i);
			dataMap.put(neuron.getName(), neuron);
/*			
			// Add cell type to list
    		int count = 0;
    		if (cellTypeMap.containsKey(neuron.getCellType())){
    			count = cellTypeMap.remove(neuron.getCellType()).intValue();
    		}
    		cellTypeMap.put(neuron.getCellType(), new Integer(count+1));
    		*/
		}
//		dataMap.put(CELL_TYPE_LIST,cellTypeMap);
		return dataMap;
	}
	
	public static Map<String,NeuronMetadata> getCellMetaDataFromFile() throws Exception{
		return getCellMetaDataFromFile(new File(MorphoProjectProperties.getInstance().getDataSource()));
	}

	public static Map<String,NeuronMetadata> getCellMetaDataFromFile(String inputFileString) throws Exception{
		return getCellMetaDataFromFile(new File(inputFileString));
	}
	
	public static Map<String,NeuronMetadataComplete> getExtendedCellMetaDataFromFile() throws Exception{
		return getExtendedCellMetaDataFromFile(new File(MorphoDataProperties.getInstance().getMetaDataFilename()));
	}
	public static Map<String,NeuronMetadataComplete> getExtendedCellMetaDataFromFile(String inputFileString) throws Exception{
		return getExtendedCellMetaDataFromFile(new File(inputFileString));
	}
	public static Map<String,NeuronMetadataComplete> getExtendedCellMetaDataFromFile(File inputFile) throws Exception{
		FileReader freader = null;
	    BufferedReader reader = null;
	    Map<String,NeuronMetadataComplete> dataMap = null;

	    try{
		    freader = new FileReader(inputFile);
		    reader = new BufferedReader(freader);

			String dataString = "";
			dataMap = new HashMap<String,NeuronMetadataComplete>();
			//name,class2,species,region1,archive(,class1,class3,region2,region3,ageClassification,gender,strain,protocol)
			Pattern p = Pattern.compile(" *\"([^#\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)(\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"?[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\")?");
			//Pattern p = Pattern.compile(" *\"?([^#,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)(\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*)\"?[,\\t]\"?([^,\\t\"]*))?$");
			Matcher m;
			NeuronMetadataComplete neuron;
//			Map<String,Integer> cellTypeMap = new HashMap<String,Integer>();
			
			while ( (dataString=reader.readLine()) != null ) { 
		        m = p.matcher(dataString);
		        //System.out.println(dataString);
		        if (m.matches()){
	        		neuron = new NeuronMetadataComplete();
	        		neuron.setName(m.group(1));
	        		neuron.setCellType(m.group(2));
	        		neuron.setAnimal(m.group(3));
	        		neuron.setBrainRegion(m.group(4));
	        		neuron.setLabName(m.group(5));
	        		
	        		if (m.group(6) != null){
	        			// Load extended meta data
	        			neuron.setClass1(m.group(7));
	        			neuron.setClass3(m.group(8));
	        			neuron.setRegion2(m.group(9));
	        			neuron.setRegion3(m.group(10));
	        			neuron.setAgeClassification(m.group(11));
	        			neuron.setGender(m.group(12));
	        			neuron.setStrain(m.group(13));
	        			neuron.setProtocol(m.group(14));
	        			neuron.setExperimentCondition(m.group(15));
	        		}
	        		
	        		dataMap.put(neuron.getName(), neuron);
/*	        		
	        		// Add cell type to list
	        		int count = 0;
	        		if (cellTypeMap.containsKey(cellType)){
	        			count = ((Integer)cellTypeMap.remove(cellType)).intValue();
	        		}
	        		cellTypeMap.put(cellType, new Integer(count+1));
	*/        		
	        	}
//		        dataMap.put(CELL_TYPE_LIST,cellTypeMap);
			}
	    } catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}

		return dataMap;
	}
	
	public static Map<String,NeuronMetadata> getCellMetaDataFromFile(File inputFile) throws Exception{
		FileReader freader = null;
	    BufferedReader reader = null;
	    Map<String,NeuronMetadata> dataMap = null;

	    try{
		    freader = new FileReader(inputFile);
		    reader = new BufferedReader(freader);

			String dataString = "";
			dataMap = new HashMap<String,NeuronMetadata>();
			//dataMap = new LinkedHashMap<String,NeuronMetadata>();
			//Pattern p = Pattern.compile(" *([^#,\\t]*)[,\\t]([^,\\t]*)[,\\t]([^,\\t]*)[,\\t]([^,\\t]*)[,\\t]([^,\\t]*)([,\\t].*)?$");
			Pattern p = Pattern.compile(" *\"([^#\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"[,\\t]\"([^\"]*)\"([,\\t].*)?$");
			Matcher m;
			NeuronMetadata neuron;
			//Map<String,Integer> cellTypeMap = new HashMap<String,Integer>();
			
			while ( (dataString=reader.readLine()) != null ) { 
		        m = p.matcher(dataString);
		        //System.out.println(dataString);
		        if (m.matches()){
	        		neuron = new NeuronMetadata();
	        		neuron.setName(m.group(1));
	        		neuron.setCellType(m.group(2));
	        		neuron.setAnimal(m.group(3));
	        		neuron.setBrainRegion(m.group(4));
	        		neuron.setLabName(m.group(5));
	        		dataMap.put(neuron.getName(), neuron);
/*	        		
	        		// Add cell type to list
	        		int count = 0;
	        		if (cellTypeMap.containsKey(cellType)){
	        			count = ((Integer)cellTypeMap.remove(cellType)).intValue();
	        		}
	        		cellTypeMap.put(cellType, new Integer(count+1));
	*/        		
	        	}
	//	        dataMap.put(CELL_TYPE_LIST,cellTypeMap);
			}
	    } catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}

		return dataMap;
	}

}
