package org.cn3.morpho.data;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.domain.NeuriteType;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.resources.MorphoConstants;
import org.cn3.morpho.resources.MorphoDataProperties;
import org.cn3.morpho.utils.StringUtils;


public class MorphoDataReader {
	
	private static Set<String> flaggedNeurons = null;
	public static Set<String> getFlaggedNeurons() throws IOException{
		if (flaggedNeurons == null){
			flaggedNeurons = new TreeSet<String>();
			FileReader freader = null;
		    BufferedReader reader = null;

		    try{
			    freader = new FileReader(MorphoDataProperties.getInstance().getFlaggedNeuronsFile());
			    reader = new BufferedReader(freader);

				String dataString = "";
				
				while ( (dataString=reader.readLine()) != null ) {
					if (!StringUtils.isEmpty(dataString)){
						flaggedNeurons.add(dataString);
					}
				}
				
			} catch (IOException ioe){
				throw ioe;
			} finally{
				if (reader != null){
					reader.close();
				}
				if (freader != null){
					freader.close();
				}
			}
		}
		Set<String> copy = new TreeSet<String>(flaggedNeurons);
		return copy;
	}
	
	public static Map<String,NeuronMetadata> getCellMetaData() throws Exception{
		return MetaDataManager.getCellMetaData();
	}

	public static Set<String> loadSet(String filename)
	throws Exception{
		return loadSet(new File(filename));
	}
	public static Set<String> loadSet(File file)
	throws Exception{
		Set<String> set = new TreeSet<String>();
		FileReader freader = null;
	    BufferedReader reader = null;
	    try{
		    freader = new FileReader(file);
		    reader = new BufferedReader(freader);
			String dataString = "";
			
			while ( (dataString=reader.readLine()) != null ) { 
		        set.add(dataString);
			}
		} catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}
		return set;
	}

	public static StringBuffer fileToStringBuffer(String file, StringBuffer sb)
	throws Exception{
		return fileToStringBuffer(new File(file), sb);
	}
	public static StringBuffer fileToStringBuffer(File file, StringBuffer sb)
	throws Exception{
		if (sb == null){
			sb = new StringBuffer();
		}
		FileReader freader = null;
	    BufferedReader reader = null;

	    try{
		    freader = new FileReader(file);
		    reader = new BufferedReader(freader);

			String dataString = "";
			
			while ( (dataString=reader.readLine()) != null ) { 
		        sb.append(dataString+"\r\n");
			}
			
		} catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}
		return sb;
	}
	
	private static Map<String,Properties> propMap = new HashMap<String,Properties>();
	
	public static Properties loadProperties() {
		return loadProperties(MorphoConstants.DATA_PROPERTIES_FILE);
	}

	public static Properties loadProperties(String propertiesFile) {
		return loadProperties(propertiesFile, false);
	}
	public static Properties loadProperties(String propertiesFile, boolean refresh) {
		try{
			if (refresh || !propMap.containsKey(propertiesFile)){
				Pattern p = Pattern.compile(".+\\.properties");
				Matcher m = p.matcher(propertiesFile);
				if (!m.matches()){
					propertiesFile = propertiesFile + ".properties";
				}
				Properties properties = new Properties();
				properties.load(new FileInputStream("resources/"+propertiesFile));
				propMap.put(propertiesFile, properties);
			}
			return (Properties)propMap.get(propertiesFile);
		} catch (IOException ioe){
			ioe.printStackTrace();
		}
		return null;
	}
	
	private static List<String> rawWellRepresentedNeuriteClasses = null;
	private static List<NeuriteType> wellRepresentedNeuriteClasses = null;
	public static List<String> getRawWellRepresentedNeuriteClassesFromFile() throws Exception{
		if (rawWellRepresentedNeuriteClasses == null){
			rawWellRepresentedNeuriteClasses = new ArrayList<String>();
			FileReader freader = null;
			BufferedReader reader = null;
			String dataString;
			try{
				freader = new FileReader(MorphoDataProperties.getInstance().getWellRepresentedNeuriteClassesFilename());
			    reader = new BufferedReader(freader);

				while ( (dataString=reader.readLine()) != null) {
					if (dataString.charAt(0) != '#'){
						rawWellRepresentedNeuriteClasses.add(dataString);
					}
			    }
			} catch (IOException ioe){
				throw new Exception(ioe);
			} finally{
				if (reader != null){
					reader.close();
				}
				if (freader != null){
					freader.close();
				}
			}
		}
		List<String> copy = new ArrayList<String>(rawWellRepresentedNeuriteClasses);
		return copy;
	}

	public static List<NeuriteType> getWellRepresentedNeuriteClassesFromFile() throws Exception{
		if (wellRepresentedNeuriteClasses == null){
			if (rawWellRepresentedNeuriteClasses == null){
				getRawWellRepresentedNeuriteClassesFromFile();
			}
			wellRepresentedNeuriteClasses = new ArrayList<NeuriteType>();
			String string, arborType, class1=null, class2=null, brainRegion=null;
			String[] types;
			for (int i = 0; i < rawWellRepresentedNeuriteClasses.size(); i++){
				string = (String)rawWellRepresentedNeuriteClasses.get(i);
				types = string.split("\\/");
				arborType = types[0];
				if (types.length > 1){
					class1 = types[1];
					if (types.length > 2){
						class2 = types[2];
						if (types.length > 3){
							brainRegion = types[3];
						}
					}
				}
				wellRepresentedNeuriteClasses.add(
						new NeuriteType(arborType,class1,class2,brainRegion));
			}
		}
		List<NeuriteType> copy = new ArrayList<NeuriteType>(wellRepresentedNeuriteClasses);
		return copy;
	}
	
	public static String readLine(InputStream bis) throws Exception{
		return readLine(bis, true);
	}
	public static String readLine(InputStream bis, boolean doubleLineBreak) throws IOException{
		String dataString = "";
		char nextChar = ' ';
		
		do {
			dataString = dataString + nextChar;
			nextChar = (char)bis.read();
		} while ((int)nextChar != 65535 && nextChar != '\n' && nextChar != '\r');
		if (doubleLineBreak && nextChar != -1){
			nextChar = (char)bis.read(); // Clear second line break
			//System.out.println("Should be line break: "+(byte)nextChar);
		}
		dataString = dataString.substring(1);

		if (nextChar == 65535 && dataString.isEmpty()){
			return null;
		}
		return dataString;
	}

	/** Read the given binary file, and return its contents as a byte array.*/ 
	public static byte[] readBinaryToByteArray(String fileName) throws FileNotFoundException, IOException{
	    return readBinaryToByteArray(new File(fileName));
	}
	public static byte[] readBinaryToByteArray(File file) throws FileNotFoundException, IOException{
		byte[] result = new byte[(int)file.length()];
	    try {
	    	InputStream input = null;
	    	try {
	    		int totalBytesRead = 0;
	    		input = new BufferedInputStream(new FileInputStream(file));
	    		while(totalBytesRead < result.length){
	    			int bytesRemaining = result.length - totalBytesRead;
	    			//input.read() returns -1, 0, or more :
	    			int bytesRead = input.read(result, totalBytesRead, bytesRemaining); 
	    			if (bytesRead > 0){
	    				totalBytesRead = totalBytesRead + bytesRead;
	    			}
	    		}
	    		/*
	         	the above style is a bit tricky: it places bytes into the 'result' array; 
	         	'result' is an output parameter;
	         	the while loop usually has a single iteration only.
	    		 */
	    	} 
	    	finally {
	    		input.close();
	    	}
	    }
	    catch (FileNotFoundException ex) {
	    	throw ex;
	    }
	    catch (IOException ex) {
	      	throw ex;
	    }
	    return result;
	  }
	
}
