package org.cn3.morpho.data;

import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Stack;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.SwcDataNode;
import org.cn3.morpho.domain.SwcTreeNode;

public class SWCWriter {

	public static SwcTreeNode convertBinaryTreeToSwcTree(BinaryTreeNode binaryRoot){
		Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
		BinaryTreeNode bNode;
		Map<BinaryTreeNode,SwcTreeNode> bToSMap = new HashMap<BinaryTreeNode,SwcTreeNode>();
		SwcTreeNode swcRoot = null, swcNode;
		stack.push(binaryRoot);
		while (!stack.isEmpty()){
			bNode = stack.pop();
			swcNode = new SwcTreeNode((SwcDataNode)bNode.getData());
			bToSMap.put(bNode, swcNode);
			if (swcRoot == null){
				swcRoot = swcNode;
			}
			if (bNode.hasParent()){
				swcNode.setParent(bToSMap.get(bNode.getParent()));
			}
		}
		return swcRoot;
	}
	public static String writeSwcTreeToString(SwcTreeNode tree) throws IOException{
		List<SwcTreeNode> trees = new ArrayList<SwcTreeNode>();
		trees.add(tree);
		return writeSwcTreesToString(trees);
	}
	public static void writeSwcTreeToFile(File file, SwcTreeNode tree) throws IOException{
		List<SwcTreeNode> trees = new ArrayList<SwcTreeNode>();
		trees.add(tree);
		writeSwcTreesToFile(file, trees);
	}
	public static void writeSwcTreesToFile(File file, List<SwcTreeNode> trees) throws IOException{
		DataWriter.writeStringToFile(file, writeSwcTreesToString(trees));
	}
	public static void writeSwcTreesToFile(File file, List<SwcTreeNode> trees, String headerComments) throws IOException{
		DataWriter.writeStringToFile(file, headerComments+writeSwcTreesToString(trees));
	}
	public static String writeSwcTreesToString(List<SwcTreeNode> trees) throws IOException{
		SwcTreeNode node, child;
		//LinkedStack<SwcTreeNode> stack = new LinkedStack<SwcTreeNode>();
		Queue<SwcTreeNode> queue = new LinkedList<SwcTreeNode>();
		int startId = 1;
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(3);
		nf.setGroupingUsed(false);
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < trees.size(); i++){
			queue.offer(trees.get(i));
			trees.get(i).setNodeId(startId++);
			//while (!stack.isEmpty()){
			while (!queue.isEmpty()){
				//node = stack.pop();
				node = queue.remove();
				sb.append(node.getNodeId()+" "+node.getType()+" ");
				sb.append(nf.format(node.getX())+" "+nf.format(node.getY())+" "+nf.format(node.getZ())+" ");
				sb.append(nf.format(node.getRadius())+" ");
				if (node.hasParent()){
					sb.append(node.getParentId()+"\n");
				}
				else{
					sb.append("-1\n");
				}
				for (int j = 0; j < node.getChildren().size(); j++){
					child = (SwcTreeNode)node.getChildren().get(j);
					//stack.push(child);
					queue.offer(child);
					child.setNodeId(startId++);
					child.setParentId(node.getNodeId());
				}
			}
		}
		return sb.toString();
	}
	
}
