package org.cn3.morpho.data.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cn3.morpho.domain.Measurements;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.domain.NeuronMetadataComplete;


public class NeuronDAO {

	private static NeuronDAO instance = null;
	private static final String key = "Neuron";

	private Connection conn;
	
	public static NeuronDAO getInstance() throws Exception{
		if (instance == null){
			instance = new NeuronDAO();
		}
        return instance;
	}
	
	private NeuronDAO() throws Exception{
		conn = getConnection();
	}
	
	private Connection getConnection() throws Exception{
		if (conn == null){
			conn = ConnectionFactory.getNeuronConnection(key);
		}
		return conn;
	}
	
	// Might cause problems with other links to this object
	public void destroy() throws Exception{
		ConnectionFactory.destroy(key);
	}
	
	public void resetConnection() throws Exception{
		ConnectionFactory.destroy(key);
		conn = ConnectionFactory.getConnection(key);
	}
	
	public void showTables() throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SHOW TABLES ");

	        s = conn.prepareStatement(sb.toString());
	        rs = s.executeQuery();
	        System.out.println("List of Tables");
	        while (rs.next()){
	            System.out.println(rs.getString(1));
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
	}
	
	public void describeTable(String table) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" DESCRIBE "+table);

	        s = conn.prepareStatement(sb.toString());
	        rs = s.executeQuery();
	        System.out.println("Table: "+table);
	        while (rs.next()){
	        	String col_name = rs.getString("field");
	            String data_type = rs.getString("TYPE");
	            String key = rs.getString("key");
	            String nullable = rs.getString("NULL");
	            System.out.println(col_name+"\t"+data_type+"\t"+key+"\t"+nullable);
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
	}
	
	public void displayTable(String table) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		
		try{
			conn = getConnection();

	        s = conn.prepareStatement("describe "+table);
	        rs = s.executeQuery();
	        System.out.println("Table: "+table);
	        int cols = 0;
	        while (rs.next()){
	        	String colName = rs.getString("field");
	        	System.out.print(colName+"\t");
	        	cols++;
        	}
	        System.out.println();
	        
	        s = conn.prepareStatement("select * from "+table);
	        rs = s.executeQuery();
	        while (rs.next()){
	        	for (int i = 1; i <= cols; i++){
	        		System.out.print(rs.getObject(i)+"\t");
	        	}
	            System.out.println();
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
	}

	public NeuronMetadata getNeuron(int id) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadata neuron = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_name, a.archive_name,        \n");
	        sb.append("       s.species_name, ct.class2, r.region1 \n");
	        sb.append(" FROM   neuron n, archive a, species s,     \n");
	        sb.append(" 	  neuron_class2 ct, neuron_region1 r   \n");
	        sb.append(" WHERE  neuron_id = ?     \n");
	        sb.append(" AND    n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.class2_id = ct.class2_id  \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");
	        s = conn.prepareStatement(sb.toString());
	        s.setInt(1, id);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	neuron = new NeuronMetadata();
	        	neuron.setId(id);
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setAnimal(rs.getString("species_name"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return neuron;
	}

	public NeuronMetadata getNeuron(String neuronName) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadata neuron = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id, a.archive_name,        \n");
	        sb.append("       s.species, ct.class2, r.region1      \n");
	        sb.append(" FROM   neuron n, archive a, species s,     \n");
	        sb.append(" 	  neuron_class2 ct, neuron_region1 r   \n");
	        sb.append(" WHERE  neuron_name = ?     \n");
	        sb.append(" AND    n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.class2_id = ct.class2_id  \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");
	        s = conn.prepareStatement(sb.toString());
	        s.setString(1, neuronName);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	neuron = new NeuronMetadata();
	        	neuron.setId(rs.getInt("neuron_id"));
	        	neuron.setName(neuronName);
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return neuron;
	}

	public List<NeuronMetadata> getNeurons() throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadata neuron = null;
		List<NeuronMetadata> list = null;
		//System.out.println("Getting neurons");
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id, a.archive_name, n.neuron_name, \n");
	        sb.append("       s.species, ct.class2, r.region1      \n");
	        sb.append(" FROM   neuron n, archive a, species s,     \n");
	        sb.append(" 	  neuron_class2 ct, neuron_region1 r   \n");
	        sb.append(" WHERE  n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.class2_id = ct.class2_id  \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");
	        s = conn.prepareStatement(sb.toString());
	        
	        rs = s.executeQuery();
	        list = new ArrayList<NeuronMetadata>();
	        while (rs.next()){
	        	neuron = new NeuronMetadata();
	        	neuron.setId(rs.getInt("neuron_id"));
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));
	        	list.add(neuron);
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return list;
	}

	public static final String METATYPE_ARCHIVE = "Archive";
	public static final String METATYPE_CLASS1 = "Class1";
	public static final String METATYPE_CELL_TYPE = "CellType";
	public static final String METATYPE_CLASS2 = "Class2";
	public static final String METATYPE_CLASS3 = "Class3";
	public static final String METATYPE_CLASS3B = "Class3B";
	public static final String METATYPE_CLASS3C = "Class3C";
	public static final String METATYPE_BRAIN_REGION = "BrainRegion";
	public static final String METATYPE_REGION1 = "Region1";
	public static final String METATYPE_REGION2 = "Region2";
	public static final String METATYPE_REGION3 = "Region3"; 
	public static final String METATYPE_SPECIES = "Species";
	public static final String METATYPE_STRAIN = "strain";
	public static final String METATYPE_AGE = "Age";
	public static final String METATYPE_STAINING = "Staining";
	public static final String METATYPE_PROTOCOL = "Protocol";
	public static final String METATYPE_SLICE_THICKNESS = "SliceThickness";
	public static final String METATYPE_SLICE_DIRECTION = "SliceDirection"; 
	public static final String METATYPE_OBJECTIVE_TYPE = "ObjectiveType";
	public static final String METATYPE_MAGNIFICATION = "Magnification";
	public static final String METATYPE_CONDITION = "ExperimentalCondition";

	
	private static Map<String,String> metatypeConvert = null;
	private static Map<String,String> getMetatypeConvert(){
		if (metatypeConvert == null){
			metatypeConvert = new HashMap<String,String>();
			metatypeConvert.put(METATYPE_ARCHIVE, "a.archive_name");
			metatypeConvert.put(METATYPE_CLASS1, "c1.class1");
			metatypeConvert.put(METATYPE_CELL_TYPE, "c2.class2");
			metatypeConvert.put(METATYPE_CLASS2, "c2.class2");
			metatypeConvert.put(METATYPE_CLASS3, "c3.class3");
			metatypeConvert.put(METATYPE_CLASS3B, "c3b.class3");
			metatypeConvert.put(METATYPE_CLASS3C, "c3c.class3");
			metatypeConvert.put(METATYPE_REGION1, "r.region1");
			metatypeConvert.put(METATYPE_BRAIN_REGION, "r.region1");
			metatypeConvert.put(METATYPE_REGION2, "r2.region2");
			metatypeConvert.put(METATYPE_REGION3, "r3.region3");
			metatypeConvert.put(METATYPE_SPECIES, "s.species");
			metatypeConvert.put(METATYPE_STRAIN, "ans.strain_name");
			metatypeConvert.put(METATYPE_AGE, "ac.age_class");
			metatypeConvert.put(METATYPE_STAINING, "sm.stain");
			metatypeConvert.put(METATYPE_PROTOCOL, "pd.protocol");
			metatypeConvert.put(METATYPE_SLICE_THICKNESS, "st.slice_thickness");
			metatypeConvert.put(METATYPE_SLICE_DIRECTION, "sd.slicing_direction");
			metatypeConvert.put(METATYPE_OBJECTIVE_TYPE, "ot.objective_type");
			metatypeConvert.put(METATYPE_MAGNIFICATION, "mag.magnification");
			metatypeConvert.put(METATYPE_CONDITION, "expc.expercond");
		}
		return metatypeConvert;
	}

	private static String getMetatypeTestString(String metatype, List<String> values){
		Map<String,String> metatypeConvert = getMetatypeConvert();
		if (metatypeConvert.containsKey(metatype)){
			StringBuffer sb = new StringBuffer();
			sb.append(" AND ( ");
			for (int i = 0; i < values.size(); i++){
				if (i > 0){
					sb.append(" OR ");
				}
				sb.append(" "+metatypeConvert.get(metatype)+" = '"+values.get(i)+"'");
			}
			sb.append(" ) ");
			
			return sb.toString();
		}
		else{
			return "";
		}
	}
	
	/**
	 * 
	 * @param Metadata map of field and allowable values
	 * @return List of NeuronMetadata objects satisfying metadata conditions
	 * @throws Exception
	 */
	public List<NeuronMetadata> getNeuronsByMetadata(Map<String,List<String>> metadata) 
	throws Exception{
		return getNeuronsByMetadata(metadata, false);
	}
	public List<NeuronMetadata> getNeuronsByMetadata(Map<String,List<String>> metadata, boolean printStatement) 
	throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadataComplete neuron = null;
		List<NeuronMetadata> list = null;

		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id, a.archive_name, n.neuron_name, \n");
	        sb.append("       s.species, ct.class2, r.region1      \n");
	        sb.append(" FROM   archive a, species s,     \n");
	        sb.append(" 	  neuron_class2 ct, neuron_region1 r, neuron n   \n");
        	sb.append(" LEFT JOIN neuron_class1 c1 ON n.class1_id = c1.class1_id    \n");
        	sb.append(" LEFT JOIN neuron_class2 c2 ON n.class2_id = c2.class2_id    \n");
        	sb.append(" LEFT JOIN neuron_class3 c3 ON n.class3_id = c3.class3_id    \n");
        	sb.append(" LEFT JOIN neuron_class3 c3b ON n.class3_idB = c3b.class3_id    \n");
        	sb.append(" LEFT JOIN neuron_class3 c3c ON n.class3_idC = c3c.class3_id    \n");
        	sb.append(" LEFT JOIN neuron_region2 r2 ON n.region2_id = r2.region2_id \n");
        	sb.append(" LEFT JOIN neuron_region3 r3 ON n.region3_id = r3.region3_id \n");
        	sb.append(" LEFT JOIN animal_strain ans ON n.strain_id = ans.strain_id  \n");
        	sb.append(" LEFT JOIN age_classification ac ON n.age_classification_id = ac.age_class_id   \n");
        	sb.append(" LEFT JOIN staining_method sm ON n.stain_id = sm.stain_id    \n");
        	sb.append(" LEFT JOIN slicing_thickness st ON n.thickness_id = st.thickness_id \n");
        	sb.append(" LEFT JOIN slicing_direction sd ON n.slice_direction_id = sd.direction_id \n");
        	sb.append(" LEFT JOIN protocol_design pd ON n.protocol_id = pd.protocol_id     \n");
        	sb.append(" LEFT JOIN objective_type ot ON n.objective_id = ot.objective_id    \n");
        	sb.append(" LEFT JOIN magnification mag ON n.magnification_id = mag.magnification_id       \n");
        	sb.append(" LEFT JOIN experimentcondition expc ON n.expercond_id = expc.expercond_id       \n");
        	
	        sb.append(" WHERE  n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.class2_id = ct.class2_id  \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");
	        
	        for (String key : metadata.keySet()){
	        	sb.append(getMetatypeTestString(key,metadata.get(key)));
	        }

	        if (printStatement) System.out.println(sb.toString());
	        
	        s = conn.prepareStatement(sb.toString());
	        
	        rs = s.executeQuery();
	        list = new ArrayList<NeuronMetadata>();
	        while (rs.next()){
	        	neuron = new NeuronMetadataComplete();
	        	neuron.setId(rs.getInt("neuron_id"));
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));
	        	list.add(neuron);
	        }

	        //System.out.println(list.size());
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return list;
	}

	public NeuronMetadataComplete getNeuronInfo(int id) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadataComplete neuron = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_name, a.archive_name, s.species,     \n");
	        sb.append("       c2.class2, r.region1, strain_name, min_age,  \n");
	        sb.append("       max_age, age_scale, min_weight, max_weight,  \n"); 
    		sb.append("       slice_thickness, age_class, gender, region1, \n");
    		sb.append("       region2, region3, class1, class2, c3.class3,    \n");
    		sb.append("       c3b.class3 as class3b, c3c.class3 as class3c,    \n");
    		sb.append("       protocol, slicing_direction, stain,          \n");
    		sb.append("       magnification, objective_type, expercond \n");
	    	
	        sb.append(" FROM  archive a, species s, neuron_region1 r,      \n");
	        sb.append("       neuron n \n");
	        sb.append(" LEFT JOIN neuron_class1 c1 ON n.class1_id = c1.class1_id    \n");
	        sb.append(" LEFT JOIN neuron_class2 c2 ON n.class2_id = c2.class2_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3 ON n.class3_id = c3.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3b ON n.class3_idB = c3b.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3c ON n.class3_idC = c3c.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_region2 r2 ON n.region2_id = r2.region2_id \n");
	        sb.append(" LEFT JOIN neuron_region3 r3 ON n.region3_id = r3.region3_id \n");
	        sb.append(" LEFT JOIN animal_strain ans ON n.strain_id = ans.strain_id  \n");
	        sb.append(" LEFT JOIN age_classification ac ON n.age_classification_id = ac.age_class_id   \n");
	        sb.append(" LEFT JOIN staining_method sm ON n.stain_id = sm.stain_id    \n");
	        sb.append(" LEFT JOIN slicing_thickness st ON n.thickness_id = st.thickness_id \n");
	        sb.append(" LEFT JOIN slicing_direction sd ON n.slice_direction_id = sd.direction_id \n");
	        sb.append(" LEFT JOIN protocol_design pd ON n.protocol_id = pd.protocol_id     \n");
	        sb.append(" LEFT JOIN objective_type ot ON n.objective_id = ot.objective_id    \n");
	        sb.append(" LEFT JOIN magnification mag ON n.magnification_id = mag.magnification_id       \n");
	        sb.append(" LEFT JOIN experimentcondition expc ON n.expercond_id = expc.expercond_id       \n");
	        
	        sb.append(" WHERE  neuron_id = ?     \n");
	        sb.append(" AND    n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");

	        s = conn.prepareStatement(sb.toString());
	        s.setInt(1, id);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	neuron = new NeuronMetadataComplete();
	        	neuron.setId(id);
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setId(id);
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));

	        	neuron.setAgeClassification(rs.getString("age_class"));
	        	neuron.setExperimentCondition(rs.getString("expercond"));
	        	neuron.setAgeScale(rs.getString("age_scale"));
	        	neuron.setClass1(rs.getString("class1"));
	        	neuron.setClass3(rs.getString("class3"));
	        	neuron.setClass3B(rs.getString("class3b"));
	        	neuron.setClass3C(rs.getString("class3c"));
	        	neuron.setGender(rs.getString("gender"));
	        	neuron.setMaxAge(rs.getInt("max_age"));
	        	neuron.setMaxWeight(rs.getInt("max_weight"));
	        	neuron.setMinAge(rs.getInt("min_age"));
	        	neuron.setMinWeight(rs.getInt("min_weight"));
	        	neuron.setObjectiveType(rs.getString("objective_type"));
	        	neuron.setProtocol(rs.getString("protocol"));
	        	neuron.setRegion2(rs.getString("region2"));
	        	neuron.setRegion3(rs.getString("region3"));
	        	neuron.setStain(rs.getString("stain"));
	        	neuron.setStrain(rs.getString("strain_name"));
	        	neuron.setThickness(rs.getString("slice_thickness"));
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return neuron;
	}

	public NeuronMetadataComplete getNeuronInfo(String neuronName) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadataComplete neuron = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id, a.archive_name, s.species,     \n");
	        sb.append("       c2.class2, r.region1, strain_name, min_age,  \n");
	        sb.append("       max_age, age_scale, min_weight, max_weight,  \n"); 
    		sb.append("       slice_thickness, age_class, gender,          \n");
    		sb.append("       region2, region3, class1, class2, c3.class3,  \n");
    		sb.append("       c3b.class3 as class3b, c3c.class3 as class3c, \n");
    		sb.append("       protocol, slicing_direction, stain,          \n");
    		sb.append("       magnification, objective_type, expercond \n");
	    	
	        sb.append(" FROM  archive a, species s, neuron_region1 r,      \n");
	        sb.append("       neuron n \n");
	        sb.append(" LEFT JOIN neuron_class1 c1 ON n.class1_id = c1.class1_id    \n");
	        sb.append(" LEFT JOIN neuron_class2 c2 ON n.class2_id = c2.class2_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3 ON n.class3_id = c3.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3b ON n.class3_idB = c3b.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3c ON n.class3_idC = c3c.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_region2 r2 ON n.region2_id = r2.region2_id \n");
	        sb.append(" LEFT JOIN neuron_region3 r3 ON n.region3_id = r3.region3_id \n");
	        sb.append(" LEFT JOIN animal_strain ans ON n.strain_id = ans.strain_id  \n");
	        sb.append(" LEFT JOIN age_classification ac ON n.age_classification_id = ac.age_class_id   \n");
	        sb.append(" LEFT JOIN staining_method sm ON n.stain_id = sm.stain_id    \n");
	        sb.append(" LEFT JOIN slicing_thickness st ON n.thickness_id = st.thickness_id \n");
	        sb.append(" LEFT JOIN slicing_direction sd ON n.slice_direction_id = sd.direction_id \n");
	        sb.append(" LEFT JOIN protocol_design pd ON n.protocol_id = pd.protocol_id     \n");
	        sb.append(" LEFT JOIN objective_type ot ON n.objective_id = ot.objective_id    \n");
	        sb.append(" LEFT JOIN magnification mag ON n.magnification_id = mag.magnification_id       \n");
	        sb.append(" LEFT JOIN experimentcondition expc ON n.expercond_id = expc.expercond_id       \n");
	        
	        sb.append(" WHERE  neuron_name = ?     \n");
	        sb.append(" AND    n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");

	        s = conn.prepareStatement(sb.toString());
	        s.setString(1, neuronName);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	neuron = new NeuronMetadataComplete();
	        	neuron.setId(rs.getInt("neuron_id"));
	        	neuron.setName(neuronName);
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));

	        	neuron.setAgeClassification(rs.getString("age_class"));
	        	neuron.setExperimentCondition(rs.getString("expercond"));
	        	neuron.setAgeScale(rs.getString("age_scale"));
	        	neuron.setClass1(rs.getString("class1"));
	        	neuron.setClass3(rs.getString("class3"));
	        	neuron.setClass3B(rs.getString("class3b"));
	        	neuron.setClass3C(rs.getString("class3c"));
	        	neuron.setGender(rs.getString("gender"));
	        	neuron.setMaxAge(rs.getInt("max_age"));
	        	neuron.setMaxWeight(rs.getInt("max_weight"));
	        	neuron.setMinAge(rs.getInt("min_age"));
	        	neuron.setMinWeight(rs.getInt("min_weight"));
	        	neuron.setObjectiveType(rs.getString("objective_type"));
	        	neuron.setProtocol(rs.getString("protocol"));
	        	neuron.setRegion2(rs.getString("region2"));
	        	neuron.setRegion3(rs.getString("region3"));
	        	neuron.setStain(rs.getString("stain"));
	        	neuron.setStrain(rs.getString("strain_name"));
	        	neuron.setThickness(rs.getString("slice_thickness"));
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return neuron;
	}
	
	public List<NeuronMetadataComplete> getNeuronInfos() throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		NeuronMetadataComplete neuron = null;
		List<NeuronMetadataComplete> list = null;

		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id, neuron_name, a.archive_name, s.species, \n");
	        sb.append("       c2.class2, r.region1, strain_name, min_age,   \n");
	        sb.append("       max_age, age_scale, min_weight, max_weight,   \n"); 
    		sb.append("       slice_thickness, age_class, gender,           \n");
    		sb.append("       region2, region3, class1, class2, c3.class3,  \n");
    		sb.append("       c3b.class3 as class3b, c3c.class3 as class3c, \n");
    		sb.append("       protocol, slicing_direction, stain,           \n");
    		sb.append("       magnification, objective_type, expercond \n");
	    	
	        sb.append(" FROM  archive a, species s, neuron_region1 r,      \n");
	        sb.append("       neuron n \n");
	        sb.append(" LEFT JOIN neuron_class1 c1 ON n.class1_id = c1.class1_id    \n");
	        sb.append(" LEFT JOIN neuron_class2 c2 ON n.class2_id = c2.class2_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3 ON n.class3_id = c3.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3b ON n.class3_idB = c3b.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3c ON n.class3_idC = c3c.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_region2 r2 ON n.region2_id = r2.region2_id \n");
	        sb.append(" LEFT JOIN neuron_region3 r3 ON n.region3_id = r3.region3_id \n");
	        sb.append(" LEFT JOIN animal_strain ans ON n.strain_id = ans.strain_id  \n");
	        sb.append(" LEFT JOIN age_classification ac ON n.age_classification_id = ac.age_class_id   \n");
	        sb.append(" LEFT JOIN staining_method sm ON n.stain_id = sm.stain_id    \n");
	        sb.append(" LEFT JOIN slicing_thickness st ON n.thickness_id = st.thickness_id \n");
	        sb.append(" LEFT JOIN slicing_direction sd ON n.slice_direction_id = sd.direction_id \n");
	        sb.append(" LEFT JOIN protocol_design pd ON n.protocol_id = pd.protocol_id     \n");
	        sb.append(" LEFT JOIN objective_type ot ON n.objective_id = ot.objective_id    \n");
	        sb.append(" LEFT JOIN magnification mag ON n.magnification_id = mag.magnification_id       \n");
	        sb.append(" LEFT JOIN experimentcondition expc ON n.expercond_id = expc.expercond_id       \n");
	        
	        sb.append(" WHERE  n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");

	        s = conn.prepareStatement(sb.toString());
	        
	        rs = s.executeQuery();
	        list = new ArrayList<NeuronMetadataComplete>();
	        while (rs.next()){
	        	neuron = new NeuronMetadataComplete();
	        	neuron.setId(rs.getInt("neuron_id"));
	        	neuron.setName(rs.getString("neuron_name"));
	        	neuron.setAnimal(rs.getString("species"));
	        	neuron.setLabName(rs.getString("archive_name"));
	        	neuron.setBrainRegion(rs.getString("region1"));
	        	neuron.setCellType(rs.getString("class2"));

	        	neuron.setAgeClassification(rs.getString("age_class"));
	        	neuron.setExperimentCondition(rs.getString("expercond"));
	        	neuron.setAgeScale(rs.getString("age_scale"));
	        	neuron.setClass1(rs.getString("class1"));
	        	neuron.setClass3(rs.getString("class3"));
	        	neuron.setClass3B(rs.getString("class3b"));
	        	neuron.setClass3C(rs.getString("class3c"));
	        	neuron.setGender(rs.getString("gender"));
	        	neuron.setMaxAge(rs.getInt("max_age"));
	        	neuron.setMaxWeight(rs.getInt("max_weight"));
	        	neuron.setMinAge(rs.getInt("min_age"));
	        	neuron.setMinWeight(rs.getInt("min_weight"));
	        	neuron.setObjectiveType(rs.getString("objective_type"));
	        	neuron.setProtocol(rs.getString("protocol"));
	        	neuron.setRegion2(rs.getString("region2"));
	        	neuron.setRegion3(rs.getString("region3"));
	        	neuron.setStain(rs.getString("stain"));
	        	neuron.setStrain(rs.getString("strain_name"));
	        	neuron.setThickness(rs.getString("slice_thickness"));
	        	list.add(neuron);
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return list;
	}
	
	private String[] morphometricTables = {"", "measurementsAX", "measurementsBS", "measurementsBS", "measurementsAP"}; 
	/**
	 * @param neuronName
	 * @return Neuron object with 
	 * @throws Exception
	 */
	public Measurements getNeuronMorphometrics(String neuronName, int arborTypeId) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		Measurements measurements = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT m.*                       \n");
	        sb.append(" FROM   "+morphometricTables[arborTypeId]+" m, neuron n  \n");
	        sb.append(" WHERE  n.neuron_name = ?         \n");
	        sb.append(" AND    n.neuron_id = m.neuron_id \n");
	        s = conn.prepareStatement(sb.toString());
	        s.setString(1, neuronName);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	measurements = new Measurements();
	        	measurements.setArborTypeId(arborTypeId);
	        	measurements.setNeuronName(neuronName);
	        	measurements.setSomaSurface(rs.getDouble("Soma_Surface"));
	        	measurements.setNStems(rs.getInt("N_stems"));
	        	measurements.setNBifs(rs.getInt("N_bifs"));
	        	measurements.setNBranches(rs.getInt("N_branch"));
	        	measurements.setWidth(rs.getDouble("Width"));
	        	measurements.setHeight(rs.getDouble("Height"));
	        	measurements.setDepth(rs.getDouble("Depth"));
	        	measurements.setDiameter(rs.getDouble("Diameter"));
	        	measurements.setLength(rs.getDouble("Length"));
	        	measurements.setSurface(rs.getDouble("Surface"));
	        	measurements.setVolume(rs.getDouble("Volume"));
	        	measurements.setEuclideanDistance(rs.getDouble("EucDistance"));
	        	measurements.setPathDistance(rs.getDouble("PathDistance"));
	        	measurements.setBranchOrder(rs.getDouble("Branch_Order"));
	        	measurements.setContraction(rs.getDouble("Contraction"));
	        	measurements.setFragmentation(rs.getDouble("Fragmentation"));
	        	measurements.setPartitionAsymmetry(rs.getDouble("Partition_asymmetry"));
	        	measurements.setPkClassic(rs.getDouble("Pk_classic"));
	        	measurements.setBifAmplLocal(rs.getDouble("bif_ampl_local"));
	        	measurements.setBifAmplRemote(rs.getDouble("bif_ampl_remote"));
	        	measurements.setFractalDim(rs.getDouble("Fractal_Dim"));
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return measurements;
	}
	/**
	 * @param arborTypeId
	 * @return
	 * @throws Exception
	 */
	public Map<String,Measurements> getNeuronMorphometricsMap() throws Exception{
		Map<String,Measurements> map = new HashMap<String,Measurements>();
		int[] arborTypeIds = {1,2,3,4};
		int arborTypeId;
		Measurements measurements;
		List<Measurements> list;
		String arborTypeString;
		for (int i = 0; i < arborTypeIds.length; i++){
			arborTypeId = arborTypeIds[i];
			arborTypeString = "-"+NeuronMetadata.ARBOR_TYPE_STRINGS[arborTypeId];
			//System.out.println(arborTypeString);
			list = getAllNeuronMorphometrics(arborTypeId);
			for (int j = 0; j < list.size(); j++){
				measurements = (Measurements)list.get(j);
				map.put(measurements.getNeuronName()+arborTypeString,measurements);
				//System.out.println("Put for "+measurements.getNeuronName()+arborTypeString);
			}
			
		}
		return map;
	}
	/**
	 * @param arborTypeId
	 * @return
	 * @throws Exception
	 */
	public Map<String,Measurements> getNeuronMorphometricsMap(int arborTypeId) throws Exception{
		List<Measurements> list = getAllNeuronMorphometrics(arborTypeId);
		Map<String,Measurements> map = new HashMap<String,Measurements>();
		Measurements measurements;
		for (int i = 0; i < list.size(); i++){
			measurements = (Measurements)list.get(i);
			map.put(measurements.getNeuronName(),measurements);
		}
		return map;
	}
	/**
	 * @param neuronName
	 * @param arborTypeId
	 * @return
	 * @throws Exception
	 */
	public List<Measurements> getAllNeuronMorphometrics(int arborTypeId) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		List<Measurements> list = null;
		Measurements measurements = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT m.*                       \n");
	        sb.append(" FROM   "+morphometricTables[arborTypeId]+" m \n");
	        s = conn.prepareStatement(sb.toString());
	        rs = s.executeQuery();
	        list = new ArrayList<Measurements>();
	        while (rs.next()){
	        	measurements = new Measurements();
	        	list.add(measurements);
	        	measurements.setArborTypeId(arborTypeId);
	        	measurements.setNeuronName(rs.getString("Neuron_name"));
	        	measurements.setSomaSurface(rs.getDouble("Soma_Surface"));
	        	measurements.setNStems(rs.getInt("N_stems"));
	        	measurements.setNBifs(rs.getInt("N_bifs"));
	        	measurements.setNBranches(rs.getInt("N_branch"));
	        	measurements.setWidth(rs.getDouble("Width"));
	        	measurements.setHeight(rs.getDouble("Height"));
	        	measurements.setDepth(rs.getDouble("Depth"));
	        	measurements.setDiameter(rs.getDouble("Diameter"));
	        	measurements.setLength(rs.getDouble("Length"));
	        	measurements.setSurface(rs.getDouble("Surface"));
	        	measurements.setVolume(rs.getDouble("Volume"));
	        	measurements.setEuclideanDistance(rs.getDouble("EucDistance"));
	        	measurements.setPathDistance(rs.getDouble("PathDistance"));
	        	measurements.setBranchOrder(rs.getDouble("Branch_Order"));
	        	measurements.setContraction(rs.getDouble("Contraction"));
	        	measurements.setFragmentation(rs.getDouble("Fragmentation"));
	        	measurements.setPartitionAsymmetry(rs.getDouble("Partition_asymmetry"));
	        	measurements.setPkClassic(rs.getDouble("Pk_classic"));
	        	measurements.setBifAmplLocal(rs.getDouble("bif_ampl_local"));
	        	measurements.setBifAmplRemote(rs.getDouble("bif_ampl_remote"));
	        	measurements.setFractalDim(rs.getDouble("Fractal_Dim"));
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return list;
	}

	private void getTotalBifs() throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT sum(N_bifs), sum(N_stems), sum(N_branch)  \n");
	        sb.append(" FROM   measurementsAPB     \n");
	        //sb.append(" WHERE  neuron_name = ?     \n");
	        s = conn.prepareStatement(sb.toString());
	        //s.setString(1, neuronName);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	System.out.println(rs.getInt(1));
	        	System.out.println(rs.getInt(2));
	        	System.out.println(rs.getInt(3));
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
	}
	
	public String getNeuronIdFromName(String neuronName) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		String id = "";
		
		try{
			conn = getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_id \n");
	    	
	        sb.append(" FROM  neuron n \n");
	        sb.append(" WHERE  neuron_name = ?     \n");

	        s = conn.prepareStatement(sb.toString());
	        s.setString(1, neuronName);
	        rs = s.executeQuery();
	        if (rs.next()){
	        	id = rs.getString("neuron_id");
	        }

		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
		return id;
	}
	
	public static void main(String args[]){
		NeuronDAO neuronDAO = null;
		try{
			neuronDAO = NeuronDAO.getInstance();
	
			String[] names = {"MJ042605-1", "C290500C-I4", "BE12D", "C180298A-I4"};
			String ids = "";
			
			for (String name : names){
				ids += ", "+neuronDAO.getNeuronIdFromName(name);
			}
			System.out.println(ids);
			System.exit(0);
			
			Map<String,List<String>> classDef = new HashMap<String,List<String>>();
			classDef.put("Class2", Arrays.asList("Pyramidal cell"));
			classDef.put("BrainRegion", Arrays.asList("Neocortex"));
			List<NeuronMetadata> neurons = neuronDAO.getNeuronsByMetadata(classDef, true);
			System.out.println(neurons.size()+" results");
//			neuronDAO.showTables();
/*
			neuronDAO.describeTable("celltype_table");
			neuronDAO.describeTable("neuron");
			neuronDAO.describeTable("neuron_class3");
			neuronDAO.describeTable("neuron_multiple");
			neuronDAO.describeTable("neuron_intermed");
*/
			
			//neuronDAO.describeTable("neuron");
			
			/*
			neuronDAO.displayTable("species");
			neuronDAO.displayTable("animal_strain");
			neuronDAO.displayTable("age_classification");
			neuronDAO.displayTable("archive");
			neuronDAO.displayTable("original_format");
			neuronDAO.displayTable("experimentcondition");
			neuronDAO.displayTable("neuron_region1");
			neuronDAO.displayTable("neuron_region2");
			neuronDAO.displayTable("neuron_region3");
			neuronDAO.displayTable("neuron_class1");
			neuronDAO.displayTable("neuron_class2");
			neuronDAO.displayTable("neuron_class3");
			neuronDAO.displayTable("reconstruction");
			neuronDAO.displayTable("slicing_direction");
			neuronDAO.displayTable("slicing_thickness");
			neuronDAO.displayTable("objective_type");
			neuronDAO.displayTable("protocol_design");
			neuronDAO.displayTable("staining_method");
			neuronDAO.displayTable("magnification");
			System.out.println();
			*/

			/*
			neuronDAO.describeTable("slicing_direction");
			System.out.println();
			neuronDAO.displayTable("reconstruction");
			System.out.println();
			neuronDAO.displayTable("animal_strain");
			System.out.println();
			neuronDAO.displayTable("age_classification");
	*/		
			
/*
			neuronDAO.describeTable("measurementsAP");
			System.out.println();
				*/
		
			//neuronDAO.describeTable("experimentcondition");
			System.out.println();
	/*
	 		NeuronInfo neuron = neuronDAO.getNeuronInfo("C010398B-p3");
			//NeuronInfo neuron = neuronDAO.getNeuronInfo(242);
			System.out.println("NeuronId: "+neuron.getId());
			System.out.println("Name: "+neuron.getName());
			System.out.println("Species: "+neuron.getAnimal());
			System.out.println("Region: "+neuron.getBrainRegion());
			System.out.println("Region3: "+neuron.getRegion3());
			
	*/		
			
			//neuronDAO.getTotalBifs();
			//neuronDAO.describeTable("measurementsAPB");
			//System.out.println(neuronDAO.getNeuronInfo("01-16-09-LatCIV"));
		} catch (Exception e){
			e.printStackTrace();
		} finally{
			try{
				if (neuronDAO != null){
					neuronDAO.destroy();
				}
			} catch (Exception e){
				e.printStackTrace();
			}
		}
		
	}
	
	private static void showDANeuronMetadata(NeuronDAO neuronDAO) throws Exception{
		Connection conn;
		ResultSet rs = null;
		PreparedStatement s = null;
		
		try{
			conn = neuronDAO.getConnection();
	        StringBuffer sb = new StringBuffer();
	        sb.append(" SELECT neuron_name, s.species,     \n");
	        sb.append("       c2.class2, r.region1,  \n");
    		sb.append("       class1, class2, c3.class3,   \n");
    		sb.append("       c3b.class3 as class3b, c3c.class3 as class3c   \n");
	    	
	        sb.append(" FROM  archive a, species s, neuron_region1 r,      \n");
	        sb.append("       neuron n \n");
	        sb.append(" LEFT JOIN neuron_class1 c1 ON n.class1_id = c1.class1_id    \n");
	        sb.append(" LEFT JOIN neuron_class2 c2 ON n.class2_id = c2.class2_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3 ON n.class3_id = c3.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3b ON n.class3_idB = c3b.class3_id    \n");
	        sb.append(" LEFT JOIN neuron_class3 c3c ON n.class3_idC = c3c.class3_id    \n");
	        
	        sb.append(" WHERE  a.archive_name = 'Cox'     \n");
	        sb.append(" AND    n.archive_id = a.archive_id \n");
	        sb.append(" AND    n.species_id = s.species_id \n");
	        sb.append(" AND    n.region1_id = r.region1_id \n");
	        
	        s = conn.prepareStatement(sb.toString());
	        rs = s.executeQuery();

	        while (rs.next()){
	        	System.out.println(rs.getString("neuron_name")+"\t"+rs.getString("class3")+"\t"+rs.getString("class3b")+"\t"+rs.getString("class3c"));
	        }
		} catch (SQLException se){
			throw new Exception(se);
		} finally{
			if (rs != null){
				rs.close();
				rs = null;
			}
			if (s != null){
				s.close();
				s = null;
			}
		}
	}

	
}
