package org.cn3.morpho.resources;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.morpho.utils.StringUtils;


public class MorphoDataProperties {

	private static MorphoDataProperties instance = null;
	private static Properties dataProperties = null;

	public static MorphoDataProperties getInstance() throws IOException{
		if (instance == null){
			instance = new MorphoDataProperties();
		}
		return instance;
	}
	
	private MorphoDataProperties() throws IOException{
		dataProperties = new Properties();
		dataProperties.load(new FileInputStream("resources/conf/data.properties"));
	}
	
	public Properties getDataProperties() {
		return dataProperties;
	}
	
	public String getWorkingDir() throws IOException{
		return MorphoProjectProperties.getInstance().getWorkingDir();
	}
	public String getResourceDirectory() throws IOException{
		return MorphoProjectProperties.getInstance().getResourceDirectory();
	}
	
	public String getMetaDataFilename() throws IOException{
		return getResourceDirectory()+dataProperties.getProperty(MorphoConstants.META_DATABASE_FILENAME);
	}

	public String getSwcDir(String dataset) throws IOException{
		return dataProperties.getProperty(MorphoConstants.SWC_LOCATION)+"/"+dataset+"/";
	}
	
	public String getResultsDir() throws IOException{
		return dataProperties.getProperty(MorphoConstants.RESULTS_LOCATION);
	}
	public String getResultsLookupDir() throws IOException{
		return dataProperties.getProperty(MorphoConstants.RESULTS_LOOKUP_LOCATION);
	}
	public String getFlaggedNeuronsFile() throws IOException{
		return dataProperties.getProperty(MorphoConstants.FLAGGED_NEURONS_FILE);
	}

	public double getMedianLengthToBranch() throws IOException{
		return Double.parseDouble(
				dataProperties.getProperty(MorphoConstants.GEOMETRY_MEDIAN_LENGTH));
	}

	public double getMedianBranchAngle() throws IOException{
		return Double.parseDouble(
				dataProperties.getProperty(MorphoConstants.GEOMETRY_MEDIAN_ANGLE));
	}

	public String getWellRepresentedNeuriteClassesFilename() throws IOException{
		return "resources/WellRepresentedNeuriteClasses.txt";
	}

	public int getDatasetTypeId(String dataset){
		return Integer.parseInt(dataProperties.getProperty("data.datasetTypeId."+dataset));
	}
	
	/**
	 * @param queryName
	 * @return File if it exists (using default dataset), otherwise null
	 * @throws Exception
	 */
	public File getSwcFileByName(String queryName) throws Exception{
		return getSwcFileByName(MorphoConstants.DEFAULT_DATASET, queryName);
	}
	/**
	 * @param dataset
	 * @param queryName
	 * @return File if it exists, otherwise null
	 * @throws Exception
	 */
	public File getSwcFileByName(String dataset, String queryName) throws Exception{
		String neuronName = StringUtils.getNeuronNameFromSequence(queryName);
		if (neuronName == null) neuronName = queryName;
		String archive = ClassificationUtils.getArchiveByNeuron(neuronName);
		String fileBase = getSwcDir(dataset)+"/"+archive+"/CNG version/";
		File file = new File(fileBase);
		if (!file.exists()){
			fileBase = getSwcDir(dataset)+"/"+archive;
			file = new File(fileBase);
			if (!file.exists()){
				return null;
			}
		}
		String filename = fileBase + neuronName + ".CNG.swc";
		file = new File(filename);
		if (!file.exists()){
			filename = fileBase + neuronName + ".swc";
			file = new File(filename);
			if (!file.exists()){
				return null;
			}
		}
		return file;
	}

}
