package org.cn3.morpho.scripts;

import java.io.File;

import org.cn3.morpho.domain.SwcDataNode;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Rotates the swc file given angles.
 * 
 * java org.cn3.morpho.scripts.SwcNeuronRotater -i "C:/Documents and Settings/gillette/My Documents/NMO/l51.CNG.swc" -o "C:/Documents and Settings/gillette/My Documents/NMO/l51-rotated.swc" -x .25 -y .9 -z .36 -a 2 --translateX 436 --translateY -1566 --translateZ 533.2
 * 
 */
public class SwcNeuronRotater extends SwcNeuronTransformer {

	private static final String PARAM_AXIS_X = "axisX";
	private static final String PARAM_AXIS_Y = "axisY";
	private static final String PARAM_AXIS_Z = "axisZ";
	private static final String PARAM_ANGLE = "angle";
	
	private static final String PARAM_TRANS_X = "translateX";
	private static final String PARAM_TRANS_Y = "translateY";
	private static final String PARAM_TRANS_Z = "translateZ";

	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String filename = config.getString("input");
			String outFilename = config.getString("output");
			
			double axisX = config.getDouble(PARAM_AXIS_X);
			double axisY = config.getDouble(PARAM_AXIS_Y);
			double axisZ = config.getDouble(PARAM_AXIS_Z);
			double angle = config.getDouble(PARAM_ANGLE);
			SwcDataNode axis = new SwcDataNode(axisX,axisY,axisZ);
			
			double translateX = config.getDouble(PARAM_TRANS_X);
			double translateY = config.getDouble(PARAM_TRANS_Y);
			double translateZ = config.getDouble(PARAM_TRANS_Z);
			SwcDataNode translation = new SwcDataNode(translateX,translateY,translateZ);
			System.out.println(translateX+ " "+translateY+" "+translateZ);
			
			File inputFile = new File(filename), outputFile = new File(outFilename);
			if (inputFile.isDirectory()){
				if (!outputFile.isDirectory()){
					outputFile = new File(inputFile.getAbsolutePath()+"/Rotated/");
					if (!outputFile.exists()){
						outputFile.mkdir();
					}
				}
				File[] files = inputFile.listFiles();
				for (int i = 0; i < files.length; i++){
					inputFile = files[i];
					if (!inputFile.isDirectory()&&
							(inputFile.getName().endsWith(".swc")||inputFile.getName().endsWith(".SWC"))){
						rotateAndWrite(inputFile, 
								new File(outputFile.getAbsolutePath()+"/"+inputFile.getName()), 
								axis, angle, translation);
					}
				}
			}
			else{
				rotateAndWrite(inputFile, outputFile, axis, angle, translation);
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption("input")
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('i') 
        .setLongFlag("input");
        jsap.registerParameter(opt);

        opt = new FlaggedOption("output")
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('o') 
        .setLongFlag("output");
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_AXIS_X)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setRequired(true)
        .setShortFlag('x') 
        .setLongFlag(PARAM_AXIS_X);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_AXIS_Y)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setRequired(true)
        .setShortFlag('y') 
        .setLongFlag(PARAM_AXIS_Y);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_AXIS_Z)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setRequired(true)
        .setShortFlag('z') 
        .setLongFlag(PARAM_AXIS_Z);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_ANGLE)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setRequired(true)
        .setShortFlag('a') 
        .setLongFlag(PARAM_ANGLE);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_TRANS_X)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setDefault("0")
        .setRequired(false)
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_TRANS_X);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_TRANS_Y)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setDefault("0")
        .setRequired(false)
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_TRANS_Y);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_TRANS_Z)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setDefault("0")
        .setRequired(false)
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_TRANS_Z);
	    jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + SwcNeuronRotater.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

	
}