package org.cn3.morpho.utils;

import java.util.ArrayList;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.resources.MorphoDataProperties;


public class ClassificationUtils {

	private static Pattern specificTypesP 
		= Pattern.compile(".*(Granule cell|Pyramidal cell|Stellate cell|Uniglomerular projection neuron|NotReported).*");
	private static Pattern interneuronP 
		= Pattern.compile(".*(Interneuron|Martinotti|Basket|Chandelier|Double bouquet).*");
	public static String reclassifyCellTypes(String cellType){
		Matcher m = specificTypesP.matcher(cellType);
		if (m.matches()){
			return m.group(1);
		}
		else{
			m = interneuronP.matcher(cellType);
			if (m.matches()){
				return "Interneuron";
			}
			else{
				return "Principal Cell";
			}
		}
	}
	
	private static Pattern dendriteP = Pattern.compile("(Dendrite|BasalDendrite)");
	public static String reclassifyArborizationTypes(String arborizationType){
		Matcher m = dendriteP.matcher(arborizationType);
		//System.out.println("BaseArbType: "+baseArborizationType);
		if (m.matches()){
			return "Dendrite";
		}
		return arborizationType;
	}
	
	public static String getLabNameBySequenceName(String sequenceName) throws Exception{
		return getArchiveBySequenceName(sequenceName);
	}
	public static String getLabNameByNeuron(String neuronName) throws Exception{
		return getArchiveByNeuron(neuronName);
	}
	public static String getArchiveBySequenceName(String sequenceName) throws Exception{
		return getArchiveByNeuron(StringUtils.getNeuronNameFromSequence(sequenceName));
	}
	public static String getArchiveByNeuron(String neuronName) throws Exception{
		//Map<String,NeuronMetadata> metaData = MetaDataManager.getCellMetaData();
		Map<String,NeuronMetadata> metaData = MetaDataManager.getCellMetaDataFromFile(MorphoDataProperties.getInstance().getMetaDataFilename());
		if (metaData.containsKey(neuronName)){
			return metaData.get(neuronName).getArchive();
		}
		return null;
	}
}
