package org.cn3.morpho.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.cn3.morpho.data.DataWriter;


public class SystemUtils {

	public static void executeCommands(String[] commands, File workingDirectory) throws Exception{
		executeCommands(commands,workingDirectory,false);
	}
	public static void executeCommands(String[] commands, File workingDirectory, boolean printOutput) throws Exception{
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < commands.length; i++){
			sb.append(commands[i]+"\r\n");
		}
		executeCommands(sb.toString(),workingDirectory,printOutput);
	}
	public static void executeCommands(String commands, File workingDirectory) throws Exception{
		executeCommands(commands,workingDirectory,false);
	}
	public static void executeCommands(String commands, File workingDirectory, boolean printOutput) throws Exception{
		// Create batch file so we use executeCommand less often
		String dir = "";
		if (workingDirectory != null){
			dir = workingDirectory.getAbsolutePath()+"/";
		}
		Date now = new Date();
		String batFileName = dir+"execFile"+now.getTime()+".bat";
		DataWriter.writeStringToFile(batFileName, commands);
		executeCommand(batFileName, workingDirectory, printOutput);
		File file = new File(batFileName);
		file.delete();
	}
	
	public static int executeCommand(String command) throws Exception{
		return executeCommand(command, null, false);
	}
	public static int executeCommand(String command, boolean printOutput) throws Exception{
		return executeCommand(command, null, printOutput);
	}
	public static int executeCommand(String command, File workingDirectory) throws Exception{
		return executeCommand(command, workingDirectory, false);
	}
	public static int executeCommand(String command, File workingDirectory, boolean printOutput)
	throws Exception{
		Process p = Runtime.getRuntime().exec(command,null,workingDirectory);
		return executeCommandSub(command, p, workingDirectory, printOutput);
	}
	public static int executeCommand(String[] command, File workingDirectory) throws Exception{
		return executeCommand(command, workingDirectory, false);
	}
	public static int executeCommand(String[] command, File workingDirectory, boolean printOutput)
	throws Exception{
		Process p = Runtime.getRuntime().exec(command,null,workingDirectory);
		String concattedCommand = command[0];
		for (int i = 1; i < command.length; i++){
			concattedCommand += " "+command[i];
		}
		return executeCommandSub(concattedCommand, p, workingDirectory, printOutput);
	}
	private static int executeCommandSub(String command, Process p, File workingDirectory, boolean printOutput) 
	throws Exception{
		int status = -1;
		try{
			ProcessHandler inputStream =
				new ProcessHandler(p.getInputStream(),"INPUT",(printOutput?1:0));
			ProcessHandler errorStream =
				new ProcessHandler(p.getErrorStream(),"ERROR", 1);
			// start the stream threads
			inputStream.start();
			errorStream.start();

			status = p.waitFor();
			if (printOutput){
				System.out.println("Exit status of "+status+" for: \n"+command);
			}
		} finally {
			p.getInputStream().close();
			try{
				p.getErrorStream().close();
			} catch (IOException ioe){
				System.out.println("IOException on closing process error stream");
			}
			p.getOutputStream().close();
			p.destroy();
			p = null;
		}
		return status;
	}
	
	public static void copyFile(File in, File out) 
    throws IOException {
	    FileChannel inChannel = new
	        FileInputStream(in).getChannel();
	    FileChannel outChannel = new
	        FileOutputStream(out).getChannel();
	    try {
	        inChannel.transferTo(0, inChannel.size(),
	                outChannel);
	    } 
	    catch (IOException e) {
	        throw e;
	    }
	    finally {
	        if (inChannel != null) inChannel.close();
	        if (outChannel != null) outChannel.close();
	    }
	}
	
	private static Random random = null;
	public static Random getRandom(){
		if (random == null){
			random = new Random();
		}
		return random;
	}
	
	public static Map<Object,Object> sortByValueDescending(Map<Object,Object> map) {
		return sortByValue(map, false);
	}
	public static Map<Object,Object> sortByValue(Map<Object,Object> map) {
		return sortByValue(map, true);
	}
	public static Map<Object,Object> sortByValue(Map<Object,Object> map, boolean ascending) {
		List<Object> list = new LinkedList<Object>(map.entrySet());
		Collections.sort(list, new Comparator<Object>() {
			public int compare(Object o1, Object o2) {
				return ((Comparable<Object>) ((Map.Entry<Object,Object>) (o1)).getValue())
				.compareTo(((Map.Entry<Object,Object>) (o2)).getValue());
			}
		});
		if (!ascending){
			Collections.reverse(list);
		}
		Map<Object,Object> result = new LinkedHashMap<Object,Object>();
		for (Iterator<Object> it = list.iterator(); it.hasNext();) {
			Map.Entry<Object,Object> entry = (Map.Entry<Object,Object>)it.next();
			result.put(entry.getKey(), entry.getValue());
		}
		return result;
	}
	
}
