package org.cn3.ns.analysis;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.DistributionBean;

/**
 * Creates file that gives length distribution info for different classifications
 * Also creates file of lengths for each classification
 **/
public class ClassificationLengthDistributions {

	private final static int MIN_TREES = 10;
	final static String DEFAULT_FILE = "C:/fastaFromSwc/temp/Arborization/Size/allData.lts.fasta";
	
	public static void main(String args[]){
		try{
			File allDatafile;
			if (args.length > 0){
				allDatafile = new File(args[0]);
			}
			else{
				allDatafile = new File(DEFAULT_FILE);
			}
			BufferedReader reader 
				= new BufferedReader(new FileReader(allDatafile));

			String dataString = "";
			StringBuffer fastaString = new StringBuffer();
			String cellName, lab, type, region, arborization, animal;
			// arbor1: axon vs dendrite; arbor2: apical vs non-apical; arbor3: basal vs general
			String arbor1, arbor2, arbor3;
			cellName = lab = type = region = arborization = animal = null;
			Map<String,DistributionBean> typeMap = new HashMap<String,DistributionBean>();
			Pattern headerP = Pattern.compile(
				"^>(.*?)(-(Axon|BasalDendrite|ApicalDendrite|Dendrite)) \\| (.*?) \\| (.*?) \\| (.*?) \\| (.*)");
			Pattern fastaP = Pattern.compile(" *([AC]+)");
			Matcher m;
			int state = 0, treeLength = 0;
			DistributionBean distBean;
			
			while ( (dataString = reader.readLine()) != null ) {
				switch(state){
				case 0:
					m = headerP.matcher(dataString);
					if (m.matches()){
						state = 1;
						fastaString = new StringBuffer();
						fastaString.append(dataString);
						fastaString.append("\r\n");
						cellName = m.group(1).replaceAll(" ", "_");
						arborization = m.group(3).replaceAll(" ", "_");
						type = m.group(4).replaceAll(" ", "_");
						region = m.group(5).replaceAll(" ", "_");
						animal = m.group(6).replaceAll(" ", "_");
						lab = m.group(7).replaceAll(" ", "_");
						treeLength = 0;
					}
					break;
				case 1:
					m = fastaP.matcher(dataString);
					if (m.matches()){
						state = 2;
						treeLength += dataString.length();
					}
					break;
				case 2:
					m = fastaP.matcher(dataString);
					if (m.matches()){
						treeLength += dataString.length();
					}
					else{
						addTree(typeMap,type,treeLength);
						addTree(typeMap,lab,treeLength);
						addTree(typeMap,region,treeLength);
						addTree(typeMap,arborization,treeLength);
						addTree(typeMap,type+"-"+lab,treeLength);
						addTree(typeMap,type+"-"+arborization,treeLength);
						addTree(typeMap,type+"-"+lab+"-"+arborization,treeLength);
						
						m = headerP.matcher(dataString);
						if (m.matches()){
							fastaString = new StringBuffer();
							fastaString.append(dataString);
							fastaString.append("\r\n");
							cellName = m.group(1).replaceAll(" ", "_");
							arborization = m.group(3).replaceAll(" ", "_");
							type = m.group(4).replaceAll(" ", "_");
							region = m.group(5).replaceAll(" ", "_");
							animal = m.group(6).replaceAll(" ", "_");
							lab = m.group(7).replaceAll(" ", "_");
							state = 1;
							treeLength = 0;
						}
						else{
							state = 0;
						}
					}
					break;
				}
			}
			
			writeClassificationFiles(typeMap, allDatafile);
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static void addTree(Map<String,DistributionBean> typeMap, String type, int treeLength){
		DistributionBean bean;
		//int count = 1;
		if (typeMap.containsKey(type)){
			bean = typeMap.get(type);
		}
		else{
			bean = new DistributionBean();
			typeMap.put(type, bean);
		}
		bean.addData(new Integer(treeLength));
	}
	
	private static void writeClassificationFiles(
			Map<String,DistributionBean> typeMap, File allDatafile)
	throws Exception{
		String type, dir, dbName, dbDivisionType, traversal;
		
		Pattern p = Pattern.compile(".*?(stl|lts|STL|LTS)\\.fasta");
		Matcher m = p.matcher(allDatafile.getName());
		if (!m.matches()){
			throw new Exception("Filename issue on traversal direction: "+allDatafile.getName());
		}
		dir = m.group(1).toUpperCase();
		
		p = Pattern.compile(".*?(Height|Size|AvgHeight).*");
		m = p.matcher(allDatafile.getPath());
		if (!m.matches()){
			throw new Exception("Filepath issue on traversal: "+allDatafile.getPath());
		}
		traversal = m.group(1);
		
		p = Pattern.compile(".*?((Arbor)ization|Tree|Cell).*");
		m = p.matcher(allDatafile.getPath());
		if (!m.matches()){
			throw new Exception("Filepath issue on division type: "+allDatafile.getPath());
		}
		if (m.group(2) != null){
			dbDivisionType = m.group(2);
		}
		else{
			dbDivisionType = m.group(1);
		}
		
		File typeFile = new File(allDatafile.getParent()+"/ClassificationMetadata/");
		if (!typeFile.exists()){
			typeFile.mkdir();
		}
		String allClassificationFile = 
			allDatafile.getParent()+"/ClassificationMetadata/classificationDistributionData.tab";
		StringBuffer classLengths;
		DistributionBean bean;
		StringBuffer allClassification = new StringBuffer();
		allClassification.append("Classification\tMean\tCount\tStdDev\tlt25\tLT30\r\n");
		int lt25, lt30;
				
		for (Iterator<String> it = typeMap.keySet().iterator(); it.hasNext();){
			type = it.next();
			if (m.matches()){
				lt25 = lt30 = 0;
				typeFile = new File(allDatafile.getParent()
						+"/ClassificationMetadata/"+type
						+"."+dbDivisionType+".txt");
				bean = (DistributionBean)typeMap.get(type);
				classLengths = new StringBuffer();
				for (int i = 0; i < bean.getData().size(); i++){
					classLengths.append(bean.getData().get(i)+"\r\n");
					if (bean.getData().get(i).intValue() < 25){
						lt25++;
						lt30++;
					}
					else if (bean.getData().get(i).intValue() < 30){
						lt30++;
					}
				}
				DataWriter.writeStringToFile(
						typeFile, classLengths.toString());
				
				allClassification.append(type+"\t");
				allClassification.append(bean.getAverage()+"\t");
				allClassification.append(bean.getCount()+"\t");
				allClassification.append(bean.getStandardDeviation()+"\t");
				allClassification.append(lt25+"\t");
				allClassification.append(lt30+"\r\n");
			}
		}
		
		DataWriter.writeStringToFile(allClassificationFile, allClassification.toString());
		
	}
	
}
