package org.cn3.ns.analysis;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.DataConverter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * Investigates the relationship between asymmetry and C-type nodes.
 * This code is probably outdated and/or incomplete.
 *
 */
public class ContinuationAsymmetryRelation {

	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			int mapping = config.getInt(NSConstants.PARAM_MAPPING_MODE);
			//int distType = config.getInt("distribution-type");
			int distType = 3;
			
			if (distType > 0){
				randoms(mapping, distType);
			}
			else{
				arborizations(mapping);
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static void arborizations(int mapping) throws Exception{
		List<String> arborizations = NSDataReader.loadSequenceList(mapping);
		Map<String,NeuronMetadata> cellData = MetaDataManager.getCellMetaData();

		String arborizationFileBase = DataProperties.getInstance().getFastaDir()+"Arborization/Mapping"+mapping+"/Size/";
		String arborization, fastaFilename;
		NeuronMetadata neuron;
		FastaReader fr;
		
		int cs;
		double asymmetry;
		Object[] data;
		Map<String,List<Object[]>> classDataMap = new HashMap<String,List<Object[]>>();
		String arborType;
		Set<String> cellTypes = new TreeSet<String>();
		Set<String> arbTypeCellTypes = new TreeSet<String>();
		Set<String> arborTypes = new TreeSet<String>();
		
		for (int i = 0; i < arborizations.size(); i++){
			arborization = (String)arborizations.get(i);
			//System.out.println(arborization);
			// Get neuron data for arborization's lab - used to find file; + classification
			neuron = cellData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
			neuron.setCellType(ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
			arborType = ClassificationUtils.reclassifyArborizationTypes(
					NeuriteSequenceStringUtils.getArborizationType(arborization));
			cellTypes.add(neuron.getCellType());
			arborTypes.add(arborType);
			arbTypeCellTypes.add(arborType+"-"+neuron.getCellType());
			
			fastaFilename = arborizationFileBase + neuron.getLabName() 
				+ "/" + arborization + ".lts.fasta";
			// Get sequence
			fr = new FastaReader(fastaFilename);
			fr.next();
			
			cs = 0;
			for (int j = 0; j < fr.getSequence().length(); j++){
				if (fr.getSequence().charAt(j) == 'C'){
					cs++;
				}
			}
			asymmetry = DataConverter.fastaACTStringToTree(fr.getSequence()).getTreeAsymmetry();
			data = new Object[3];
			data[0] = new Double(asymmetry);
			data[1] = new Double(100*(double)cs/fr.getSequenceLength());
			data[2] = new Integer(fr.getSequenceLength());
			
			addToList(classDataMap,"All",data);
			addToList(classDataMap,neuron.getCellType(),data);
			addToList(classDataMap,arborType,data);
			addToList(classDataMap,arborType+"-"+neuron.getCellType(),data);
			
			fr.close();
		}

		System.gc();
		
		// Write to file
		int maxDataSize = 0, size;
		StringBuffer sb = new StringBuffer();
		Set<String> allClasses = new LinkedHashSet<String>();
		allClasses.add("All");
		allClasses.addAll(arborTypes);
		allClasses.addAll(cellTypes);
		allClasses.addAll(arbTypeCellTypes);
		for (String className : allClasses){
			sb.append(className+",,");
			size = classDataMap.get(className).size();
			if (size > maxDataSize){
				maxDataSize = size;
			}
		}
		sb.append("\r\n");
		for (String className : allClasses){
			sb.append("Asymmetry,%Continuations,Length,");
		}
		sb.append("\r\n");
		List<Object[]> dataList;
		NumberFormat nf = NumberFormat.getInstance();
		for (int i = 0; i < maxDataSize; i++){
			for (String className : allClasses){
				dataList = classDataMap.get(className);
				if (i < dataList.size()){
					data = dataList.get(i);
					sb.append(data[0]+","+nf.format(data[1])+","+data[2]+",");
				}
				else{
					sb.append(",,,");
				}
			}
			sb.append("\r\n");
		}
		
		DataWriter.writeStringToFile(
				DataProperties.getInstance().getResultsDir()+"continuationAsymmetryRelation.csv", sb.toString());
		
	}

	private static void randoms(int mapping, int method) throws Exception{
		List<Integer> sizes = NSDataReader.loadUniqueSequenceLengths(mapping);

		String fileBase = DataProperties.getInstance().getFastaRandomsDir()+"/Method"+method+"/Mapping"+mapping+"/SizeLTS/";
		String name, fastaFilename;
		FastaReader fr;
		
		int cs, size;
		double asymmetry;
		Object[] data;
		Map<String,List<Object[]>> classDataMap = new HashMap<String,List<Object[]>>();
		
		for (int i = 0; i < sizes.size(); i++){
			size = sizes.get(i);
			for (int num = 1; num <= 100; num++){
				name = "LEN"+size+"NUM"+num;
				
				fastaFilename = fileBase + "/" + name + "LTS.fasta";
				// Get sequence
				fr = new FastaReader(fastaFilename);
				fr.next();
				
				cs = 0;
				for (int j = 0; j < fr.getSequence().length(); j++){
					if (fr.getSequence().charAt(j) == 'C'){
						cs++;
					}
				}
				asymmetry = DataConverter.fastaACTStringToTree(fr.getSequence()).getTreeAsymmetry();
				data = new Object[3];
				data[0] = new Double(asymmetry);
				data[1] = new Double(100*(double)cs/fr.getSequenceLength());
				data[2] = new Integer(fr.getSequenceLength());
				fr.close();
				
				addToList(classDataMap,"Randoms",data);
			}			
		}

		System.gc();
		
		// Write to file
		int maxDataSize = 0;
		StringBuffer sb = new StringBuffer();
		Set<String> allClasses = new LinkedHashSet<String>();
		allClasses.add("Randoms");
		for (String className : allClasses){
			sb.append(className+",,,");
			size = classDataMap.get(className).size();
			if (size > maxDataSize){
				maxDataSize = size;
			}
		}
		sb.append("\r\n");
		for (String className : allClasses){
			sb.append("Asymmetry,%Continuations,Length,");
		}
		sb.append("\r\n");
		List<Object[]> dataList;
		NumberFormat nf = NumberFormat.getInstance();
		for (int i = 0; i < maxDataSize; i++){
			for (String className : allClasses){
				dataList = classDataMap.get(className);
				if (i < dataList.size()){
					data = dataList.get(i);
					sb.append(data[0]+","+nf.format(data[1])+","+data[2]+",");
				}
				else{
					sb.append(",,,");
				}
			}
			sb.append("\r\n");
		}
		
		DataWriter.writeStringToFile(
				DataProperties.getInstance().getResultsDir()+"continuationAsymmetryRelationOnRand.csv", sb.toString());
		
	}
	private static void addToList(Map<String,List<Object[]>> map, String className, Object[] data){
		if (!map.containsKey(className)){
			map.put(className, new ArrayList<Object[]>());
		}
		map.get(className).add(data);
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
    		.setStringParser(JSAP.INTEGER_PARSER)
    		.setRequired(false)
    		.setDefault(""+NSConstants.DEFAULT_MAPPING_MODE)
    		.setShortFlag('M') 
    		.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption("distribution-type")
    		.setStringParser(JSAP.INTEGER_PARSER)
    		.setRequired(false)
    		.setDefault(""+NSConstants.DISTRIBUTION_NEUROMORPHO)
    		.setShortFlag('d') 
    		.setLongFlag("distribution-type");
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + ContinuationAsymmetryRelation.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}

}
