package org.cn3.ns.analysis;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.domain.XYBean;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * Investigates the relationship between the proportion of C nodes and the length of sequences.
 * This code is old and may require updates to properly read in data.
 *
 */
public class LengthContinuationRelation {

	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			int mapping = config.getInt(NSConstants.PARAM_MAPPING_MODE);
			List<String> arborizations = NSDataReader.loadSequenceList(mapping);
			Map<String,NeuronMetadata> cellData = MetaDataManager.getCellMetaData();

			String arborizationFileBase = DataProperties.getInstance().getFastaDir()+"Arborization/Mapping"+mapping+"/Size/";
			String fastaFilename;
			NeuronMetadata neuron;
			FastaReader fr;
			
			int cs;
			XYBean xy;
			Map<String,List<XYBean>> classDataMap = new HashMap<String,List<XYBean>>();
			String arborType;
			Set<String> cellTypes = new TreeSet<String>();
			Set<String> arbTypeCellTypes = new TreeSet<String>();
			Set<String> arborTypes = new TreeSet<String>();

			for (String arborization : arborizations){
				//System.out.println(arborization);
				// Get neuron data for arborization's lab - used to find file; + classification
				neuron = cellData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
				neuron.setCellType(ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
				arborType = ClassificationUtils.reclassifyArborizationTypes(
						NeuriteSequenceStringUtils.getArborizationType(arborization));
				cellTypes.add(neuron.getCellType());
				arborTypes.add(arborType);
				arbTypeCellTypes.add(arborType+"-"+neuron.getCellType());
				
				fastaFilename = arborizationFileBase + neuron.getLabName() 
					+ "/" + arborization + ".lts.fasta";
				// Get sequence
				fr = new FastaReader(fastaFilename);
				fr.next();
				
				cs = 0;
				for (int j = 0; j < fr.getSequence().length(); j++){
					if (fr.getSequence().charAt(j) == 'C'){
						cs++;
					}
				}
				xy = new XYBean(fr.getSequenceLength(), 100*(double)cs/fr.getSequenceLength());
				
				addToList(classDataMap,"All",xy);
				addToList(classDataMap,neuron.getCellType(),xy);
				addToList(classDataMap,arborType,xy);
				addToList(classDataMap,arborType+"-"+neuron.getCellType(),xy);
				fr.close();
			}
			
			// Write to file
			int maxDataSize = 0, size;
			StringBuffer sb = new StringBuffer();
			Set<String> allClasses = new LinkedHashSet<String>();
			allClasses.add("All");
			allClasses.addAll(arborTypes);
			allClasses.addAll(cellTypes);
			allClasses.addAll(arbTypeCellTypes);
			for (String className : allClasses){
				sb.append(className+",,");
				size = classDataMap.get(className).size();
				if (size > maxDataSize){
					maxDataSize = size;
				}
			}
			for (String className : allClasses){
				sb.append("Length,%C,");
			}
			List<XYBean> dataList;
			sb.append("\r\n");
			NumberFormat nf = NumberFormat.getInstance();
			for (int i = 0; i < maxDataSize; i++){
				for (String className : allClasses){
					dataList = classDataMap.get(className);
					if (i < dataList.size()){
						xy = dataList.get(i);
						sb.append(xy.getX()+","+nf.format(xy.getY())+",");
					}
					else{
						sb.append(",,");
					}
				}
				sb.append("\r\n");
			}
			
			DataWriter.writeStringToFile(
					DataProperties.getInstance().getResultsDir()+"lengthContinuationsRelation.csv", sb.toString());
			
		} catch (Exception e){
			e.printStackTrace();
		}

	}

	private static void addToList(Map<String,List<XYBean>> map, String className, XYBean data){
		if (!map.containsKey(className)){
			map.put(className, new ArrayList<XYBean>());
		}
		map.get(className).add(data);
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
    		.setStringParser(JSAP.INTEGER_PARSER)
    		.setRequired(false)
    		.setDefault(""+NSConstants.DEFAULT_MAPPING_MODE)
    		.setShortFlag('M') 
    		.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + LengthContinuationRelation.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}

}
