package org.cn3.ns.analysis;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.SWCReader;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.Soma;
import org.cn3.morpho.utils.BinaryTreeUtils;

/**
 * 
 * @author gillette
 *
 * An old program, and possibly suboptimal. I cannot recall whether it was
 * written based on any existing algorithm or just cobbled together for a quick
 * analysis.
 *
 */
public class MaximumCommonSubtree {

	/**
	 * 
	 * @param tree1
	 * @param tree2
	 * @return List of maximum common subtrees (there could be several distinct ones)
	 */
	public static List<BinaryTreeNode> findMaximumCommonSubtree(
			BinaryTreeNode tree1, BinaryTreeNode tree2) throws Exception{
		return findMaximumCommonSubtree(tree1, tree2, false, false);
	}
	/**
	 * 
	 * @param tree1
	 * @param tree2
	 * @param labeled
	 * @return List of maximum common subtrees (there could be several distinct ones)
	 */
	public static List<BinaryTreeNode> findMaximumCommonSubtree(
			BinaryTreeNode tree1, BinaryTreeNode tree2, boolean labeled) throws Exception{
		return findMaximumCommonSubtree(tree1, tree2, labeled, false);
	}

	/**
	 * Determined iteratively without recursion
	 * @param tree1
	 * @param tree2
	 * @param labeled
	 * @param uniqueMCSs
	 * @return List of maximum common subtrees (there could be several distinct ones)
	 */
	public static List<BinaryTreeNode> findMaximumCommonSubtree (
			BinaryTreeNode tree1, BinaryTreeNode tree2, boolean labeled, boolean uniqueMCSs) throws Exception{
		// For a given node in tree1, the maximum common subtree(s)
		Map<BinaryTreeNode,List<BinaryTreeNode>> tree1MCSs = new HashMap<BinaryTreeNode,List<BinaryTreeNode>>();
		// Indicates whether the MCS of a node's subtree is rooted at the node itself
		Map<BinaryTreeNode,Boolean> isMCSRootedAtNode = new HashMap<BinaryTreeNode,Boolean>();

		// Stack of tree 1 nodes enabling post-order traversal
		Stack<BinaryTreeNode> tree1Stack = BinaryTreeUtils.getLeavesToRootStack(tree1, false);
		// List of tree2 terminal nodes
		List<BinaryTreeNode> tree2Terms = BinaryTreeUtils.createTerminalNodeList(tree2);

		// Stack of tree 1 bifurcations enabling post-order traversal
		//LinkedStack tree1Stack = BinaryTreeUtils.getLeavesToRootStack(tree1, true);
		// List of tree2 terminal bifurcations (bifurcations with degree 2)
		//List<BinaryTreeNode> tree2Terms = BinaryTreeUtils.createTerminalBifurcationList(tree2);

		//int count = 1;

		List<BinaryTreeNode> matchingNodes, MCSNodesLeft, MCSNodesRight;
		Set<BinaryTreeNode> MCSs, skip;
		BinaryTreeNode node1;
		while (!tree1Stack.isEmpty()){
			//System.out.println("\nNode "+(count++));
			node1 = tree1Stack.pop();
			// Node has degree 2
			//if (node1.getDegree() == 2){
			// Node is a leaf (degree of 1)
			if (node1.getDegree() == 1){
				if (labeled){
					matchingNodes = new ArrayList<BinaryTreeNode>();
					for (int i = 0; i < tree2Terms.size(); i++){
						if (isSameLabel(node1.getData(), tree2Terms.get(i).getData())){
							matchingNodes.add(tree2Terms.get(i));
						}
					}
					tree1MCSs.put(node1, matchingNodes);
				}
				else{
					tree1MCSs.put(node1, tree2Terms);
				}
				if (tree1MCSs.size() > 0){
					isMCSRootedAtNode.put(node1, true);
				}
			}
			// Node has degree > 1
			else{
				//System.out.println(MappingUtils.traverseTreeToString(node1, NSConstants.MAPPING_MODE_TOPO_ACT, "LTS"));
				MCSs = new HashSet<BinaryTreeNode>();
				MCSNodesLeft = new ArrayList<BinaryTreeNode>(tree1MCSs.get(node1.getLeft()));
				MCSNodesRight = new ArrayList<BinaryTreeNode>(tree1MCSs.get(node1.getRight()));
				
				// If both subtrees of current node have MCS rooted at the node
				if (isMCSRootedAtNode.get(node1.getLeft()) && isMCSRootedAtNode.get(node1.getRight())){
					// Determine whether matching subtrees in tree2 are siblings. If so, match current node to 
					// subtree matches parent.
					skip = new HashSet<BinaryTreeNode>();
					if (MCSNodesLeft.get(0) == MCSNodesRight.get(0)){ // If one is the same, then they all are
						// Subtree shapes are the same, so they'll have the same MCSs. This is a faster method.
						for (int i = 0; i < MCSNodesLeft.size()-1; i++){
							// Skip any Left node match that has already been joined as a Right node match
							if (!skip.contains(MCSNodesLeft.get(i))){
								for (int j = i+1; j < MCSNodesRight.size(); j++){
									// If parent of left match and right match are the same (no risk of duplicating parent)
									// 		AND if labeling is not required OR labels match
									// THEN parent roots match to current node1
									if (MCSNodesLeft.get(i).getParent() == MCSNodesRight.get(j).getParent() &&
											(!labeled || isSameLabel(node1.getData(), MCSNodesLeft.get(i).getParent().getData()))){
										MCSs.add(MCSNodesLeft.get(i).getParent());
										skip.add(MCSNodesRight.get(j));
										isMCSRootedAtNode.put(node1, true);
										// No reason to keep looking through matchingNodesRight for current left node match
										j = MCSNodesRight.size();
									}
								}
							}
						}
					}
					else{
						// Basic case in which subtrees are different so 
						// there is no chance of right and left having the same matches.
						for (int i = 0; i < MCSNodesLeft.size(); i++){
							for (int j = 0; j < MCSNodesRight.size(); j++){
								// If parent of left match and right match are the same (no risk of duplicating parent)
								// 		AND if labeling is not required OR labels match
								// THEN parent roots match to current node1
								if (MCSNodesLeft.get(i).getParent() == MCSNodesRight.get(j).getParent() &&
										(!labeled || isSameLabel(node1.getData(), MCSNodesLeft.get(i).getParent().getData()))){
									MCSs.add(MCSNodesLeft.get(i).getParent());
									isMCSRootedAtNode.put(node1, true);
									// No reason to keep looking through matchingNodesRight for current left node match
									j = MCSNodesRight.size();
								}
							}
						}
					}
				}
				// If either child subtrees' MCS isn't rooted at each child, or if current node doesn't have a match  
				if (MCSs.size() == 0){
					isMCSRootedAtNode.put(node1, false);
					// Get the largest MCS of each child
					if (MCSNodesLeft == null || MCSNodesLeft.isEmpty()){
						if (MCSNodesRight != null && !MCSNodesRight.isEmpty()){
							// Need all of them, since there could be more than one MCS with different shapes
							MCSs.addAll(MCSNodesRight);
						}
					}
					else{
						// Right has no MCSs or left MCSs are bigger
						if (MCSNodesRight == null || MCSNodesRight.isEmpty()
								|| MCSNodesLeft.get(0).getDegree() > MCSNodesRight.get(0).getDegree()){
							MCSs.addAll(MCSNodesLeft);
						}
						// Right MCSs are bigger than left MCSs
						else if (MCSNodesRight.get(0).getDegree() > MCSNodesLeft.get(0).getDegree()){
							MCSs.addAll(MCSNodesRight);
						}
						// Left and right MCSs are of equal size, add both
						else{
							MCSs.addAll(MCSNodesLeft);
							MCSs.addAll(MCSNodesRight);
						}
					}
				}
				tree1MCSs.put(node1, new ArrayList<BinaryTreeNode>(MCSs));
				
				// Debug
				/*
				System.out.println("MCSs:");
				for (Iterator<BinaryTreeNode> it = MCSs.iterator(); it.hasNext();){
					System.out.println(MappingUtils.traverseTreeToString(it.next(), NSConstants.MAPPING_MODE_TOPO_ACT));
				}*/
			}
		}
		// This should only happen when dealing with a labeled tree
		if (tree1MCSs.get(tree1) == null || tree1MCSs.get(tree1).size() == 0){
			return null;
		}

		// Make copies
		List<BinaryTreeNode> topMCSs = new ArrayList<BinaryTreeNode>();
		for (int i = 0; i < tree1MCSs.get(tree1).size(); i++){
			topMCSs.add(BinaryTreeUtils.copyTree(tree1MCSs.get(tree1).get(i), true));
		}
		
		// For returned trees, only return one of each tree shape (uniquely labeled if labeling applies)
		if (uniqueMCSs){
			skip = new HashSet<BinaryTreeNode>();
			Stack<BinaryTreeNode> tree2Stack;
			for (int i = 0; i < topMCSs.size()-1; i++){
				if (!skip.contains(topMCSs.get(i))){
					for (int j = i+1; j < topMCSs.size(); j++){
						// If shape number is the same
						if (topMCSs.get(i).getShapeNum().equals(topMCSs.get(j).getShapeNum())){
							if (labeled){
								// Have to also make sure all node labels are the same
								tree1Stack = BinaryTreeUtils.getLeavesToRootStack(topMCSs.get(i), true);
								tree2Stack = BinaryTreeUtils.getLeavesToRootStack(topMCSs.get(j), true);
								boolean same = true;
								while (!tree1Stack.isEmpty()){
									if (!isSameLabel(
											((BinaryTreeNode)tree1Stack.pop()).getData(), 
											((BinaryTreeNode)tree2Stack.pop()).getData())){
										same = false;
										break;
									}
								}
								// All node labels are the same, add it to the skip set
								if (same){
									skip.add(topMCSs.get(j));
								}
							}
							// No label requirement, shapes are the same, add to skip set
							else{
								skip.add(topMCSs.get(j));
							}
						}
					}
				}
			}
			topMCSs.removeAll(skip);
		}
		
		return topMCSs;
	}

	private static boolean isSameLabel(Object data1, Object data2){
		return (data1 == null && data2 == null) ||
			data1 != null && data2 != null && data1.equals(data2);
	}
	
	public static void main(String args[]){
		try{
			String order = "LTS";
			File archiveDir, directoryOfFiles, swcDir = new File("C:/swcData/NM/");
			File[] directories = swcDir.listFiles();
			List<File> allFiles = new ArrayList<File>();
			for (int i = 0; i < 5; i++){
			//for (int i = 0; i < directories.length; i++){
				archiveDir = directories[i];
				// Makes sure we are looking at a directory
				if (directories[i].isDirectory()){
					directoryOfFiles = new File(archiveDir.getAbsolutePath()+"\\CNG Version\\");
					if (!directoryOfFiles.exists()){
						directoryOfFiles = new File(archiveDir.getAbsolutePath()+"\\CNGVersion");
					}
					if (!directoryOfFiles.exists()){
						directoryOfFiles = new File(archiveDir.getAbsolutePath());
					}
					if (directoryOfFiles.exists()){
						allFiles.addAll(Arrays.asList(directoryOfFiles.listFiles()));
					}
				}
			}
			int numComparisons = allFiles.size()*(allFiles.size()-1)/2;
			List<Integer> length1 = new ArrayList<Integer>(numComparisons);
			List<Integer> length2 = new ArrayList<Integer>(numComparisons);
			List<Long> time = new ArrayList<Long>(numComparisons);
			List<Integer> mcsSize = new ArrayList<Integer>(numComparisons);
			long timeStart;
			List<BinaryTreeNode> treeList1, treeList2, mcs;
			BinaryTreeNode tree1, tree2;
			int count = 0;
			for (int i = 0; i < allFiles.size()-1; i++){
				for (int j = i+1; j < allFiles.size(); j++){
					System.out.println("i: "+(i+1)+"; j: "+(j+1)+" of "+(allFiles.size()));
					treeList1 = fillTreeList(allFiles.get(i));
					treeList2 = fillTreeList(allFiles.get(j));
					for (int x = 0; x < treeList1.size(); x++){
						for (int y = 0; y < treeList2.size(); y++){
							tree1 = treeList1.get(x);
							tree2 = treeList2.get(y);
							length1.add(tree1.getDegree());
							length2.add(tree2.getDegree());
							timeStart = new Date().getTime();
							mcs = findMaximumCommonSubtree(tree1, tree2);
							time.add(new Date().getTime() - timeStart);
							mcsSize.add(mcs.get(0).getDegree()); 
							count++;
						}
					}
					treeList1 = null;
					treeList2 = null;
					mcs = null;
					System.gc();
				}
			}
			StringBuffer sb = new StringBuffer();
			sb.append("Degree1,Degree2,Time_ms,mcsSize\n");
			for (int i = 0; i < numComparisons; i++){
				sb.append(length1.get(i)+","+length2.get(i)+","+time.get(i)+","+mcsSize.get(i)+"\n");
			}
			DataWriter.writeStringToFile("C:/MCS_Analysis.csv", sb.toString());
			/*
			Soma soma1 = SWCReader.convertSwcToSoma(new File("C:/swcData/NM/acsady/CNG version/cZI_1.CNG.swc"));
			BinaryTreeNode dendrite1 = soma1.getBasalDendriteTree().getTreeHead();
			Soma soma2 = SWCReader.convertSwcToSoma(new File("C:/swcData/NM/acsady/CNG version/cZI_2.CNG.swc"));
			BinaryTreeNode dendrite2 = soma2.getBasalDendriteTree().getTreeHead();
			System.out.println("cZI_1: "+MappingUtils.traverseTreeToString(dendrite1, NSConstants.MAPPING_MODE_TOPO_ACT));
			System.out.println("cZI_2: "+MappingUtils.traverseTreeToString(dendrite2, NSConstants.MAPPING_MODE_TOPO_ACT));
			List<BinaryTreeNode> dendriteMCSs = findMaximumCommonSubtree(
					dendrite1, dendrite2, false, false);
			for (int i = 0; i < dendriteMCSs.size(); i++){
				System.out.println("MCS"+(i+1)+": "+MappingUtils.traverseTreeToString(
						dendriteMCSs.get(i), NSConstants.MAPPING_MODE_TOPO_ACT, order));
			}*/
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static List<BinaryTreeNode> fillTreeList(File file) throws Exception{
		List<BinaryTreeNode> treeList = new ArrayList<BinaryTreeNode>();
		Soma<BinaryTreeNode> soma = SWCReader.convertSwcToBinarySoma(file);
		if (soma.hasAxonalTree() && soma.getAxonalTree().getTreeHead().getTreeSize() > 21){
			treeList.add(soma.getAxonalTree().getTreeHead());
		}
		if (soma.hasApicalDendriteTree() && soma.getApicalDendriteTree().getTreeHead().getTreeSize() > 21){
			treeList.add(soma.getApicalDendriteTree().getTreeHead());
		}
		if (soma.hasBasalDendriteTree() && soma.getBasalDendriteTree().getTreeHead().getTreeSize() > 21){
			treeList.add(soma.getBasalDendriteTree().getTreeHead());
		}
		return treeList;
	}
	
}
