package org.cn3.ns.analysis;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.domain.XYBean;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.ProjectProperties;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Node type (A or C) profile along branch order
 * 	Averaged over tree set OR summed over tree set
 *
 */
public class PartitionAsymmetryBranchOrderAnalysis {

	private static final String PARAM_PROFILE_MODE = "profile-mode";
	private static final String PARAM_INCLUDE_C = "include-c";
/*	private static final String PARAM_NODE_TYPE = "node-type";
	private static final String PARAM_METRIC = "metric";
	
	private static final int NODE_TYPE_ALL = 0;
	private static final int NODE_TYPE_A = 1;
	private static final int NODE_TYPE_C = 2;
	private static final int NODE_TYPE_T = 3;
	*/
	// Combines all instances of nodes at a given branch order, then gets %C
	private static final int PROFILE_MODE_SUM = 1;
	// For a given branch order, a percent of total branch order is taken
	// That % branch order and then the %C for the sequence is a data point
	private static final int PROFILE_MODE_PROPORTION = 2;
	private static final int PROFILE_MODE_DEGREE = 3;
	
	private static final int MAPPING = 2;
	
	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			int mapping = MAPPING;
			int profileMode = config.getInt(PARAM_PROFILE_MODE);
			boolean includeC = config.getBoolean(PARAM_INCLUDE_C);
			List<String> arborizations = NSDataReader.loadSequenceList(mapping);
			Map<String,NeuronMetadata> cellData = MetaDataManager.getCellMetaData();

			// Variable preparation
			String arborizationFileBase = DataProperties.getInstance().getFastaDir()+"Arborization/Mapping"+mapping+"/Size/";
			String arborization, fastaFilename;
			NeuronMetadata neuron;
			FastaReader fr;
			
			String arborType;
			Map<String,List<Object>> classDataMap = new HashMap<String,List<Object>>();
			classDataMap.put("All", new ArrayList<Object>());
			List<Object> globalData, allTypesGlobalData = classDataMap.get("All");
			XYBean xy;
			Set<String> arborTypes = new TreeSet<String>();
			DistributionBean instanceDist, globalDist;
			
			// Get partition data from all arbors
			for (int i = 0; i < arborizations.size(); i++){
				arborization = (String)arborizations.get(i);
				//System.out.println(arborization);
				
				// Get neuron data for arborization's lab - used to find file; + classification
				neuron = (NeuronMetadata)cellData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
				neuron.setCellType(ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
				arborType = ClassificationUtils.reclassifyArborizationTypes(
						NeuriteSequenceStringUtils.getArborizationType(arborization));
				arborTypes.add(arborType);
				
				// Generate filename and retrieve sequence
				fastaFilename = arborizationFileBase + neuron.getLabName() 
					+ "/" + arborization + ".lts.fasta";
				// Get sequence
				fr = new FastaReader(fastaFilename);
				fr.next();

				// Get partition profile
				List<DistributionBean> partitionProfile = generateProfileFromRoot(fr.getSequence(), includeC);
				if (!classDataMap.containsKey(arborType)){
					classDataMap.put(arborType, new ArrayList<Object>());
				}
				fr.close();
				
				// Get dataset for given arbor type
				globalData = classDataMap.get(arborType);
				
				for (int j = 0; j < partitionProfile.size(); j++){
					// Pull distribution of asymmetry for the branch order
					instanceDist = (DistributionBean)partitionProfile.get(j);
					switch (profileMode){
					case PROFILE_MODE_SUM:
						//Get global Ap distribution for this branch order
						if (j >= globalData.size()){
							globalData.add(new DistributionBean());
						}
						globalDist = (DistributionBean)globalData.get(j);
						// Add all node instances to distribution
						globalDist.addData(instanceDist.getData());

						// Same as above, but for All arbor types set
						if (j >= allTypesGlobalData.size()){
							allTypesGlobalData.add(new DistributionBean());
						}
						globalDist = (DistributionBean)allTypesGlobalData.get(j);
						globalDist.addData(instanceDist.getData());
						break;
					case PROFILE_MODE_PROPORTION:
						// Add normalized branch order, %C XY pair 
						xy = new XYBean(
								(double)j/partitionProfile.size(),
								instanceDist.getMean());
						globalData.add(xy);
						allTypesGlobalData.add(xy);

						break;
	/*				case PROFILE_MODE_DEGREE:
						dataObs = new Object[2];
						dataObs[0] = new Integer(degree);
						dataObs[1] = new Double(partition);
						break;*/
					}
				}
			}

			System.gc();
			
			Set<String> allClasses = new LinkedHashSet<String>();
			allClasses.add("All");
			allClasses.addAll(arborTypes);
	
			String outputData = "";
			String outfile = ProjectProperties.getInstance().getResultsDir()+"partitionAnalysisByBranchOrder";
			if (profileMode == PROFILE_MODE_SUM){
				outfile = outfile + "sum-";
				outputData = writeCombinedAbsoluteFile(classDataMap, allClasses);
			}
			else if (profileMode == PROFILE_MODE_PROPORTION){
				outfile = outfile + "proportion-";
				outputData = writeProportionFile(classDataMap, allClasses);
			}
			if (includeC){
				outfile = outfile+"-All";
			}
			else{
				outfile = outfile+"-AOnly";
			}
			outfile = outfile+".csv";
			DataWriter.writeStringToFile(outfile, outputData);
						
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static String writeCombinedAbsoluteFile(Map classDataMap, Set allClasses){
		StringBuffer sb = new StringBuffer();
		int maxBranchOrder = 0;
		String className;
		List list;
		NumberFormat nf = NumberFormat.getInstance();
		nf.setGroupingUsed(false);
		nf.setMaximumFractionDigits(4);
		for (Iterator it = allClasses.iterator(); it.hasNext();){
			list = (List)classDataMap.get(it.next());
			if (list.size() > maxBranchOrder){
				maxBranchOrder = list.size()-1;
			}
		}
		sb.append("BranchOrder");
		for (int branchOrder = 0; branchOrder <= maxBranchOrder; branchOrder++){
			sb.append(","+branchOrder);
		}
		sb.append("\r\n");
		for (Iterator it = allClasses.iterator(); it.hasNext();){
			className = (String)it.next();
			sb.append(className+" Mean");
			list = (List)classDataMap.get(className);
			for (int i = 0; i <= maxBranchOrder; i++){
				sb.append(",");
				if (i < list.size()){
					//sb.append(nf.format(((DistributionBean)list.get(i)).getCount()));
					sb.append(nf.format(((DistributionBean)list.get(i)).getMean()));
				}
			}
			sb.append("\r\n");
			sb.append(className+" N");
			for (int i = 0; i <= maxBranchOrder; i++){
				sb.append(",");
				if (i < list.size()){
					//sb.append(nf.format(((DistributionBean)list.get(i)).getCount()));
					sb.append(nf.format(((DistributionBean)list.get(i)).getCount()));
				}
			}
			sb.append("\r\n");
			sb.append(className+" SD");
			for (int i = 0; i <= maxBranchOrder; i++){
				sb.append(",");
				if (i < list.size()){
					//sb.append(nf.format(((DistributionBean)list.get(i)).getCount()));
					sb.append(nf.format(((DistributionBean)list.get(i)).getStandardDeviation()));
				}
			}
			sb.append("\r\n");
		}
		return sb.toString();
	}
	
	private static String writeProportionFile(Map classDataMap, Set allClasses){
		StringBuffer sb = new StringBuffer();
		String className;
		int size, maxDataSize = 0;
		NumberFormat nf = NumberFormat.getInstance();
		nf.setGroupingUsed(false);
		nf.setMaximumFractionDigits(4);
		for (Iterator it = allClasses.iterator(); it.hasNext();){
			className = (String)it.next();
			System.out.println(className);
			sb.append(className+",,");
			size = ((List)classDataMap.get(className)).size();
			if (size > maxDataSize){
				maxDataSize = size;
			}
		}
		sb.append("\r\n");
		for (Iterator it = allClasses.iterator(); it.hasNext();){
			it.next();
			sb.append("ProportionalBranchOrder,%C,");
		}
		sb.append("\r\n");
		List dataList;
		XYBean xy;
		//NumberFormat nf = NumberFormat.getInstance();
		for (int i = 0; i < maxDataSize; i++){
			for (Iterator it = allClasses.iterator(); it.hasNext();){
				className = (String)it.next();
				dataList = (List)classDataMap.get(className);
				if (i < dataList.size()){
					xy = (XYBean)dataList.get(i);
					sb.append(nf.format(xy.getX())+","+nf.format(xy.getY())+",");
				}
				else{
					sb.append(",,");
				}
			}
			sb.append("\r\n");
		}
		return sb.toString();
	}
	
	private static List<DistributionBean> generateProfileFromRoot(String sequence, boolean includeC){
		List<DistributionBean> partitionProfile = new ArrayList<DistributionBean>();
		DistributionBean dist;
		//LinkedStack helpers = new LinkedStack();
		SequenceTraversalHelper helper = new SequenceTraversalHelper();
		double partition;
		int degree;
		String[] subtrees;
		String subtree;
		
		for (int i = 0; i < sequence.length(); i++){
			helper.traverse(sequence.charAt(i));
			// Add to data
			if (helper.getBranchOrder()+1 > partitionProfile.size()){
				partitionProfile.add(new DistributionBean());
			}
			dist = partitionProfile.get(helper.getLevel());
			
			partition = -1;
			if (sequence.charAt(i) == 'A'){
				subtrees = findChildTrees(sequence, i);
				degree = subtrees[0].length()+subtrees[1].length()+2;
				partition = ((double)(degree-2*(subtrees[1].length()+1)))/(degree-2);
			}
			else if (includeC && sequence.charAt(i) == 'C'){
				subtree = findSubtree(sequence, i);
				degree = subtree.length()+1;
				partition = 1;
			}
			
			// If partition asymmetry was calculated, add to the distribution at this branch order
			if (partition > -1){
				dist.addData(partition);
			}
		}
		return partitionProfile;
	}
	
	private static void addDataToClass(Map dataMap, Object key, Object[] data){
		if (!dataMap.containsKey(key)){
			dataMap.put(key, new ArrayList());
		}
		List list = (List)dataMap.get(key);
		list.add(data);
	}

	private static String[] findChildTrees(String treeString, int nodeIndex){
		if (treeString.charAt(nodeIndex)!='A' && treeString.charAt(nodeIndex)!='G'){
			return null;
		}
		String[] subtrees = new String[2];
		subtrees[0] = findSubtree(treeString, nodeIndex+1);
		subtrees[1] = findSubtree(treeString, nodeIndex+subtrees[0].length()+1);
		return subtrees;
	}
	
	private static String findSubtree(String treeString, int nodeIndex){
		int tees = 0, currIndex = nodeIndex;
		char chr;
		while (tees < 1){
			chr = treeString.charAt(currIndex++);
			if (chr=='A'){
				tees--;
			}
			else if (chr=='T'){
				tees++;
			}
		}
		return treeString.substring(nodeIndex, currIndex);
	}
	

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(PARAM_PROFILE_MODE)
        	.setStringParser(JSAP.INTEGER_PARSER)
        	.setRequired(false)
        	.setDefault(""+PROFILE_MODE_SUM)
        	.setShortFlag('m') 
        	.setLongFlag(PARAM_PROFILE_MODE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_INCLUDE_C)
    		.setStringParser(JSAP.BOOLEAN_PARSER)
    		.setRequired(false)
    		.setDefault("false")
    		.setShortFlag('c') 
    		.setLongFlag(PARAM_INCLUDE_C);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + PartitionAsymmetryBranchOrderAnalysis.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}
}
