package org.cn3.ns.data;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.StringTreeNode;
import org.cn3.ns.resources.NSConstants;

public class DataConverter {

	public static BinaryTreeNode fastaStringToTree(String fastaString) throws Exception{
		return fastaStringToTree(NSConstants.MAPPING_MODE_TOPO_ACT,fastaString);
	}
	public static BinaryTreeNode fastaStringToTree(int mapping, String fastaString) throws Exception{
		switch (mapping){
		case NSConstants.MAPPING_MODE_TOPO_CA:
			return fastaCAStringToTree(fastaString);
		case NSConstants.MAPPING_MODE_TOPO_ACT:
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			return fastaACTStringToTree(fastaString);
		default:
			return null;
		}
	}

	public static BinaryTreeNode fastaCAStringToTree(String fastaString) throws Exception{
		BinaryTreeNode node, treeHead;
		node = treeHead = new BinaryTreeNode();
		boolean done;
    	for (int i = 0; i < fastaString.length(); i++){
    		if (fastaString.charAt(i) == 'A'){
    			node.setData("A");
    			done = false;
    			while (node.getParent() != null && !done){
    				if (node.isLeft()){
    					node = node.getParent().getRight();
    					done = true;
    				}
    				else{
    					node = node.getParent();
    				}
    			}
    		}
    		else if (fastaString.charAt(i) == 'C'){
    			node.setData("C");
    			node.setRight(new BinaryTreeNode());
    			node.setLeft(new BinaryTreeNode());
    			node = node.getLeft();
    		}
    	}
		
		return treeHead;
	}

	public static BinaryTreeNode fastaStringToTree(String fastaString, int mapping) throws Exception{
		switch (mapping){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			return fastaACTStringToTree(fastaString);
		case NSConstants.MAPPING_MODE_TOPO_CA:
			return fastaCAStringToTree(fastaString);
		default: 
			return null;
		}
	}

	public static BinaryTreeNode fastaACTStringToTree(String fastaString) throws Exception{
		BinaryTreeNode node, treeHead;
		node = treeHead = new BinaryTreeNode();
		boolean done;
		//System.out.println(fastaString);
    	for (int i = 0; i < fastaString.length(); i++){
			node.setRight(new BinaryTreeNode());
			node.setLeft(new BinaryTreeNode());
			//System.out.print(""+fastaString.charAt(i));
    		if (fastaString.charAt(i) == 'A' || fastaString.charAt(i) == 'G'){
    			node.setData(""+fastaString.charAt(i));
    			node = node.getLeft();
    		}
    		else if (fastaString.charAt(i) == 'C'){
    			node.setData("C");
    			node = node.getLeft();
    		}
    		else if (fastaString.charAt(i) == 'T'){
    			node.setData("T");
    			done = false;
    			while (node.getParent() != null && !done){
    				if (node.getParent().getData().equals("C") || node.isRight()){
    					node = node.getParent();
    					//System.out.println("node is right or parent is C");
    				}
    				else if (node.isLeft()){
    					node = node.getParent().getRight();
    					done = true;
    					//System.out.println("node is left, setting right side");
    				}
    			}
    			
    		}
    	}
		
		return treeHead;
	}
	
	public static List<StringTreeNode> fastaACTStringToSwcTree(String fastaString) throws Exception{
		StringTreeNode node = new StringTreeNode();
		List<StringTreeNode> roots = new ArrayList<StringTreeNode>();
		roots.add(node);
		
		Stack<StringTreeNode> stack = new Stack<StringTreeNode>();
		
		boolean endOfTree = false; // For determining when to start a new root
		//System.out.println(fastaString);
    	for (int i = 0; i < fastaString.length(); i++){
    		if (endOfTree){
    			// Create new root
				node = new StringTreeNode();
				roots.add(node);
				endOfTree = false;
    		}
			node.addChildNode(new StringTreeNode());
			node.addChildNode(new StringTreeNode());
			//System.out.print(""+fastaString.charAt(i));
    		if (fastaString.charAt(i) == 'A' || fastaString.charAt(i) == 'G'){
    			node.setData(""+fastaString.charAt(i));
    			node = node.getChildren().get(0);
    			stack.push(node);
    		}
    		else if (fastaString.charAt(i) == 'C'){
    			node.setData("C");
    			node = node.getChildren().get(0);
    		}
    		else if (fastaString.charAt(i) == 'T'){
    			node.setData("T");
    			if (!stack.isEmpty()){
    				node = stack.pop().getChildren().get(1);
    			}
    			else{
    				endOfTree = true;
    			}
    		}
    	}
		
		return roots;
	}
	
	private static Set<String> aNodes = null;
	private static Set<String> cNodes = null;
	private static Set<String> tNodes = null;
	private static void initNodeSets(){
		if (aNodes == null || cNodes == null || tNodes == null){
			aNodes = new TreeSet<String>();
			aNodes.add("A");aNodes.add("R");aNodes.add("N");
			aNodes.add("D");aNodes.add("C");aNodes.add("Q");
			cNodes = new TreeSet<String>();
			cNodes.add("E");cNodes.add("G");cNodes.add("H");
			cNodes.add("I");cNodes.add("L");cNodes.add("K");
			cNodes.add("M");cNodes.add("F");
			tNodes = new TreeSet<String>();
			tNodes.add("P");tNodes.add("S");tNodes.add("T");
			tNodes.add("W");tNodes.add("Y");tNodes.add("V");
		}		
	}
	private static Set<String> getANodes(){
		initNodeSets();
		return aNodes;
	}
	private static Set<String> getCNodes(){
		initNodeSets();
		return cNodes;
	}
	private static Set<String> getTNodes(){
		initNodeSets();
		return tNodes;
	}
	
	public static BinaryTreeNode fastaTopoGeoStringToTree(String fastaString) throws Exception{
		Set<String> aNodes = getANodes();
		Set<String> cNodes = getCNodes();
		Set<String> tNodes = getTNodes();
		BinaryTreeNode node, treeHead;
		node = treeHead = new BinaryTreeNode();
		boolean done;
		//System.out.println(fastaString);
    	for (int i = 0; i < fastaString.length(); i++){
			node.setRight(new BinaryTreeNode());
			node.setLeft(new BinaryTreeNode());
			//System.out.print(""+fastaString.charAt(i));
    		if (aNodes.contains(""+fastaString.charAt(i))){
    			node.setData(""+fastaString.charAt(i));
    			node = node.getLeft();
    		}
    		else if (cNodes.contains(""+fastaString.charAt(i))){
    			node.setData(""+fastaString.charAt(i));
    			node = node.getLeft();
    		}
    		else if (tNodes.contains(""+fastaString.charAt(i))){
    			node.setData(""+fastaString.charAt(i));
    			done = false;
    			while (node.getParent() != null && !done){
    				if (cNodes.contains(node.getParent().getData()) || node.isRight()){
    					node = node.getParent();
    					//System.out.println("node is right or parent is C");
    				}
    				else if (node.isLeft()){
    					node = node.getParent().getRight();
    					done = true;
    					//System.out.println("node is left, setting right side");
    				}
    			}
    		}
    	}
		return treeHead;
	}
	
	public static BinaryTreeNode convertTreeDataToStructureData(BinaryTreeNode node){
    	Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
    	BinaryTreeNode headNode = node, tmp;
    	stack.push(node);

    	while (!stack.isEmpty()){
    		node = (BinaryTreeNode)stack.pop();
    		if (node.isLeaf()){
    			node.setData("A");
    		}
			else{
				node.setData("C");
				if (node.hasLeft()){
					// Add left branch to stack
					stack.push(node.getLeft());
					// If this node only has one child, remove it
					if (!node.hasRight()){
						tmp = node.getLeft();
						// If head node, replace head node with its only child
						if (node.getParent() == null){
							headNode = tmp;
							tmp.setParent(null);
						}
						// Otherwise replace current node with its only child (checking for right or left)
						else if (node.isRight()){
							node.getParent().setRight(tmp);
						}
						else{
							node.getParent().setLeft(tmp);
						}
					}
				}
				if (node.hasRight()){
					stack.push(node.getRight());
					if (!node.hasLeft()){
						tmp = node.getRight();
						// If head node, replace head node with its only child
						if (node.getParent() == null){
							headNode = tmp;
						}
						// Otherwise replace current node with its only child (checking for right or left)
						else if (node.isRight()){
							node.getParent().setRight(tmp);
						}
						else{
							node.getParent().setLeft(tmp);
						}
					}
				}
			}
    	}
    	return headNode;
	}
	public static BinaryTreeNode fastaToTree(File inputFile) throws Exception{
		FileReader freader = null;
		BufferedReader reader = null;
		BinaryTreeNode treeHead = null;
		
		try{
			freader = new FileReader(inputFile);
		    reader = new BufferedReader(freader);
	
			String dataString = "", treeString = "";
			Pattern p = Pattern.compile(" *([AC]+)");
			Matcher m;
			int state = 0;
			
			while ( (dataString=reader.readLine()) != null && state < 2) { 
		        m = p.matcher(dataString);
		        if (m.matches()){
		        	treeString = treeString.concat(m.group(1));
		        	state = 1;
			        m = null;
		        }
		        else if (state == 1){
		        	state = 2;
		        }
		    }
			treeHead = fastaStringToTree(treeString);
		} catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}
		return treeHead;
	}
	public static BinaryTreeNode fastaToTree(String inputFile) throws Exception{
		return fastaToTree(new File(inputFile));
	}

}
