package org.cn3.ns.data;

import java.io.File;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.data.SWCReader;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadataComplete;
import org.cn3.morpho.domain.NeuronProcess;
import org.cn3.morpho.domain.Soma;
import org.cn3.morpho.domain.SwcTreeNode;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Extracts the number of neurons and specific arbors in the dataset given specified conditions
 * 
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.data.DataNumbers -s <swc-dir>
 *
 */
public class DataNumbers {

	private static final String PARAM_SWC_DIR = "swc-dir";

	public static void main(String args[]){
		try{
			System.out.println("Running");
			JSAPResult config = readParameters(args);
			String swcDir = config.getString(PARAM_SWC_DIR);
			//String swcDir = "C:/swcData/";
			String dataset = "NM";

			Map<String,Map<Integer,Integer>> allLikelyMultifurcationCounts = new HashMap<String,Map<Integer,Integer>>();
			allLikelyMultifurcationCounts.put("Axon", new HashMap<Integer,Integer>());
			allLikelyMultifurcationCounts.put("Apical", new HashMap<Integer,Integer>());
			allLikelyMultifurcationCounts.put("Dendrite", new HashMap<Integer,Integer>());
			Map<String,Map<Integer,Integer>> allPossibleMultifurcationCounts = new HashMap<String,Map<Integer,Integer>>();
			allPossibleMultifurcationCounts.put("Axon", new HashMap<Integer,Integer>());
			allPossibleMultifurcationCounts.put("Apical", new HashMap<Integer,Integer>());
			allPossibleMultifurcationCounts.put("Dendrite", new HashMap<Integer,Integer>());

			DistributionBean basalStemsDist = new DistributionBean();
			
			List<Soma<BinaryTreeNode>> somas = convertSwcArchivesToBinaryTreeSomas(
					swcDir+"/"+dataset+"/", 
					allLikelyMultifurcationCounts, 
					allPossibleMultifurcationCounts,
					basalStemsDist);
			NeuronMetadataComplete neuronMetadata;
			Set<String> flaggedNeurons = NSDataReader.getFlaggedNeurons();
			
			for (String arborType : allLikelyMultifurcationCounts.keySet()){
				Map<Integer,Integer> counts = allLikelyMultifurcationCounts.get(arborType);
				System.out.println(arborType+" Likely Multifurcations");
				for (int mSize : counts.keySet()){
					System.out.println("Size "+mSize+": "+counts.get(mSize));
				}
			}

			for (String arborType : allPossibleMultifurcationCounts.keySet()){
				Map<Integer,Integer> counts = allPossibleMultifurcationCounts.get(arborType);
				System.out.println(arborType+" Possible Multifurcations");
				for (int mSize : counts.keySet()){
					System.out.println("Size "+mSize+": "+counts.get(mSize));
				}
			}

			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(1);
			System.out.println("Non-apical stems. Mean: "+nf.format(basalStemsDist.getMean())+"; SD: "+nf.format(basalStemsDist.getStandardDeviation())+"; Median: "+nf.format(basalStemsDist.getMedian()));

			System.out.println("Total number of somas: "+somas.size());
			
			int acuteCulture = 0, nonControl = 0, duplicates = 0,
				dendrites = 0, apicals = 0, axons = 0, dends20Up = 0, apicals20Up = 0, axons20Up = 0;
			List<Soma<BinaryTreeNode>> remaining = new ArrayList<Soma<BinaryTreeNode>>();  
			for (Soma<BinaryTreeNode> soma : somas){
				neuronMetadata = (NeuronMetadataComplete)soma.getNeuron();
				if (neuronMetadata.getArchive().equalsIgnoreCase("ascoli") && neuronMetadata.getProtocol().equalsIgnoreCase("Culture")){
					acuteCulture++;
				} else if (!neuronMetadata.getExperimentCondition().equalsIgnoreCase("Control")){
					nonControl++;
				} else if (flaggedNeurons.contains(neuronMetadata.getName())){
					duplicates++;
				} else {
					remaining.add(soma);
					if (!flaggedNeurons.contains(neuronMetadata.getName()+"-ApicalDendrite") && soma.hasApicalDendriteTree()){
						apicals++;
						if (soma.getApicalDendriteTree().getTreeHead().getDegree()-1 >= 20){
							apicals20Up++;
						}
					}
					if (!flaggedNeurons.contains(neuronMetadata.getName()+"-BasalDendrite") && soma.hasBasalDendriteTree()){
						dendrites++;
						if (soma.getBasalDendriteTree().getTreeHead().getDegree()-1 >= 20){
							dends20Up++;
						}
					}
					if (!flaggedNeurons.contains(neuronMetadata.getName()+"-Axon") && soma.hasAxonalTree()){
						axons++;
						if (soma.getAxonalTree().getTreeHead().getDegree()-1 >= 20){
							axons20Up++;
						}
					}
					
				}
			}

			System.out.println("Cultures: "+acuteCulture);
			System.out.println("NonControl: "+nonControl);
			System.out.println("Duplicates: "+duplicates);
			System.out.println("Remaining: "+remaining.size());
			
			System.out.println("Axons: "+axons);
			System.out.println("Axons 20up: "+axons20Up);
			System.out.println("Dendrites: "+dendrites);
			System.out.println("Dendrites20up: "+dends20Up);
			System.out.println("Apicals: "+apicals);
			System.out.println("Apicals 20up: "+apicals20Up);
			
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	public static List<Soma<BinaryTreeNode>> convertSwcArchivesToBinaryTreeSomas(
			String swcDir, Map<String,Map<Integer,Integer>> allLikelyMultifurcationCounts, 
			Map<String,Map<Integer,Integer>> allPossibleMultifurcationCounts, DistributionBean basalStemsDist)
	throws Exception{
		String classification = "Arborization";
		float spurLength = 2;

		File inputFile;
		
		// Load metadata of neurons/files
		Map<String,NeuronMetadataComplete> metaData = MetaDataManager.getExtendedCellMetaData();
		//System.out.println("Metadata size: "+metaData.size());
		NeuronMetadataComplete neuronMeta = null;
		Soma<SwcTreeNode> swcSoma;
		Soma<BinaryTreeNode> soma;
		String currentNeuronName;
		Pattern namePattern = Pattern.compile("(.*?)(\\.AutoOnly)?(\\.CNG)?\\.swc");
		Matcher nameMatcher;
		
		Map<String,Map<Integer,Integer>> likelyMultifurcationCounts, possibleMultifurcationCounts;
		
		// Directory of SWC files to load
		File listFile = new File(swcDir);
		if (!listFile.exists() || !listFile.isDirectory()){
			throw new Exception("swc directory parameter (--swc-dir/-s) does not exist or is not a directory! Value: "+listFile.getAbsolutePath());
		}
		File archiveDir, directoryOfFiles;
		File[] directories = listFile.listFiles();
		File[] fileList;
		
		List<Soma<BinaryTreeNode>> somaList = new ArrayList<Soma<BinaryTreeNode>>();
	    // Loops through directories of swc file contributors
		System.out.println("Directories: "+directories.length);
		for (int i = 0; i < directories.length; i++){
			// Makes sure we are looking at a directory
			if (directories[i].isDirectory()){
				archiveDir = directories[i];

				// Gets directory containing the swc files
				directoryOfFiles = new File(archiveDir.getAbsolutePath()+"/CNG version/");
				//System.out.println(directoryOfFiles + " " + directoryOfFiles.exists());
				if (!directoryOfFiles.exists()){
					directoryOfFiles = new File(archiveDir.getAbsolutePath()+"/CNGversion");
				}
				if (!directoryOfFiles.exists()){
					directoryOfFiles = new File(archiveDir.getAbsolutePath());
				}
				if (directoryOfFiles.exists()){
					fileList = directoryOfFiles.listFiles();

					// Runs through all files for a given contributor
					System.out.println("Dir "+archiveDir.getName()+" with "+fileList.length+" files");
					for (int j = 0; j < fileList.length; j++){
						inputFile = fileList[j];
						if (inputFile.isFile()){
							
							// pull neuron metadata
							nameMatcher = namePattern.matcher(inputFile.getName());
							if (!nameMatcher.matches()){
								System.out.println("ERROR: Name pattern mismatch: "+inputFile.getName()+" (from "+archiveDir.getName()+")");
							}
							else{
								currentNeuronName = nameMatcher.group(1);
								neuronMeta = metaData.get(currentNeuronName);
								//System.out.println(currentNeuronName);
								if (neuronMeta != null){
									neuronMeta.setLabName(archiveDir.getName());
									// Convert SWC data to binary trees for axon and dendrites
									swcSoma = SWCReader.convertSwcToSoma(inputFile);
									likelyMultifurcationCounts = countLikelyMultifurcations(swcSoma);
									incrementMultifurcationCounts(allLikelyMultifurcationCounts, likelyMultifurcationCounts);
									possibleMultifurcationCounts = countPossibleMultifurcations(swcSoma);
									incrementMultifurcationCounts(allPossibleMultifurcationCounts, possibleMultifurcationCounts);
									
									countStems(swcSoma, basalStemsDist);
									
									for (String arborType : likelyMultifurcationCounts.keySet()){
										Map<Integer,Integer> neuronCountsForType = likelyMultifurcationCounts.get(arborType);
										for (int mSize : neuronCountsForType.keySet()){
											if (mSize > 2 && mSize < 5){
												System.out.println("Likely Multifurcation with "+mSize+" children in "+currentNeuronName+"-"+arborType);
											}
										}
									}
									 
									
									soma = convertSwcToBinarySoma(swcSoma);
									soma.setNeuron(neuronMeta);

									if (soma.hasBasalDendriteTree() && soma.getBasalDendriteTree().getTreeHead().getDegree() > 10){
										soma.getBasalDendriteTree().getTreeHead().removeSpurs(spurLength);
									}
									
									somaList.add(soma);
								}
								else{
									System.out.println("Neuron "+currentNeuronName+" not found in metadata file");
								}
							}
						}
					}
				}
			}
		}
		return somaList;
	}
	
	public static Soma<BinaryTreeNode> convertSwcToBinarySoma(
			Soma<SwcTreeNode> swcSoma) throws Exception{
		
		Soma<BinaryTreeNode> binarySoma = new Soma<BinaryTreeNode>();
		binarySoma.setNumDendriteStems(swcSoma.getNumDendriteStems());
		
		boolean calcPathDist = true;
		int minSize = 0;
		
		// 2. Removes continuations.
		// 3. Builds basal dendrite arborization and apical dendrite arborization
		// 4. Transforms each into binary tree since all tree info is now known (provides stability in transformation).
		// 5. Changes data to structure data (A - termination, C - bifurcation)
		List<SwcTreeNode> apicalTrees = swcSoma.getApicalDendriteTrees(), basalTrees = swcSoma.getBasalDendriteTrees();
		BinaryTreeNode neurite, binaryTree;
		
		if (swcSoma.getAxonalTree() != null){
			SwcTreeNode axon = swcSoma.getAxonalTree().getTreeHead();
			axon = SWCReader.removeContinuations(axon, calcPathDist);
			SwcTreeNode.updateStatisticsDown(axon);
			binaryTree = SWCReader.convertTreeToBinary(axon);
			if (binaryTree.getTreeSize() >= minSize){
				binarySoma.setAxonalTree(new NeuronProcess<BinaryTreeNode>());
				binarySoma.getAxonalTree().setTreeHead(binaryTree);
			}
		}
		if (apicalTrees != null && apicalTrees.size() > 0){
			neurite = SWCReader.assembleProcesses(apicalTrees, calcPathDist);
			if (neurite.getTreeSize() >= minSize){
				binarySoma.setApicalDendriteTree(new NeuronProcess<BinaryTreeNode>());
				binarySoma.getApicalDendriteTree().setTreeHead(neurite);
			}
		}
		if (basalTrees != null && basalTrees.size() > 0){
			neurite = SWCReader.assembleProcesses(basalTrees, calcPathDist);
			if (neurite.getTreeSize() >= minSize){
				binarySoma.setBasalDendriteTree(new NeuronProcess<BinaryTreeNode>());
				binarySoma.getBasalDendriteTree().setTreeHead(neurite);
			}
		}
		
		return binarySoma;
	}


	private static void countStems(Soma<SwcTreeNode> soma, DistributionBean stemsDist) throws Exception{
		if (soma.hasBasalDendriteTrees()){
			stemsDist.addData(soma.getBasalDendriteTrees().size());
		}
	}

	/**
	 * 
	 * @param soma
	 * @param classification
	 * @param calcPathDist
	 * @return Map keyed by arbor type containing map keyed by number of multifurcation children with value of frequency of occurrence
	 * @throws Exception
	 */
	private static Map<String,Map<Integer,Integer>> countLikelyMultifurcations(Soma<SwcTreeNode> soma) throws Exception{
		Map<String,Map<Integer,Integer>> arborMultifurcationCounts = new HashMap<String,Map<Integer,Integer>>();
		Map<Integer,Integer> treeMultifurcationCounts;
		arborMultifurcationCounts.put("Axon", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Apical", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Dendrite", new HashMap<Integer,Integer>());
		
		//List<SwcTreeNode> arborTrees;
		
		if (soma.hasApicalDendriteTrees()){
			//arborTrees = new ArrayList<SwcTreeNode>();
			for (SwcTreeNode tree : soma.getApicalDendriteTrees()){
				//tree = SWCReader.removeContinuations(tree, true);
				//arborTrees.add(tree);
				SWCReader.calculatePathLength(tree);
				treeMultifurcationCounts = countLikelyMultifurcations(tree);
				incrementMultifurcationCounts("Apical", arborMultifurcationCounts, treeMultifurcationCounts);
			}
			//soma.setApicalDendriteTrees(arborTrees);
		}
		if (soma.hasBasalDendriteTrees()){
			//arborTrees = new ArrayList<SwcTreeNode>();
			for (SwcTreeNode tree : soma.getBasalDendriteTrees()){
				//tree = SWCReader.removeContinuations(tree, true);
				//arborTrees.add(tree);
				SWCReader.calculatePathLength(tree);
				treeMultifurcationCounts = countLikelyMultifurcations(tree);
				incrementMultifurcationCounts("Dendrite", arborMultifurcationCounts, treeMultifurcationCounts);
			}
			//soma.setBasalDendriteTrees(arborTrees);
		}
		if (soma.hasAxonalTree()){
			//soma.getAxonalTree().setTreeHead(SWCReader.removeContinuations(soma.getAxonalTree().getTreeHead(),true));
			SWCReader.calculatePathLength(soma.getAxonalTree().getTreeHead());
			treeMultifurcationCounts = countLikelyMultifurcations(soma.getAxonalTree().getTreeHead());
			incrementMultifurcationCounts("Axon", arborMultifurcationCounts, treeMultifurcationCounts);
		}
		return arborMultifurcationCounts;
	}
	/**
	 * 
	 * @param soma
	 * @param classification
	 * @param calcPathDist
	 * @return Map keyed by arbor type containing map keyed by number of multifurcation children with value of frequency of occurrence
	 * @throws Exception
	 */
	private static Map<String,Map<Integer,Integer>> countPossibleMultifurcations(Soma<SwcTreeNode> soma) throws Exception{
		Map<String,Map<Integer,Integer>> arborMultifurcationCounts = new HashMap<String,Map<Integer,Integer>>();
		Map<Integer,Integer> treeMultifurcationCounts;
		arborMultifurcationCounts.put("Axon", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Apical", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Dendrite", new HashMap<Integer,Integer>());
		
		//List<SwcTreeNode> arborTrees;
		
		if (soma.hasApicalDendriteTrees()){
			//arborTrees = new ArrayList<SwcTreeNode>();
			for (SwcTreeNode tree : soma.getApicalDendriteTrees()){
				//tree = SWCReader.removeContinuations(tree, true);
				//arborTrees.add(tree);
				SWCReader.calculatePathLength(tree);
				treeMultifurcationCounts = countPossibleMultifurcations(tree);
				incrementMultifurcationCounts("Apical", arborMultifurcationCounts, treeMultifurcationCounts);
			}
			//soma.setApicalDendriteTrees(arborTrees);
		}
		if (soma.hasBasalDendriteTrees()){
			//arborTrees = new ArrayList<SwcTreeNode>();
			for (SwcTreeNode tree : soma.getBasalDendriteTrees()){
				//tree = SWCReader.removeContinuations(tree, true);
				//arborTrees.add(tree);
				SWCReader.calculatePathLength(tree);
				treeMultifurcationCounts = countPossibleMultifurcations(tree);
				incrementMultifurcationCounts("Dendrite", arborMultifurcationCounts, treeMultifurcationCounts);
			}
			//soma.setBasalDendriteTrees(arborTrees);
		}
		if (soma.hasAxonalTree()){
			//soma.getAxonalTree().setTreeHead(SWCReader.removeContinuations(soma.getAxonalTree().getTreeHead(),true));
			SWCReader.calculatePathLength(soma.getAxonalTree().getTreeHead());
			treeMultifurcationCounts = countPossibleMultifurcations(soma.getAxonalTree().getTreeHead());
			incrementMultifurcationCounts("Axon", arborMultifurcationCounts, treeMultifurcationCounts);
		}
		return arborMultifurcationCounts;
	}
	
	private static void incrementMultifurcationCounts(Map<String,Map<Integer,Integer>> arborCounts, Map<String,Map<Integer,Integer>> neuronCounts){
		Map<Integer,Integer> allCountsForType, neuronCountsForType;
		for (String arborType : neuronCounts.keySet()){
			allCountsForType = arborCounts.get(arborType);
			neuronCountsForType = neuronCounts.get(arborType);
			for (int mSize : neuronCountsForType.keySet()){
				if (!allCountsForType.containsKey(mSize)){
					allCountsForType.put(mSize, neuronCountsForType.get(mSize));
				}else{
					allCountsForType.put(mSize, allCountsForType.get(mSize)+neuronCountsForType.get(mSize));
				}
			}
		}
	}
	private static void incrementMultifurcationCounts(String arborType, Map<String,Map<Integer,Integer>> arborCounts, Map<Integer,Integer> treeCounts){
		Map<Integer,Integer> allCounts = arborCounts.get(arborType);
		for (int mSize : treeCounts.keySet()){
			if (!allCounts.containsKey(mSize)){
				allCounts.put(mSize, treeCounts.get(mSize));
			}else{
				allCounts.put(mSize, allCounts.get(mSize)+treeCounts.get(mSize));
			}
		}

	}

	/**
	 * 
	 * @param inputFile
	 * @param classification
	 * @param calcPathDist
	 * @return Map keyed by arbor type containing map keyed by number of multifurcation children with value of frequency of occurrence
	 * @throws Exception
	 *
	private static Map<String,Map<Integer,Integer>> countMultifurcations(File inputFile) throws Exception{
		Map<String,Map<Integer,Integer>> arborMultifurcationCounts = new HashMap<String,Map<Integer,Integer>>();
		Map<Integer,Integer> allMultifurcationCounts, treeMultifurcationCounts;
		arborMultifurcationCounts.put("Axon", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Apical", new HashMap<Integer,Integer>());
		arborMultifurcationCounts.put("Dendrite", new HashMap<Integer,Integer>());
		List<SwcTreeNode> trees = SWCReader.convertSwcToTreeNodes(inputFile);
		String typeName;
		for (int i = 0; i < trees.size(); i++){
			if (trees.get(i) != null){
				switch(trees.get(i).getType()){
				case SwcDataNode.TYPE_AXON:
					typeName = "Axon";
					break;
				case SwcDataNode.TYPE_APICAL:
					typeName = "Apical";
					break;
				case SwcDataNode.TYPE_DENDRITE:
					typeName = "Dendrite";
					break;
				default:
					typeName = null;
				}
				if (typeName != null){
					treeMultifurcationCounts = countMultifurcations(trees.get(i));
					allMultifurcationCounts = arborMultifurcationCounts.get(typeName);
					for (int mSize : treeMultifurcationCounts.keySet()){
						if (!allMultifurcationCounts.containsKey(mSize)){
							allMultifurcationCounts.put(mSize, treeMultifurcationCounts.get(mSize));
						}else{
							allMultifurcationCounts.put(mSize, allMultifurcationCounts.get(mSize)+treeMultifurcationCounts.get(mSize));
						}
					}
				}
			}
		}
		return arborMultifurcationCounts;
	}
	*/
	private static Map<Integer,Integer> countPossibleMultifurcations(SwcTreeNode tree){
		Map<Integer,Integer> multifurcationCounts = new HashMap<Integer,Integer>();
		Stack<SwcTreeNode> stack = new Stack<SwcTreeNode>();
		SwcTreeNode node, childNode, parentNode;
		stack.add(tree);
		int numChildren;
		while (!stack.isEmpty()){
			node = stack.pop();
			if (node.hasChildren()){
				numChildren = node.getChildren().size();
				parentNode = node.getParentNode(); // Get the next bi- or multifurcation node ancestor to current node
				// Only try if this is a bi- or multifurcation and the branch order is at least 2
				if (numChildren > 1 && parentNode != null){
					// If this node is within 2 um of one or both of its child nodes, and its child nodes are bifurcations,
					// count that as a multifurcation
					for (SwcTreeNode child : node.getChildren()){
						childNode = child.getChildNode();
						if (childNode.hasChildren() && childNode.getSecondaryData().getPathLength() < getMinChildDiameter(childNode.getChildren())){
							numChildren += childNode.getChildren().size()-1;
							stack.addAll(childNode.getChildren());
						}else{
							stack.add(childNode);
						}
					}
					
					if (!multifurcationCounts.containsKey(numChildren)){
						multifurcationCounts.put(numChildren,1);
					}else{
						multifurcationCounts.put(numChildren, multifurcationCounts.get(numChildren)+1);
					}
					
					if (numChildren > 2){
						System.out.println("Possible Multifurcation of "+numChildren+" at "+node.getData().positionString());
					}
				}else{
					stack.addAll(node.getChildren());
				}
			}
		}
		return multifurcationCounts;
	}

	private static Map<Integer,Integer> countLikelyMultifurcations(SwcTreeNode tree){
		Map<Integer,Integer> multifurcationCounts = new HashMap<Integer,Integer>();
		Stack<SwcTreeNode> stack = new Stack<SwcTreeNode>();
		SwcTreeNode node, parentNode;
		stack.add(tree);
		int numChildren;
		while (!stack.isEmpty()){
			node = stack.pop();
			if (node.hasChildren()){
				numChildren = node.getChildren().size();
				parentNode = node.getParentNode(); // Get the next bi- or multifurcation node ancestor to current node
				// Only try if this is a bi- or multifurcation and the branch order is at least 2
				if (numChildren > 1 && parentNode != null){
					// If this node is within 2 um of one or both of its child nodes, and its child nodes are bifurcations,
					// count that as a multifurcation
					for (SwcTreeNode child : node.getChildren()){
						if (child.hasChildren() && child.getChildren().size() > 1 && child.getSecondaryData().getPathLength() < getMinChildDiameter(child.getChildren())){
							numChildren += child.getChildren().size()-1;
							stack.addAll(child.getChildren());
						}else{
							stack.add(child.getChildNode());
						}
					}
					
					if (!multifurcationCounts.containsKey(numChildren)){
						multifurcationCounts.put(numChildren,1);
					}else{
						multifurcationCounts.put(numChildren, multifurcationCounts.get(numChildren)+1);
					}
					if (numChildren > 2){
						System.out.println("LikelyMultifurcation of "+numChildren+" at "+node.getData().positionString());
					}
				}else{
					stack.addAll(node.getChildren());
				}
			}
		}
		return multifurcationCounts;
	}
	
	private static double getMinChildDiameter(List<SwcTreeNode> children){
		double minDiameter = -1;
		for (SwcTreeNode node : children){
			if (minDiameter == -1 || node.getDiameter() < minDiameter){
				minDiameter = node.getDiameter();
			}
		}
		return minDiameter;
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_SWC_DIR)
    		.setStringParser(JSAP.STRING_PARSER)
    		.setRequired(true) 
    		.setShortFlag('s') 
    		.setLongFlag(PARAM_SWC_DIR);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + SwcToFastaConverter.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
