package org.cn3.ns.generation;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadataComplete;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;
/**
 * 
 * @author gillette
 * Takes NB results for a given mapping/baseline/score params, puts them in vs matrix format
 * 
 * java org.cn3.ns.generation.GenerateMetaDataCSV -o <output-dir> -d <dataset> -E <encoding>
 *
 */
public class GenerateMetaDataCSV {

	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String dataset = config.getString(NSConstants.PARAM_DATASET);
			String outputDir = config.getString("outputDir");
			//String dataset = "NM_SpurLen2D";
			int mapping = config.getInt(NSConstants.PARAM_MAPPING_MODE);
			
			List<String> arborizations = NSDataReader.loadSequenceListFromFastaFile(mapping, dataset, false);
			Map<String,NeuronMetadataComplete> metaData = MetaDataManager.getExtendedCellMetaData();
			Map<String,Integer> lenMap = NSDataReader.loadLengthDatabaseFromFastaFile(mapping, dataset);
			String querySequence;
			String output = outputDir+"/arborMetaData-"+dataset+"-MapType"+NSConstants.mappingModeSequenceType[mapping]+".csv";
			NeuronMetadataComplete neuron;
			int i;
			
			StringBuffer sb = new StringBuffer();
			// col labels
			//sb.append("Sequence,ArborType,CellType,BrainRegion,Animal,Archive,SeqLen,Region2,Region3,Class1,Class2,Class3,Gender,Protocol,Strain\n");
			sb.append("Sequence,ArborType,BrainRegion,Species,Archive,SeqLen,Region2,Region3,Class1,Class2,Class3,Class3b,Class3c,Gender,Protocol,AgeClass,Strain,ExperCond\n");
			DataWriter.writeStringToFile(output, sb.toString());
			
			// Last result set is irrelevent
			Set<String> used = new TreeSet<String>();
			for (i = 0; i < arborizations.size(); i++){
				querySequence = arborizations.get(i);
				neuron = metaData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(querySequence));
				if (used.contains(querySequence)){
					System.out.println(i+" " +querySequence);
				}
				if (neuron != null){
					sb = new StringBuffer();
					sb.append(querySequence);
					sb.append(","+NeuriteSequenceStringUtils.getArborizationType(querySequence));
					//sb.append(","+ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
					sb.append(","+neuron.getBrainRegion());
					sb.append(","+neuron.getAnimal());
					sb.append(","+neuron.getLabName());
					sb.append(","+lenMap.get(querySequence));
					sb.append(",\""+neuron.getRegion2()+"\"");
					sb.append(",\""+neuron.getRegion3()+"\"");
					sb.append(","+neuron.getClass1());
					sb.append(","+neuron.getClass2());
					sb.append(","+neuron.getClass3());
					sb.append(","+neuron.getClass3B());
					sb.append(","+neuron.getClass3C());
					sb.append(","+neuron.getGender());
					sb.append(","+neuron.getProtocol());
					sb.append(","+neuron.getAgeClassification());
					sb.append(","+neuron.getStrain());
					sb.append(","+neuron.getExperimentCondition());
					sb.append("\n");
					DataWriter.appendStringToFile(output, sb.toString());
					used.add(querySequence);
				}
			}
			System.out.println("Done generating meta data csv file: "+output);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption("outputDir")
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(false)
		.setDefault("C:/")
		.setShortFlag('o') 
		.setLongFlag("outputDir");
        jsap.registerParameter(opt);

        opt = new FlaggedOption(NSConstants.PARAM_DATASET)
			.setStringParser(JSAP.STRING_PARSER)
			.setRequired(false)
			//.setDefault(""+NSConstants.DEFAULT_DATASET)
			.setDefault("NM_SpurLen2D")
			.setShortFlag('d') 
			.setLongFlag(NSConstants.PARAM_DATASET);
	    jsap.registerParameter(opt);
	    
    	opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.DEFAULT_MAPPING_MODE)
			.setShortFlag('M') 
			.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);
	    
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + GenerateMetaDataCSV.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
