package org.cn3.ns.generation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.data.persistence.NeuronDAO;
import org.cn3.morpho.domain.Measurements;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

public class NMMorphometricsFeatureMatrix {

	public static void main(String args[]){
		try{
			NeuronDAO dao = NeuronDAO.getInstance();
			Measurements measurements;
			Map<String,Measurements> morphometricsMap;
			Map<String,Map<String,Measurements>> morphometricsMapMap = new HashMap<String,Map<String,Measurements>>();
			morphometricsMapMap.put("ApicalDendrite", dao.getNeuronMorphometricsMap(NeuronMetadata.ARBOR_TYPE_APICAL));
			morphometricsMapMap.put("Dendrite", dao.getNeuronMorphometricsMap(NeuronMetadata.ARBOR_TYPE_BASAL));
			//morphometricsMapMap.put("BasalDendrite", morphometricsMapMap.get("Dendrite"));
			morphometricsMapMap.put("Axon", dao.getNeuronMorphometricsMap(NeuronMetadata.ARBOR_TYPE_AXON));
			
			Map<String,NeuronMetadata> metadata = MetaDataManager.getCellMetaDataFromDB();
			NeuronMetadata neuronMetadata;
			
			String dataset = "NM_SpurLen2D";
			
			String fastaFile = DataProperties.getInstance().getFastaFilename(dataset, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			System.out.println("Fasta file: "+fastaFile);
			//List<String> sequenceNames = NSDataReader.loadSequenceListFromFastaFile(fastaFile);
			
			// Load up flagged neurons to ignore
			Set<String> flaggedNeurons = NSDataReader.getFlaggedNeurons();
			
			// Set up both a matrix and a string buffer
			StringBuffer sb = new StringBuffer();
			String sequenceName; //, neuronName, arborType
			//double[][] matrix = new double[sequenceNames.size()][20];
//			double[][] matrix = new double[morphometricsMapMap.get("ApicalDendrite").size()+
//			                               morphometricsMapMap.get("Dendrite").size()+
//			                               morphometricsMapMap.get("Axon").size()][20];
//			
			sb.append("Sequence");
			for (int i = 0; i < Measurements.MEASUREMENT_NAMES.length; i++){
				sb.append(","+Measurements.MEASUREMENT_NAMES[i]);
			}
			sb.append("\n");
			int i = 0, numBifs;
			for (String arborType : morphometricsMapMap.keySet()){
				morphometricsMap = morphometricsMapMap.get(arborType);
				for (String neuronName : morphometricsMap.keySet()){
					neuronMetadata = metadata.get(neuronName);
					measurements = morphometricsMap.get(neuronName);
					numBifs = measurements.getNBifs()+measurements.getNStems();
					if (numBifs > 16 && 
							!flaggedNeurons.contains(neuronName) && 
							!neuronMetadata.getArchive().equalsIgnoreCase("chiang")){
						if (arborType.equals("Dendrite") && neuronMetadata.getCellType().equalsIgnoreCase("Pyramidal cell")){
							sequenceName = neuronName + "-BasalDendrite";
						}else{
							sequenceName = neuronName + "-" + arborType;
						}
						
						if (measurements.getNBifs() > 0){
							sb.append(sequenceName);
							for (int j = 0; j < 20; j++){
								//matrix[i][j] = measurements.getMetricValue(j+1);
								//sb.append(","+matrix[i][j]);
								sb.append(","+measurements.getMetricValue(j+1));
							}
							sb.append("\n");
						}
						i++;
					}
				}
			}
//			for (int i = 0; i < sequenceNames.size(); i++){
//				sequenceName = sequenceNames.get(i);
//				arborType = NeuriteSequenceStringUtils.getArborizationType(sequenceName);
//				neuronName = NeuriteSequenceStringUtils.getNeuronNameFromSequence(sequenceName);
//				//System.out.println(sequenceName);
//				if (!flaggedNeurons.contains(neuronName) && morphometricsMapMap.get(arborType).containsKey(neuronName)){
//					measurements = morphometricsMapMap.get(arborType).get(neuronName);
//					if (measurements.getNBifs() > 0){
//						sb.append(sequenceName);
//						for (int j = 0; j < 20; j++){
//							matrix[i][j] = measurements.getMetricValue(j+1);
//							sb.append(","+matrix[i][j]);
//						}
//						sb.append("\n");
//					}
//				}
//				if (!flaggedNeurons.contains(neuronName) && !morphometricsMapMap.get(arborType).containsKey(neuronName)){
//					System.out.println(neuronName + " not found in database or flagged neuron file");
//				}
//			}
			DataWriter.writeStringToFile("G:/cvGUI Data/NM-Morphometrics.csv",sb.toString());
			DataWriter.writeStringToFile("C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/NM-Morphometrics.csv",sb.toString());
			System.out.println("Wrote to C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/NM-Morphometrics.csv");
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
}
