package org.cn3.ns.motif.analysis;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class MotifSetStdDevs {
	
	private final static String PARAM_MIN_MOTIF_SIZE = "min-motif-size";
	private final static String PARAM_MAX_MOTIF_SIZE = "max-motif-size";
	private final static String PARAM_MOTIF_SIZE = "motif-size";
	private final static String PARAM_BASELINE = "baseline";
	private final static int DEFAULT_MAX_MOTIF_SIZE = 7;
	private final static int DEFAULT_MIN_MOTIF_SIZE = 1;

	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			int mapping = config.getInt(NSConstants.PARAM_MAPPING_MODE);
			
			Map cellData = MetaDataManager.getCellMetaData();
			List arborizations = NSDataReader.loadSequenceList(mapping);
			String arborizationFileBase = 
				DataProperties.getInstance().getFastaDir()+"Arborization/Mapping"
				+config.getInt(NSConstants.PARAM_MAPPING_MODE)+"/Size/";
			String fastaFilename;

			Map currMotifMap, mapOfMotifMaps = new HashMap(), countMap = new HashMap();
			// Initialize mapOfMotifMaps - keys are motifs
			// Each motif has a distribution of frequencies in which it shows up in a sequence
			// The Z-score can be for a given observation, or for the mean of a sub-distribution
			
			NeuronMetadata neuron;
			String arborization, arborizationType, cellTypeArbType;
			FastaReader fr;
			Set motifs = new TreeSet(), cellTypes = new TreeSet(), arbTypes = new TreeSet(), 
				cellTypeArbTypes = new TreeSet(), arbTypeLens = new TreeSet(), 
				randLenBins = new TreeSet();
			Set lengthTypes = new LinkedHashSet(), all = new TreeSet();
			all.add("Arborizations"); all.add("Randoms"); all.add("DBRandoms");
			
			int baseline = config.getInt(PARAM_BASELINE);
			boolean nmBaseline = 
				(baseline == NSConstants.DISTRIBUTION_NEUROMORPHO);
			String randomFileBase = DataProperties.getInstance().getFastaRandomsDir()+"Method"+baseline+
				"/Mapping"+mapping+"/SizeLTS/";
			Map currentPerLength = new HashMap();
			int length, current;
			Integer lenObj;
			
			// Loop arborizations
			for (int i = 0; i < arborizations.size(); i++){
				arborization = (String)arborizations.get(i);
				System.out.println(arborization);
				// Get neuron data for arborization's lab - used to find file; + classification
				neuron = (NeuronMetadata)cellData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
				fastaFilename = arborizationFileBase + neuron.getLabName() 
					+ "/" + arborization + ".lts.fasta";
				// Get sequence
				fr = new FastaReader(fastaFilename);
				fr.next();
				
				// Get motif frequencies
				currMotifMap = MotifFinder.findMotifFrequencies(
						fr.getSequence(),config.getInt(PARAM_MOTIF_SIZE));
				motifs.addAll(currMotifMap.keySet());
				
				// ClassNames & reclassification
				arborizationType = ClassificationUtils.reclassifyArborizationTypes(
						NeuriteSequenceStringUtils.getArborizationType(arborization));
				neuron.setCellType(ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
				cellTypeArbType = neuron.getCellType()+"."+arborizationType;
				
				// Add to class type lists
				cellTypes.add(neuron.getCellType());
				arbTypes.add(arborizationType);
				cellTypeArbTypes.add(cellTypeArbType);
				//lengthTypes.add("ArbLen"+fr.getSequenceLength());
				
				// Add motif map to respective class specific maps
				addToDistributions(mapOfMotifMaps,countMap,"Arborizations",motifs,currMotifMap);
				addToDistributions(mapOfMotifMaps,countMap,neuron.getCellType(),motifs,currMotifMap);
				addToDistributions(mapOfMotifMaps,countMap,arborizationType,motifs,currMotifMap);
				addToDistributions(mapOfMotifMaps,countMap,cellTypeArbType,motifs,currMotifMap);

				if (nmBaseline){
					addToDistributions(mapOfMotifMaps,countMap,"baseline",motifs,currMotifMap);
				}
				else{
					// Repeat above process, but for random sequence (used as baseline)
					length = fr.getSequenceLength();
					lenObj = new Integer(length);
					if (!currentPerLength.containsKey(lenObj)){
						currentPerLength.put(lenObj, new Integer(0));
					}
					current = ((Integer)currentPerLength.get(lenObj)).intValue()+1;
					if (current == 101){
						current = 1;
					}
					currentPerLength.put(lenObj, new Integer(current));
					
					fastaFilename = randomFileBase + "LEN"+length+"NUM"+current+"LTS.fasta";
					// Get sequence
					fr = new FastaReader(fastaFilename);
					fr.next();
					
					currMotifMap = MotifFinder.findMotifFrequencies(
							fr.getSequence(),config.getInt(PARAM_MOTIF_SIZE));
					motifs.addAll(currMotifMap.keySet());
					
					addToDistributions(mapOfMotifMaps,countMap,"baseline",motifs,currMotifMap);
				}

				if (i%10==0){
					System.gc();
				}
			}

			String outputBase = DataProperties.getInstance().getResultsDir()
				+"Motif"+config.getInt(PARAM_MOTIF_SIZE)+"-ZScores-Map"+mapping
				+"-Baseline"+baseline+"-";
			writeDistributions(outputBase+"ArbType.csv",
					arbTypes, mapOfMotifMaps, motifs, countMap, "baseline");
			Set subCellTypeArbTypes;
			for (Iterator it = arbTypes.iterator(); it.hasNext();){
				arborizationType = (String)it.next();
				subCellTypeArbTypes = new TreeSet();
				for (Iterator it2 = cellTypes.iterator(); it2.hasNext();){
					subCellTypeArbTypes.add(it2.next()+"."+arborizationType);
				}
				writeDistributions(outputBase+"CellTypes_in_"+arborizationType+".csv",
						subCellTypeArbTypes, mapOfMotifMaps, motifs, countMap, arborizationType);
			}
/*			writeDistributions(outputBase+"Motif-ZScores-CellType.csv",
					cellTypes, mapOfMotifMaps, motifs, countMap, "baseline");*/
			writeDistributions(outputBase+"CellTypeArbType.csv",
					cellTypeArbTypes, mapOfMotifMaps, motifs, countMap, "baseline");
			/*
			writeDistributions(outputBase+"Motif-ZScores-Length.csv",
					lengthTypes, mapOfMotifMaps, countMap);
			writeDistributions(outputBase+"Motif-ZScores-ArbTypeLength.csv",
					arbTypeLens, mapOfMotifMaps, countMap);*/

/*				writeDistributions(outputBase+"Motif-ZScores-RandLengthBins.csv",
						randLenBins, mapOfMotifMaps, countMap);
			*/
			
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static void addToDistributions(Map masterMap, Map countMap, String className, 
			Set motifs, Map motifCountMap){
		String motif;
		Map classDistMap;
		double freq;
		DistributionBean distBean;
		for (Iterator motifIt = motifs.iterator(); motifIt.hasNext();){
			motif = (String)motifIt.next();
			if (!masterMap.containsKey(motif)){
				masterMap.put(motif, new HashMap());
			}
			classDistMap = (Map)masterMap.get(motif);
			if (!classDistMap.containsKey(className)){
				classDistMap.put(className, new DistributionBean());
			}
			distBean = (DistributionBean)classDistMap.get(className);
			if (motifCountMap.containsKey(motif)){
				freq = ((Number)motifCountMap.get(motif)).doubleValue();
				distBean.addData(freq);
			}
			else{
				distBean.addData((double)0);
			}
		}
		if (countMap.containsKey(className+"Count")){
			countMap.put(className+"Count", 
				new Integer(((Integer)countMap.get(className+"Count")).intValue()+1));
		}
		else{	
			countMap.put(className+"Count", new Integer(1));
		}
	}
	
	private static void writeDistributions(
			String filename, Set classNamesSet, Map mapOfMotifMaps, 
			Set motifs, Map countMap, String baselineClass)
	throws Exception{
		System.out.println("Baseline: "+baselineClass);
		DistributionBean distBeanBaseline, distBeanClass;
		double zScore;
		boolean usedMotif;
		Map currMotifMap;
		String motif, className;
		StringBuffer sb = new StringBuffer(), line;
		Iterator it1, it2;
		sb.append("Motif");
		for (it1 = classNamesSet.iterator(); it1.hasNext();){
			className = (String)it1.next();
			sb.append(","+className+"("+countMap.get(className+"Count")+")");
		}
		sb.append("\r\n");
		
		for (it1 = motifs.iterator(); it1.hasNext();){
			motif = (String)it1.next();
			System.out.println(motif);
			currMotifMap = (Map)mapOfMotifMaps.get(motif);
			usedMotif = false;

			line = new StringBuffer();
			line.append(motif);
			
			distBeanBaseline = (DistributionBean)currMotifMap.get(baselineClass);
			
			for (it2 = classNamesSet.iterator(); it2.hasNext();){
				className = (String)it2.next();
				if (currMotifMap.containsKey(className)){
					distBeanClass = (DistributionBean)currMotifMap.get(className);
					if (distBeanBaseline.getCount() > 1){
						zScore = (distBeanClass.getMean() - distBeanBaseline.getMean())
							/ distBeanBaseline.getStandardDeviation();
						line.append(","+zScore);
					}
					else{
						
					}
					usedMotif = true;
				}
				else{
					line.append(",0");
				}
			}
			if (usedMotif){
				sb.append(line+"\r\n");
			}
		}

		DataWriter.writeStringToFile(filename, sb.toString());
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.MAPPING_MODE_TOPO_ACT)
			.setShortFlag('M') 
			.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MOTIF_SIZE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(true)
			//.setDefault(""+DEFAULT_MOTIF_SIZE)
			.setShortFlag('S') 
			.setLongFlag(PARAM_MOTIF_SIZE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MAX_MOTIF_SIZE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+DEFAULT_MAX_MOTIF_SIZE)
			.setShortFlag('x') 
			.setLongFlag(PARAM_MAX_MOTIF_SIZE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MIN_MOTIF_SIZE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+DEFAULT_MIN_MOTIF_SIZE)
			.setShortFlag('n') 
			.setLongFlag(PARAM_MIN_MOTIF_SIZE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_BASELINE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.DISTRIBUTION_NEUROMORPHO)
			.setShortFlag('B') 
			.setLongFlag(PARAM_BASELINE);
        jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + MotifMonitor.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}
	
}
