package org.cn3.ns.motif.generation;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.DataFormatException;

import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.data.persistence.GACTCompressor;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class ConstraintsDataReader {

	/**
	 * 
	 * @param keyById
	 * @param order
	 * @param method
	 * @param sets
	 * @return Map[k,Map[sequence name or id,List(surrogates)]]
	 * @throws Exception
	 **/
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			String dataset, String order, int method, int numSurrogates)
	throws DataFormatException, IOException{
		return loadSurrogates(-1, dataset, order, method, numSurrogates, false);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			int setNum, String dataset, String order, int method, int numSurrogates)
	throws DataFormatException, IOException{
		//System.out.println(MotifConstants.getSurrogatesFilename(setNum, dataset, order, method, numSurrogates));
		return loadSurrogates(
				MotifConstants.getSurrogatesFilename(setNum, dataset, order, method, numSurrogates), 
				numSurrogates, false);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			int setNum, String dataset, String order, int method, int numSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		String surrogatesFilename = MotifConstants.getSurrogatesFilename(setNum, dataset, order, method, numSurrogates);
		System.out.println("Loading surrogates from "+surrogatesFilename);
		return loadSurrogates(surrogatesFilename, numSurrogates, compressed);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			int setNum, String order, int method, int numSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		return loadSurrogates(setNum, NSConstants.DEFAULT_DATASET, order, method, numSurrogates, 0, compressed);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			int setNum, String order, int method, int numSurrogates, int sampledSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		return loadSurrogates(setNum, NSConstants.DEFAULT_DATASET, order, method, numSurrogates, sampledSurrogates, compressed);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			int setNum, String dataset, String order, int method, int numSurrogates, int sampledSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		return loadSurrogates(
				MotifConstants.getSurrogatesFilename(setNum, dataset, order, method, numSurrogates, sampledSurrogates), 
				numSurrogates, compressed);
	}
	public static Map<Integer,Map<String,List<String>>> loadSurrogates(
			String filename, int numSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		List<String> surrogateList;
		File file = new File(filename);
		System.out.println(filename);
		if (!file.exists()){
			return null;
		}
		Map<String,List<String>> sequenceMap = null;
		int seqId;
		
		Map<Integer,Map<String,List<String>>> kNameSurrogateSeqsMap = new HashMap<Integer,Map<String,List<String>>>();
		List<String> sequenceNames = NSDataReader.loadSequenceList();
		
		FileInputStream fis = null;
		DataInputStream dis = null;
		
		try{
			fis = new FileInputStream(file);
			dis = new DataInputStream(fis);

			String dataString;
			String virtualSequence;
			byte[] compressedSequence;
			Pattern pK = Pattern.compile("K:(\\d)$");
			Pattern p = Pattern.compile("S:(.+)$");
			Matcher m;

			// Read # of sets
			dataString = NSDataReader.readLine(dis,false);
			if (!dataString.equals("NumSurrogates:"+numSurrogates)){
				throw new DataFormatException("First line incorrect: "+dataString+"\nExpect 'NumSurrogates:"+numSurrogates+"'");
			}
			
			int k = -1, count = 0;
			// Read first line (after "Sets:N")
			dataString = NSDataReader.readLine(dis,false);
			do{
				m = pK.matcher(dataString);
				if (m.matches()){
					if (k > -1){
						System.out.println("Done reading k "+k);
					}
					k = Integer.parseInt(m.group(1));
					count = 0;
					if (!kNameSurrogateSeqsMap.containsKey(k)){
						kNameSurrogateSeqsMap.put(k, new HashMap<String,List<String>>());
					}
					sequenceMap = kNameSurrogateSeqsMap.get(k);
				}
				else{
					m = p.matcher(dataString);
					if (m.matches()){
						surrogateList = new ArrayList<String>();
						try{
							seqId = Integer.parseInt(m.group(1));
							sequenceMap.put(sequenceNames.get(seqId), surrogateList);
						}catch(NumberFormatException e){
							// Sequence identifier is a string, must be the sequence name
							sequenceMap.put(m.group(1), surrogateList);
						}
						
						count++;
						//System.out.println(sequenceNames.get(kKey.intValue())+" K"+k+"; Count: "+count);
						if (compressed){
							for (int i = 0; i < numSurrogates; i++){
					        	// Get and decompress virtual sequence
					        	compressedSequence = GACTCompressor.getByteSequence(dis);
					        	virtualSequence = GACTCompressor.decompressSequence(compressedSequence);
					        	surrogateList.add(virtualSequence);
					        	System.out.println(virtualSequence);
							}
							dis.readByte();
						}
						else{
							for (int i = 0; i < numSurrogates; i++){
								virtualSequence = NSDataReader.readLine(dis,false);
								surrogateList.add(virtualSequence);
							}
						}
						//DataReader.readLine(dis,false);
					}
					else{
						throw new DataFormatException("Expecting new sequence, got: "+dataString);
					}
				}
				dataString = NSDataReader.readLine(dis,false);
			} while (dataString != null);

			System.out.println("Done reading "+k+" / all surrogates");
		} catch (IOException ioe){
			throw ioe;
		} finally{
			if (dis != null){
				dis.close();
			}
			if (fis != null){
				fis.close();
			}
		}

		return kNameSurrogateSeqsMap;
	}
	public static Map<Integer,Map<Integer,List<String>>> loadSurrogatesIdKeyed(
			String order, int method, int numSurrogates)
	throws DataFormatException, IOException{
		return loadSurrogatesIdKeyed(-1, order, method, numSurrogates, false);
	}
	public static Map<Integer,Map<Integer,List<String>>> loadSurrogatesIdKeyed(
			int setNum, String order, int method, int numSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		return loadSurrogatesIdKeyed(
				MotifConstants.getSurrogatesFilename(setNum, order, method, numSurrogates), 
				numSurrogates, compressed);
	}
	public static Map<Integer,Map<Integer,List<String>>> loadSurrogatesIdKeyed(
			int setNum, String order, int method, int numSurrogates, int sampledSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		return loadSurrogatesIdKeyed(
				MotifConstants.getSurrogatesFilename(setNum, order, method, numSurrogates, sampledSurrogates), 
				numSurrogates, compressed);
	}
	public static Map<Integer,Map<Integer,List<String>>> loadSurrogatesIdKeyed(
			String filename, int numSurrogates, boolean compressed)
	throws DataFormatException, IOException{
		List<String> surrogateList;
		File file = new File(filename);
		System.out.println(filename);
		if (!file.exists()){
			return null;
		}
		Map<Integer,List<String>> sequenceMap = null;
		int seqId;
		
		Map<Integer,Map<Integer,List<String>>> kNameSurrogateSeqsMap = new HashMap<Integer,Map<Integer,List<String>>>();
		
		FileInputStream fis = null;
		DataInputStream dis = null;
		
		try{
			fis = new FileInputStream(file);
			dis = new DataInputStream(fis);

			String dataString;
			String virtualSequence;
			byte[] compressedSequence;
			Pattern pK = Pattern.compile("K:(\\d)$");
			Pattern p = Pattern.compile("S:(.+)$");
			Matcher m;

			// Read # of sets
			dataString = NSDataReader.readLine(dis,false);
			if (!dataString.equals("NumSurrogates:"+numSurrogates)){
				throw new DataFormatException("First line incorrect: "+dataString+"\nExpect 'NumSurrogates:"+numSurrogates+"'");
			}
			
			int k = -1, count = 0;
			// Read first line (after "Sets:N")
			dataString = NSDataReader.readLine(dis,false);
			do{
				m = pK.matcher(dataString);
				if (m.matches()){
					if (k > -1){
						System.out.println("Done reading k "+k);
					}
					k = Integer.parseInt(m.group(1));
					count = 0;
					if (!kNameSurrogateSeqsMap.containsKey(k)){
						kNameSurrogateSeqsMap.put(k, new HashMap<Integer,List<String>>());
					}
					sequenceMap = kNameSurrogateSeqsMap.get(k);
				}
				else{
					m = p.matcher(dataString);
					if (m.matches()){
						surrogateList = new ArrayList<String>();
						try{
							seqId = Integer.parseInt(m.group(1));
							sequenceMap.put(seqId, surrogateList);
						}catch(NumberFormatException e){
							// Sequence identifier is a string, must be the sequence name
							throw new DataFormatException("Given keyById is true, expected sequence identifier to be numeric. Found: "+m.group(1));
						}
						
						count++;
						//System.out.println(sequenceNames.get(kKey.intValue())+" K"+k+"; Count: "+count);
						if (compressed){
							for (int i = 0; i < numSurrogates; i++){
					        	// Get and decompress virtual sequence
					        	compressedSequence = GACTCompressor.getByteSequence(dis);
					        	virtualSequence = GACTCompressor.decompressSequence(compressedSequence);
					        	surrogateList.add(virtualSequence);
					        	System.out.println(virtualSequence);
							}
							dis.readByte();
						}
						else{
							for (int i = 0; i < numSurrogates; i++){
								virtualSequence = NSDataReader.readLine(dis,false);
								surrogateList.add(virtualSequence);
							}
						}
						//DataReader.readLine(dis,false);
					}
					else{
						throw new DataFormatException("Expecting new sequence, got: "+dataString);
					}
				}
				dataString = NSDataReader.readLine(dis,false);
			} while (dataString != null);

			System.out.println("Done reading "+k+" / all surrogates");
		} catch (IOException ioe){
			throw ioe;
		} finally{
			if (dis != null){
				dis.close();
			}
			if (fis != null){
				fis.close();
			}
		}

		return kNameSurrogateSeqsMap;
	}

	public static void main2(String args[]){
		try{
			int setNum = 1;
			int numSurrogates = 40;
			int method = 8;
			String dataset = "NM";
			String order = "LTS";
			
			//String filename = MotifConstants.getSurrogatesFilename(setNum, "LTS", method, numSurrogates);
			Map<Integer,Map<String,List<String>>> kSequenceSurrogateMap = 
				ConstraintsDataReader.loadSurrogates(
					setNum, dataset, order, method, numSurrogates, false);
			Map<String,List<String>> seqMap;
			List<String> surrogates;
			int count;
			for (Integer k : kSequenceSurrogateMap.keySet()){
				System.out.println("K: "+k);
				count = 0;
				seqMap = kSequenceSurrogateMap.get(k);
				for (String seqName : seqMap.keySet()){
					count++;
					if (count % 100 == 0){
						System.out.println(seqName);
						surrogates = seqMap.get(seqName);
						for (int i = 0; i < 20 && i < surrogates.size(); i++){
							System.out.println(surrogates.get(i));
						}
					}
				}
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static final String PARAM_ID = "id";
	private static final String PARAM_SEQUENCE_NAME = "sequence-name";
	private static final String PARAM_SUCCESSES = "successes";
	private static final String PARAM_COUNTS = "counts";
	private static final String PARAM_MAX_K = "max-k";
	
	private static int setNum;
	private static int method;
	private static double threshold;
	private static String order;
	private static int seqThresholdType;
	private static int kmerThresholdType;
	private static int maxK;
	private static boolean successes;
	private static boolean counts;
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			threshold = config.getDouble(MotifConstants.PARAM_THRESHOLD);
			order = config.getString(NSConstants.PARAM_ORDER);
			method = config.getInt(NSConstants.PARAM_RANDOM_METHOD);
			setNum = config.getInt(MotifConstants.PARAM_SET_NUM);
			int id = config.getInt(PARAM_ID);
			String sequenceNameQuery = config.getString(PARAM_SEQUENCE_NAME);
			seqThresholdType = config.getInt(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE);
			kmerThresholdType = config.getInt(MotifConstants.PARAM_KMER_THRESHOLD_TYPE);
			successes = config.getBoolean(PARAM_SUCCESSES);
			counts = config.getBoolean(PARAM_COUNTS);
			maxK = config.getInt(PARAM_MAX_K);
			boolean doubleLineBreak = config.getBoolean(MotifConstants.PARAM_DOUBLE_LINEBREAK);
		
			List<String> sequenceNames = NSDataReader.loadSequenceList();
			
			FileInputStream fis = null;
			DataInputStream dis = null;
			
			File file = new File(
					MotifConstants.getConstrainedTreesFilename(
							order, method, setNum, threshold, kmerThresholdType, seqThresholdType));

			Map<String,?> metaData = MetaDataManager.getCellMetaData();
			if (file.exists()){
				try{
					fis = new FileInputStream(file);
					dis = new DataInputStream(fis);
		
					String dataString, sequenceName, realSequence, virtualSequence;
					Pattern p = Pattern.compile("(\\d+)\\|(\\d)\\-(\\d)");
					Matcher m;
					int idInFile, kmin, kmax;
					int count = 0;
					
					Map<String,Number> realNMerMap, virtualNMerMap;
					DistributionBean err;
					@SuppressWarnings("unchecked")
					List<String>[] nmers = new List[10];
					String nmer;
					NumberFormat nf = NumberFormat.getInstance();
					if (!counts){
						nf.setMaximumFractionDigits(3);
						nf.setMinimumFractionDigits(3);
					}
					
					dataString = NSDataReader.readLine(dis, doubleLineBreak);
					
					while (dataString != null){
						//System.out.println(dataString);
						m = p.matcher(dataString);
				        
				        if (m.matches()){
				        	// Get sequence info
				        	idInFile = Integer.parseInt(m.group(1));
				        	sequenceName = (String)sequenceNames.get(idInFile);
				        	kmin = Integer.parseInt(m.group(2));
				        	kmax = Integer.parseInt(m.group(3));
				        	int totalKmerCount;
				        	
				        	if (maxK == 0 || maxK > kmin){
					        	// Get and decompress virtual sequence
					        	virtualSequence = GACTCompressor.decompressSequence(dis);
					        	// Clear the line
					        	if (doubleLineBreak){
					        		dis.read();
					        		dis.read();
					        	}
				        		
					        	if ((id > -1 && idInFile == id)
					        			|| (!NeuriteSequenceStringUtils.isEmpty(sequenceNameQuery) && sequenceName.equals(sequenceNameQuery))
					        			|| (id == -1 && NeuriteSequenceStringUtils.isEmpty(sequenceNameQuery) && !successes)
					        			|| (successes && (kmax > kmin))){
						        	System.out.println(idInFile+" - "+sequenceName + " ("+ClassificationUtils.getArchiveBySequenceName(sequenceName)+")");
						        	realSequence = NSDataReader.getSequenceByName(sequenceName, metaData);
					        		System.out.println(realSequence);
					        		System.out.println("Virtual Sequence ("+kmin+"-"+kmax+"):");
					        		System.out.println(virtualSequence);

					        		if (maxK > 0){
					        			kmax = Math.min(maxK, kmax);
					        		}

					        		if ((id > -1 && idInFile == id)
						        			|| (!NeuriteSequenceStringUtils.isEmpty(sequenceNameQuery) && sequenceName.equals(sequenceNameQuery))){
					        			if (counts){
						        			realNMerMap = MotifFinder.findMotifs(realSequence, kmin, kmax+1);
						        			virtualNMerMap = MotifFinder.findMotifs(virtualSequence, kmin, kmax+1);
					        			}
					        			else{
						        			realNMerMap = MotifFinder.findMotifFrequencies(realSequence, kmin, kmax+1);
						        			virtualNMerMap = MotifFinder.findMotifFrequencies(virtualSequence, kmin, kmax+1);
					        			}
					        			System.out.println("    Real     Virtual");
					        			for (int i = Math.max(kmin,1); i <= kmax+1; i++){
					        				if (nmers[i] == null){
					        					nmers[i] = NeuriteSequenceStringUtils.generatePossibleACTMotifs(i);
					        				}
					        				err = new DistributionBean();
					        				for (int j = 0; j < nmers[i].size(); j++){
					        					nmer = nmers[i].get(j);
							        			try{
						        					System.out.println(nmer+"  "+nf.format(realNMerMap.get(nmer))+"     "+nf.format(virtualNMerMap.get(nmer)));
						        					if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE){
					        							err.addData(Math.abs(realNMerMap.get(nmer).doubleValue()-virtualNMerMap.get(nmer).doubleValue()));
						        					}
						        					else if (kmerThresholdType == MotifConstants.KMER_THRESHOLD_RELATIVE){
						        						err.addData(Math.abs(((Number)realNMerMap.get(nmer)).doubleValue()-virtualNMerMap.get(nmer).doubleValue())
						        								/ realNMerMap.get(nmer).doubleValue());
						        					}
						        					else {
						        						err.addData(Math.abs(realNMerMap.get(nmer).doubleValue()-virtualNMerMap.get(nmer).doubleValue()));
						        					}
							        			} catch (IllegalArgumentException iae){
							        				System.out.println("iae: "+realNMerMap.get(nmer)+" "+virtualNMerMap.get(nmer));
							        			}
					        				}
					        				if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE){
					        					totalKmerCount = realSequence.length() - i + 1;
						        				if (counts){
					        						System.out.println("Err: "+(err.getTotal()/(2*totalKmerCount)));
					        					}
					        					else{
					        						System.out.println("Err: "+(err.getTotal()/2));
					        					}
					        				}
					        				else{
					        					System.out.println("Err: "+err.getMean());
					        				}
					        			}
					        		}
					        		
					        		System.out.println();
					        	}
				        	}
				        }
				        count++;
				        dataString = NSDataReader.readLine(dis, doubleLineBreak);
					};
					
				} catch (IOException ioe){
					throw new Exception(ioe);
				} finally{
					if (dis != null){
						dis.close();
					}
					if (fis != null){
						fis.close();
					}
				}
			}
			else{
				System.out.println("File "+file.getName()+" does not exist");
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = MotifConstants.getParameters();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_ID)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(true)
	        .setDefault("-1")
	        .setShortFlag('I') 
	        .setLongFlag(PARAM_ID);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_SEQUENCE_NAME)
	        .setStringParser(JSAP.STRING_PARSER)
	        .setRequired(false)
	        .setDefault("")
	        .setShortFlag('N') 
	        .setLongFlag(PARAM_SEQUENCE_NAME);
	    jsap.registerParameter(opt);
	
        opt = new FlaggedOption(PARAM_SUCCESSES)
	        .setStringParser(JSAP.BOOLEAN_PARSER)
	        .setRequired(false)
	        .setDefault("false")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_SUCCESSES);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_COUNTS)
	        .setStringParser(JSAP.BOOLEAN_PARSER)
	        .setRequired(false)
	        .setDefault("false")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_COUNTS);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MAX_K)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(true)
	        .setDefault("0")
	        .setShortFlag('M') 
	        .setLongFlag(PARAM_MAX_K);
	    jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + ConstraintsDataReader.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
