package org.cn3.ns.motif.generation;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigInteger;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.generation.NodeSequenceBinaryTreeGenerator;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappedBinaryTreeUtils;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * Old class for producing trimer constrained surrogates. This method takes too long to be useful.
 *
 */
public class LengthTrimerDistGenerator {

	private static final String PARAM_SET_NUM = "set-num";
	private static final String PARAM_HOUR_CUTOFF = "hour-cutoff";
	private static final String PARAM_STORAGE_DEGREE = "storage-degree";
	private static final String PARAM_METHOD = "method";
	private static final int ARBORS_PER_CLASS = 400;
	private static final int MAX_SEQUENCE_LENGTH = 600;
	private static final int MAX_COUNT_STORAGE_DEGREE = 18;
	private static final int MOTIF_SIZE = 3;
	private static final int HOUR_CUTOFF = 7;
	
	private static Map treeMotifCountMap = new HashMap();
	private static List<String> motifs = NeuriteSequenceStringUtils.generatePossibleACTMotifs(MOTIF_SIZE);
	private static int storageDegree = MAX_COUNT_STORAGE_DEGREE;
	private static int hourCutoff;
	private static int method;
	
	public static void main(String args[]){
		try{
			int degree = 9;
			String order = "LTS";
			BigInteger possibleShapes = MappedBinaryTreeUtils.numShapes(degree);
			for (int i = 0; i < 5; i++){
				BigInteger shapeNum = NodeSequenceBinaryTreeGenerator.randomBigInteger(possibleShapes);
//				shapeNum = new BigInteger("35");
				Map virtualMotifCounts = LengthTrimerDistGenerator.getMotifCounts(degree, shapeNum, order);
				String virtualSequence = NodeSequenceBinaryTreeGenerator.generateHardingTreeMap2Sequence(degree, shapeNum, order, storageDegree);
				System.out.println("\nVirtual Sequence: "+virtualSequence);
				System.out.println(shapeNum);
				int count = 0;
				for (Iterator it = virtualMotifCounts.keySet().iterator(); it.hasNext();){
					String motif = (String)it.next();
					System.out.print(motif +": "+virtualMotifCounts.get(motif)+"; ");
					count++;
					if (count%10==9){
						System.out.println();
					}
				}
				System.out.println();
			}

			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	public static void main2(String args[]){
		try{
			JSAPResult config = readParameters(args);
			storageDegree = config.getInt(PARAM_STORAGE_DEGREE);
			hourCutoff = config.getInt(PARAM_HOUR_CUTOFF);
			method = config.getInt(PARAM_METHOD);
			
			List arborizations = NSDataReader.loadSequenceList(NSConstants.MAPPING_MODE_TOPO_ACT);
			Map chosenArbors = chooseAllArbors(arborizations);

			String arborType;
			for (Iterator it = chosenArbors.keySet().iterator(); it.hasNext();){
				arborType = (String)it.next();
				generatePopulations(arborType, (Set)chosenArbors.get(arborType), config.getInt(PARAM_SET_NUM));
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static Map chooseAllArbors(List arborizations) throws Exception{
		Map chosenArborMap;
		Set outliers = NSDataReader.loadTrimerTooUnlikely(NSConstants.MAPPING_MODE_TOPO_ACT, method);
		Map lengthMap = NSDataReader.loadLengthDatabase(NSConstants.MAPPING_MODE_TOPO_ACT);
		
		// Generate filenames of arbor sets
		DataProperties props = DataProperties.getInstance();
		File chosenArborsFile = new File(props.getResourceDirectory()+"selectedArbors.txt");

		// If the files doesn't exist, generate 
		if (!chosenArborsFile.exists()){
			// Sort arbors for choosing
			Map arborTypesMap = sortArborsByType(arborizations);
			chosenArborMap = new HashMap();
			String arborType;
			Set chosenArbors;
			StringBuffer sb = new StringBuffer();
			for (Iterator it = arborTypesMap.keySet().iterator(); it.hasNext();){
				arborType = (String)it.next();
				// Choosen arbors and save in map
				chosenArbors = chooseArbors(
						((List)arborTypesMap.get(arborType)), ARBORS_PER_CLASS, outliers, lengthMap);
				chosenArborMap.put(arborType, chosenArbors);
				
				// Add arborType and chosen arbors to file data
				sb.append(arborType+"\r\n");
				for (Iterator chosenIt = chosenArbors.iterator(); chosenIt.hasNext();){
					sb.append(chosenIt.next()+"\r\n");
				}
			}
			// Write chosen arbors file
			DataWriter.writeStringToFile(chosenArborsFile, sb.toString());
		}
		else{
			// File has already been made, read it
			chosenArborMap = readChosenArbors(chosenArborsFile);
		}
		return chosenArborMap;
	}
	
	private static Set chooseArbors(List arbors, int number, Set outliers, Map lengthMap){
		Set set = new TreeSet();
		int index;
		String arbor;
		for (int i = 0; i < number; i++){
			do {
				// Generate random number to choose from the list of arbors
				index = (int)(Math.random()*arbors.size());
				arbor = (String)arbors.get(index);

				// Make sure the sequence isn't too long and it won't take way too long to generate
			} while(
					outliers.contains(arbor) || 
					((Integer)lengthMap.get(arbor)).intValue() > MAX_SEQUENCE_LENGTH);
			
			// Add the selected arbor to the destination set, remove it from the source list
			set.add(arbors.remove(index));
		}
		return set;
	}
	
	// Goes through arbor list, sorting into lists by arbor type
	private static Map sortArborsByType(List arborizations){
		Map arborMap = new HashMap();
		arborMap.put("Dendrite", new ArrayList());
		arborMap.put("ApicalDendrite", new ArrayList());
		arborMap.put("Axon", new ArrayList());
		String arbor, type;
		for (int i = 0; i < arborizations.size(); i++){
			arbor = (String)arborizations.get(i);
			type = ClassificationUtils.reclassifyArborizationTypes(
					NeuriteSequenceStringUtils.getArborizationType(arbor));
			((List)arborMap.get(type)).add(arbor);
		}
		return arborMap;
	}
	
	private static Map readChosenArbors(File file) throws Exception{
		Map map = new HashMap();
		map.put("Dendrite", new TreeSet());
		map.put("ApicalDendrite", new TreeSet());
		map.put("Axon", new TreeSet());
		FileReader freader = null;
	    BufferedReader reader = null;

	    try{
		    freader = new FileReader(file);
		    reader = new BufferedReader(freader);

			String dataString = "", currentType = null;
			
			while ( (dataString=reader.readLine()) != null ) { 
		        if (dataString.equals("Dendrite") || dataString.equals("ApicalDendrite") || dataString.equals("Axon")){
		        	currentType = dataString;
		        }
		        else if (currentType == null){
		        	throw new Exception("File does not start with an arbor type");
		        }
		        else{
		        	((Set)map.get(currentType)).add(dataString);
		        }
			}
	    } catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}
		return map;
	}


	private static void generatePopulations(String arborType, Set arborizations, int setNum) throws Exception{
		Map metaData = MetaDataManager.getCellMetaData();
		Set outliers = NSDataReader.loadTrimerTooUnlikely(NSConstants.MAPPING_MODE_TOPO_ACT, method);
		DataProperties props = DataProperties.getInstance();
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		
		// Get files of existing n-mer generated sets to avoid duplicating existing arbor sequences
		String existingRandomsFilename = props.getFastaRandomsDir()+"Method3/Mapping2/3-merDistSet"+setNum+"-"+arborType+".fasta";
		Set existingRandoms = getExistingRandoms(arborType, props, setNum);

		motifs = NeuriteSequenceStringUtils.generatePossibleACTMotifs(MOTIF_SIZE);
		String arborization;
		NeuronMetadata neuron;
		Map arborMotifFreqs, randomMotifFreqs;
		int degree, tryCount, tryCountSample;
		BigInteger shapeNum;
		String generatedSequence;
		long startTime, timeTaken, tooLongTime = hourCutoff * 60 * 60 * 1000; // HOUR_CUTOFF hours in milliseconds 
		boolean tooLong;
		FastaReader fr = new FastaReader();
		
		for (Iterator arborIt = arborizations.iterator(); arborIt.hasNext();){
			arborization = (String)arborIt.next();
			// Only make random of this sequence if it hasn't been generated already (no previously existing file or exhausted list)
			// Make sure this sequence isn't an outlier or would take too long to run (before checking the randFr)
			if (!existingRandoms.contains(arborization) && !outliers.contains(arborization)){
				// Pull sequence info
				neuron = (NeuronMetadata)metaData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
				
				// Get sequence
				fr.setFile(props.getFastaArborDir()+"Mapping2/Size/"+neuron.getLabName()+"/"+arborization+".lts.fasta");
				fr.next();
				
				// Get motif frequencies
				arborMotifFreqs = MotifFinder.findMotifFrequencies(fr.getSequence(), MOTIF_SIZE);

				System.out.println(arborization+" length: "+fr.getSequenceLength()+"; frequencies: "+arborMotifFreqs);
				
				// Find a tree shape that has a %C within +/- 2.5% of the sequence's within a reasonable timeframe
				tooLong = false; tryCount = 0;
				startTime = new Date().getTime();
				degree = fr.getSequenceLength()+1;
				tryCountSample = 100000000 / fr.getSequenceLength();
				do {
					// Generate random tree number and check its %C
					BigInteger possibleShapes = MappedBinaryTreeUtils.numShapes(degree);
					shapeNum = NodeSequenceBinaryTreeGenerator.randomBigInteger(possibleShapes);
					randomMotifFreqs = getMotifFrequencies(degree, shapeNum);
					//System.out.println("trying; attempt is "+BinaryTreeGenerator.generateHardingTreeMap2Sequence(degree, shapeNum));
					//System.out.println("Motif Freqs: "+randomMotifFreqs);
					// Make sure this isn't taking too long
					tryCount++;
					if (tryCount % tryCountSample == 0){
						timeTaken = new Date().getTime() - startTime;
						tooLong = timeTaken > tooLongTime;
						System.out.println("TryCount: " + tryCount+"; Time taken (min): " + nf.format((double)timeTaken/(60000)));
						//System.out.println(randomMotifFreqs);
						System.gc();
					}
				} while (!isWithinRange(arborMotifFreqs, randomMotifFreqs) && !tooLong);
				
				if (tooLong){
					DataWriter.appendStringToFile(
							props.getTrimerTooUnlikelyFile(NSConstants.MAPPING_MODE_TOPO_ACT, method),
							arborization+"\r\n");
					System.out.println("Took too long: "+arborization);
				}
				else{
					timeTaken = new Date().getTime() - startTime;
					DataWriter.appendStringToFile("trimerTimeSet"+setNum+".csv", 
							arborization+","+degree+","+tryCount+","+timeTaken+"\r\n");
					System.out.println("Tree found ("+tryCount+" tries). freqs%: "+randomMotifFreqs);
					generatedSequence = NodeSequenceBinaryTreeGenerator.generateHardingTreeMap2Sequence(degree, shapeNum);
					DataWriter.appendStringToFile(existingRandomsFilename, 
							"> Equiv-"+arborization+"\r\n"+NeuriteSequenceStringUtils.insertBreaksAtLength(generatedSequence, 80)+"\r\n");
				}
			}
		}
		
	}

	private static Set getExistingRandoms(String arborType, DataProperties props, int setNum) throws Exception{
		Set existingRandoms = new TreeSet();
		
		File existingRandomsFile = new File(props.getFastaRandomsDir()+"Method3/Mapping2/3-merDistSet"+setNum+"-"+arborType+".fasta");
		if (existingRandomsFile.exists()){
			FastaReader fr = new FastaReader(existingRandomsFile);
			String name;
			while (fr.next()){
				name = fr.getHeader().substring(8);
				existingRandoms.add(name);
			}
		}
		
		return existingRandoms;
	}

	private static boolean isWithinRange(Map arborMotifFreqs, Map randomMotifFreqs){
		String motif;
		double arborFreq;
		double randomFreq;
		for (int i = 0; i < motifs.size(); i++){
			motif = (String)motifs.get(i);
			arborFreq = arborMotifFreqs.containsKey(motif) ?
				((Double)arborMotifFreqs.get(motif)).doubleValue() : 0;
			randomFreq = randomMotifFreqs.containsKey(motif) ? 
				((Double)randomMotifFreqs.get(motif)).doubleValue() : 0;
			if (Math.abs(arborFreq-randomFreq) > .025) return false;
		}
		return true;
	}
	
	public static Map getMotifFrequencies(int degree, BigInteger shapeNum)
	throws Exception{
		return getMotifFrequencies(degree, shapeNum, true);
	}
	public static Map getMotifFrequencies(int degree, BigInteger shapeNum, String order)
	throws Exception{
		return getMotifFrequencies(degree, shapeNum, order.equals("LTS"));
	}
	public static Map getMotifFrequencies(int degree, BigInteger shapeNum, boolean orderLTS)
	throws Exception{
		int effectiveLength = degree-MOTIF_SIZE;
		Map countMap = getMotifCounts(degree, shapeNum, orderLTS);
		Map frequencyMap = new HashMap();
		String motif;
		int count;
		for (Iterator it = motifs.iterator(); it.hasNext();){
			motif = (String)it.next();
			count = ((Integer)countMap.get(motif)).intValue();
			//System.out.println(motif+" "+count);
			frequencyMap.put(motif, new Double((double)count/effectiveLength));
		}
		
		return frequencyMap;
	}
	
	public static Map<String,Number> getMotifCounts(int degree, BigInteger shapeNum)
	throws Exception{
		return getMotifCounts(degree, shapeNum, "LTS");
	}
	public static Map<String,Number> getMotifCounts(int degree, BigInteger shapeNum, String order)
	throws Exception{
		Map<String,Object> countsHolder = getMotifCounts(degree, shapeNum, order.equals("LTS"));
		Map<String,Number> motifCounts = new HashMap<String,Number>();
		for (String s : countsHolder.keySet()){
			if (countsHolder.get(s).getClass().equals(Integer.class)){
				motifCounts.put(s, (Integer)countsHolder.get(s));
			}
		}
		return motifCounts;
	}
	/**
	 * DON'T RUN THIS TWICE WITH DIFFERENT ORDERS WITHOUT CLEARING MOTIF COUNTS MAP
	 */
	public static Map<String,Object> getMotifCounts(int degree, BigInteger shapeNum, boolean orderLTS)
	throws Exception{
		//System.out.println("degree:" +degree);
		Object storageKey = (orderLTS ? "L" : "S") + degree;
		Map<String,Object> countMap, largerCountMap, smallerCountMap;
		// Resolve simple cases
		if (degree == 2){
			countMap = new TreeMap<String,Object>();
			for (int i = 0; i < motifs.size(); i++){
				countMap.put(motifs.get(i), new Integer(0));
			}
			countMap.put("1", "T");
			countMap.put("e", "T"); // No second to last node, so use the last node of the larger tree which must be a T
		}
		else if (degree == 3){
			countMap = new TreeMap<String,Object>();
			for (int i = 0; i < motifs.size(); i++){
				countMap.put(motifs.get(i), new Integer(0));
			}
			countMap.put("1", "C");
			countMap.put("2", "T");
			countMap.put("e", "C");
		}
		else{
			// Check to see if it's been calculated before
			if (degree <= storageDegree){ // Limits possible storage
				if (treeMotifCountMap.containsKey(storageKey)){
					Map shapeMotifCountMap = (Map)treeMotifCountMap.get(storageKey);
					if (shapeMotifCountMap.containsKey(shapeNum)){
						return (Map)shapeMotifCountMap.get(shapeNum);
					}
				}
				else{
					synchronized(treeMotifCountMap){
						treeMotifCountMap.put(storageKey, 
								new HashMap(MappedBinaryTreeUtils.numShapes(degree).intValue()/4));
					}
				}
			}
			countMap = new TreeMap<String,Object>();
			
			// Find whether the top node of this tree is a C node
			int smallerDegree = 0;
			String motif, first, second, third;
			BigInteger partialSum;
			do{
				smallerDegree++;
				partialSum = MappedBinaryTreeUtils.partialSumOfNumShapes(degree,smallerDegree);
			} while(partialSum.compareTo(shapeNum) < 0);
			if (smallerDegree == 1){
				largerCountMap = getMotifCounts(degree-1, shapeNum, orderLTS);
				first = "C";
				second = (String)largerCountMap.get("1");
				third = (String)largerCountMap.get("2");
				countMap = new TreeMap(largerCountMap);
				incrementCount(first + second + third, countMap);
				// Transfer up end of this tree
				countMap.put("1", "C");
				countMap.put("2", second);
				//countMap.put("e", largerCountMap.get("e"));
			}
			else{
				// Special case, A with only 1 more node in it's "larger" subtree
				if (degree == 4){
					for (int i = 0; i < motifs.size(); i++){
						countMap.put(motifs.get(i), new Integer(0));
					}
					first = "A";
					second = "T";
					countMap.put("1", "A");
					countMap.put("2", "T");
					countMap.put("e", "T");
					incrementCount("ATT", countMap);
				}
				else{
					int largerDegree = degree - smallerDegree;
					BigInteger remainder, numShapesLeft, rightShape, leftShape;
					remainder =
						shapeNum.subtract(MappedBinaryTreeUtils.partialSumOfNumShapes(degree,degree-largerDegree-1));
					if (largerDegree == smallerDegree){
						// Left and Right degrees are the same
						// Swapping right & left for proper traversal output
						numShapesLeft = MappedBinaryTreeUtils.numShapes(smallerDegree);
						leftShape = numShapesLeft;
						rightShape = leftShape.add(BigInteger.ONE);
						BigInteger two = BigInteger.ONE.add(BigInteger.ONE);
						// Find the shape out of n*(n+1)/2 
						leftShape = NodeSequenceBinaryTreeGenerator.findRightShape(remainder,BigInteger.ONE,numShapesLeft);
						rightShape = remainder.subtract(
								leftShape.multiply(leftShape.subtract(BigInteger.ONE))
									.divide(two));
					}
					else{
						numShapesLeft = MappedBinaryTreeUtils.numShapes(smallerDegree);
						rightShape = remainder.subtract(BigInteger.ONE).divide(numShapesLeft).add(BigInteger.ONE);
						leftShape = remainder.subtract(numShapesLeft.multiply(rightShape.subtract(BigInteger.ONE)));
					}
					//System.out.println("LeftDeg:" +leftDegree+"; rightDeg: " +rightDegree);
					largerCountMap = getMotifCounts(largerDegree, rightShape, orderLTS);
					smallerCountMap = getMotifCounts(smallerDegree, leftShape, orderLTS);
					for (int i = 0; i < motifs.size(); i++){
						motif = (String)motifs.get(i);
						countMap.put(motif, 
							new Integer(
									((Integer)largerCountMap.get(motif)).intValue()+
									((Integer)smallerCountMap.get(motif)).intValue()));
					}
					// Increment first & middle
					first = "A";
					// LTS
					if (orderLTS){
						second = (String)largerCountMap.get("1");
						third = (String)largerCountMap.get("2");
						incrementCount(first + second + third, countMap);
						// Transfer up end of this tree
						countMap.put("e", smallerCountMap.get("e"));
						// Deal with end of larger tree
						// Last two characters of the larger tree with first of the smaller tree
						incrementCount(largerCountMap.get("e") + "T" + smallerCountMap.get("1"), countMap);
						// If the smaller tree is more than just a T node, last character of larger tree with first two of smaller tree
						if (smallerDegree > 2){
							incrementCount("T" + smallerCountMap.get("1") + smallerCountMap.get("2"), countMap);
						}
					}
					// STL
					else{
						second = (String)smallerCountMap.get("1");
						// Transfer up end of this tree
						countMap.put("e", largerCountMap.get("e"));
						if (smallerDegree == 2){
							third = (String)largerCountMap.get("1");
							incrementCount(first + second + third, countMap);
							// Last character of smaller tree with first two of larger tree
							incrementCount("T" + largerCountMap.get("1") + largerCountMap.get("2"), countMap);
						}
						else{
							third = (String)smallerCountMap.get("2");
							incrementCount(first + second + third, countMap);
							// Deal with end of smaller tree
							// Last two characters of the smaller tree with first of the larger tree
							incrementCount(smallerCountMap.get("e") + "T" + largerCountMap.get("1"), countMap);
							// Last character of smaller tree with first two of larger tree
							incrementCount("T" + largerCountMap.get("1") + largerCountMap.get("2"), countMap);
						}
					}
				}
			}

			countMap.put("1", first);
			countMap.put("2", second);
	
			// Store new result (if appropriate)
			if (degree <= storageDegree){
				synchronized (treeMotifCountMap) {
					Map shapeMotifCountMap = (Map)treeMotifCountMap.get(storageKey);
					shapeMotifCountMap.put(shapeNum, countMap);
				}
			}
		}
		return countMap;
	}

	private static void incrementCount(String motif, Map countMap){
		int count = countMap.containsKey(motif) 
			? ((Integer)countMap.remove(motif)).intValue() : 0;
			//System.out.println("incrementing "+motif+" count");
		countMap.put(motif, new Integer(count+1));
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_SET_NUM)
            .setStringParser(JSAP.INTEGER_PARSER)
            .setRequired(true)
            .setShortFlag('s') 
            .setLongFlag(PARAM_SET_NUM);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_STORAGE_DEGREE)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+MAX_COUNT_STORAGE_DEGREE)
	        .setShortFlag('d') 
	        .setLongFlag(PARAM_STORAGE_DEGREE);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_HOUR_CUTOFF)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+HOUR_CUTOFF)
	        .setShortFlag('c') 
	        .setLongFlag(PARAM_HOUR_CUTOFF);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_METHOD)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+NSConstants.DEFAULT_BASELINE)
	        .setShortFlag('m') 
	        .setLongFlag(PARAM_METHOD);
	    jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + LengthTrimerDistGenerator.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

	public static void setStorageDegree(int storageDeg){
		storageDegree = storageDeg;
	}
	public static int getStorageDegree(){
		return storageDegree;
	}
	
	public static void clearMap(){
		treeMotifCountMap.clear();
		System.gc();
	}
	
}
