package org.cn3.ns.motif.generation;

import java.io.IOException;

import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;

public class MotifConstants {

	public static final String PARAM_ID = "id";
	public static final String PARAM_SEQUENCE_NAME = "sequence-name";
	public static final String PARAM_THRESHOLD = "threshold";
	public static final String PARAM_SET_NUM = "set-num";
	public static final String PARAM_METHOD = "method";
	public static final String PARAM_THRESHOLD_TYPE = "threshold-type";
	public static final String PARAM_SUCCESSES = "successes";
	public static final String PARAM_COUNTS = "counts";
	public static final String PARAM_DOUBLE_LINEBREAK = "double-break";

	public static final String PARAM_KMER_THRESHOLD_TYPE = "kmer-threshold-type";
	public static final String PARAM_SEQ_THRESHOLD_TYPE = "seq-threshold-type";
	public static final String PARAM_HOUR_CUTOFF = "hour-cutoff";
	public static final String PARAM_STORAGE_DEGREE = "storage-degree";
	public static final String PARAM_MAX_CONSTRAINT = "max-constraint";
	public static final String PARAM_START = "start";
	public static final String PARAM_MAX_PROCESSORS = "max-processors";
	
	public static final int DEFAULT_MAX_CONSTRAINT = 3;

	public static final int KMER_THRESHOLD_NONE = 0;
	public static final int KMER_THRESHOLD_ABSOLUTE = 1;
	public static final int KMER_THRESHOLD_RELATIVE = 2;
	
	public static final int SEQ_THRESHOLD_WHOLE = 0; // With a KMER threshold of "none"
	public static final int SEQ_THRESHOLD_STRICT = 1;
	public static final int SEQ_THRESHOLD_AVG = 2;
	public static final int SEQ_THRESHOLD_CUTOFF = 3;
	public static final int SEQ_THRESHOLD_WHOLE_EFF1 = 4; // With a KMER threshold of "none"
	public static final int SEQ_THRESHOLD_WHOLE_EFF2 = 5;
	
	public static void getParameters(JSAP jsap) throws Exception{
		getParameters(jsap, true);
	}
	public static void getParameters(JSAP jsap, boolean includeSetNum) throws Exception{
		FlaggedOption opt;
		
        opt = new FlaggedOption(PARAM_THRESHOLD)
	        .setStringParser(JSAP.DOUBLE_PARSER)
	        .setDefault("0")
	        .setRequired(false)
	        .setShortFlag('T') 
	        .setLongFlag(PARAM_THRESHOLD);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(NSConstants.PARAM_ORDER)
	        .setStringParser(JSAP.STRING_PARSER)
	        .setDefault(NSConstants.DEFAULT_ORDER)
	        .setRequired(false)
	        .setShortFlag('o') 
	        .setLongFlag(NSConstants.PARAM_ORDER);
	    jsap.registerParameter(opt);

	    if (includeSetNum){
		    opt = new FlaggedOption(PARAM_SET_NUM)
		        .setStringParser(JSAP.INTEGER_PARSER)
		        .setRequired(true)
		        .setShortFlag('s') 
		        .setLongFlag(PARAM_SET_NUM);
		    jsap.registerParameter(opt);
	    }
	    
        opt = new FlaggedOption(PARAM_METHOD)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+NSConstants.DEFAULT_BASELINE)
	        .setShortFlag('m') 
	        .setLongFlag(PARAM_METHOD);
	    jsap.registerParameter(opt);
	
	    opt = new FlaggedOption(PARAM_KMER_THRESHOLD_TYPE)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+KMER_THRESHOLD_ABSOLUTE)
	        .setShortFlag('K') 
	        .setLongFlag(PARAM_KMER_THRESHOLD_TYPE);
	    jsap.registerParameter(opt);
	
	    opt = new FlaggedOption(PARAM_SEQ_THRESHOLD_TYPE)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+SEQ_THRESHOLD_STRICT)
	        .setShortFlag('S') 
	        .setLongFlag(PARAM_SEQ_THRESHOLD_TYPE);
	    jsap.registerParameter(opt);
	    
        opt = new FlaggedOption(PARAM_DOUBLE_LINEBREAK)
	        .setStringParser(JSAP.BOOLEAN_PARSER)
	        .setRequired(false)
	        .setDefault("false")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_DOUBLE_LINEBREAK);
	    jsap.registerParameter(opt);

	}
	
	public static JSAP getParameters() throws Exception{
		JSAP jsap = new JSAP();

		getParameters(jsap,true);

		return jsap;
	}

	public static JSAP getParameters(boolean includeSetNum) throws Exception{
		JSAP jsap = new JSAP();

		getParameters(jsap, includeSetNum);

		return jsap;
	}
	
	public static String getThresholdTypeString(int kmerThresholdType, int seqThresholdType){
		String str;
		if (seqThresholdType == SEQ_THRESHOLD_WHOLE){
			str = "-Whole";
		}
		else if (seqThresholdType == SEQ_THRESHOLD_WHOLE_EFF1){
			str = "-WholeEff1";
		}
		else if (seqThresholdType == SEQ_THRESHOLD_WHOLE_EFF2){
			str = "-WholeEff2";
		}
		else{
			if (kmerThresholdType == KMER_THRESHOLD_RELATIVE)
				str = "-Rel";
			else
				str = "-Abs";
			
			if (seqThresholdType == SEQ_THRESHOLD_AVG)
				str = str + "Avg";
			else if (seqThresholdType == SEQ_THRESHOLD_CUTOFF)
				str = str + "Cut";
			else
				str = str + "Str";
		}
		return str;
	}
	
	public static String getConstrainedTreesFilename(String order, int method, 
			int setNum, double threshold, 
			int kmerThresholdType, int seqThresholdType) throws IOException{
		String filename = "/Method"+method+
		"/Mapping2/constrainedTrees"+order+"-set"+setNum+"-threshold-"+threshold
			+ getThresholdTypeString(kmerThresholdType, seqThresholdType)
			+ ".dat";
		
		//System.out.println(DataProperties.getInstance().getFastaRandomsDir()+filename);
		return DataProperties.getInstance().getFastaRandomsDir()+filename;
	}

	public static String getSurrogatesFilename(
			String order, int method, int sets) throws IOException{
		return getSurrogatesFilename(-1, order, method, sets);
	}
	public static String getSurrogatesFilename(
			int setNum, String order, int method, int sets) throws IOException{
		return getSurrogatesFilename(setNum, order, method, sets, 0);
	}
	public static String getSurrogatesFilename(
			int setNum, String dataset, String order, int method, int sets) throws IOException{
		return getSurrogatesFilename(setNum, dataset, order, method, sets, 0);
	}
	public static String getSurrogatesFilename(
			int setNum, String order, int method, int sets, int sampledSequences) throws IOException{
		return getSurrogatesFilename(setNum, NSConstants.DEFAULT_DATASET, method, sets, sampledSequences);
	}
	public static String getSurrogatesFilename(
			int setNum, String dataset, String order, int method, int sets, int sampledSequences) throws IOException{
		String filename = "/Method"+method+
			"/Mapping2/surrogates"+order+"-"+sets+"sets_B"+method;
		if (sampledSequences > 0){
			filename += "ApproxUniform"+sampledSequences;
		}
		if (setNum > -1){
			filename += "_Set"+setNum+".dat";
		}
		else{
			filename += ".dat";
		}
		
		
		//System.out.println(DataProperties.getInstance().getFastaRandomsDir()+filename);
		return DataProperties.getInstance().getFastaRandomsDir(dataset)+filename;
	}
	
}
