package org.cn3.ns.nodeencoding;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.SwcDataNode;
import org.cn3.morpho.utils.SwcDataUtils;

public class Topology1AndRemoteAngleEncoder extends NodeEncoder {

	// Angles are in radians
	protected static final double[] QUARTILES = {0.77, 1.2, 1.72}; // Degrees: 44.1, 68.8, 98.5
	protected static final double[] QUINTILES = {0.68, 1.03, 1.39, 1.87}; // Degrees: 39, 59, 79.6, 107.1
	//protected static final double[] SEXTILES = {, , , , }; // Degrees: , , , ,  
	protected static final double[] DEFAULT_DIVISIONS = QUINTILES;
	
	private static final Character[] ATypesArray = {'A','H','N','X','W','V','Y'};
	private static final Character[] CTypesArray = {'C','D','Q','O','G','S','U'};
	private static final Character[] TTypesArray = {'T','I','E','F','L','Z','R'};

	private static final List<Character> ATypes = Arrays.asList(ATypesArray);
	private static final List<Character> CTypes = Arrays.asList(CTypesArray);
	private static final List<Character> TTypes = Arrays.asList(TTypesArray);

	private double[] radianDivisions;
	
	public Topology1AndRemoteAngleEncoder(){
		this.radianDivisions = DEFAULT_DIVISIONS;
		setEncodingMode(EncodingMode.TOPO1_REMOTE_ANGLE);
	}
	public Topology1AndRemoteAngleEncoder(double[] radianDivisions){
		this.radianDivisions = radianDivisions;
		setEncodingMode(EncodingMode.TOPO1_REMOTE_ANGLE);
	}
	/* Possible future option: enable user to set number of quantiles, defaulting to 2
	public Topology1AndRemoteAngleEncoder(BinaryTreeNode tree){
		// Calculate median angle
		
	}

	public Topology1AndRemoteAngleEncoder(BinaryTreeNode tree, int numDivisions) throws Exception{
		if (numDivisions < 2){
			throw new Exception("At least 2 divisions required");
		}
		// Calculate quantiles based on numDivisions
		
	}
*/
	public double[] getRadianDivisions(){
		return(radianDivisions.clone());
	}
	public void setRadianDivisions(double[] radianDivisions){
		this.radianDivisions = radianDivisions;
	}
	
	public String encodeNode(BinaryTreeNode node) throws Exception{
		if (node.isLeaf()){
			return "";
		}
		
		double branchAngle;
    	SwcDataNode parentData, leftData, rightData;
    	
    	// Uses a code to enable conversion to character based on several properties
    	int code = 0;

		parentData = (SwcDataNode)node.getData();
		leftData = (SwcDataNode)node.getLeft().getData();
		rightData = (SwcDataNode)node.getRight().getData();
		branchAngle = SwcDataUtils.getAngle(parentData, leftData, rightData);

		if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
			code = 10;
		}
		else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
				||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
			code = 20;
		}
		else{
			code = 30;
		}

		int i = 1;
		while (i <= radianDivisions.length && branchAngle > radianDivisions[i-1]){
			i++;
		}
		
		code += i;
		
		return decode(code);
	}
	
	private static Set<Character> AType = new TreeSet<Character>(ATypes);

	/** 
	 * 10: Activation		(A)
	 * 20: Continuation		(C)
	 * 30: Termination		(T)
	 * 1-numDivisions: 
	 */ 
	private static String decode(int code){
		List<Character> charList;
		switch (code/10){
		case 1:
			charList = ATypes;
			break;
		case 2:
			charList = CTypes;
			break;
		case 3:
			default:
			charList = TTypes;
		}
		return ""+charList.get(code - 10*(code/10) - 1);
	}
	
	public BinaryTreeNode decodeChar(char character, BinaryTreeNode node) throws EncodeException{
		node.setRight(new BinaryTreeNode());
		node.setLeft(new BinaryTreeNode());
		node.setData(""+character);

		if (ATypes.contains(character) || CTypes.contains(character)){
			node = node.getLeft();
		}else if (TTypes.contains(character)){
			boolean done = false;
			while (node.getParent() != null && !done){
				if (!AType.contains(node.getParent().getData()) || node.isRight()){
					node = node.getParent();
				}
				else if (node.isLeft()){
					node = node.getParent().getRight();
					done = true;
				}
			}
		}else{
			throw new EncodeException(
					"Character "+character+" not part of set for Encoder "+this.getClass());
		}
		return node;
	}
}
