package org.cn3.ns.resources;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.cn3.morpho.data.NeuronTreeClassification;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

public class DataProperties {

	private static DataProperties instance = null;
	private static Properties dataProperties = null;

	public static DataProperties getInstance() throws IOException{
		if (instance == null){
			instance = new DataProperties();
		}
		return instance;
	}
	
	private DataProperties() throws IOException{
		dataProperties = new Properties();
		dataProperties.load(new FileInputStream("resources/conf/data.properties"));
	}
	
	public Properties getDataProperties() {
		return dataProperties;
	}
	
	public String getWorkingDir() throws IOException{
		return ProjectProperties.getInstance().getWorkingDir();
	}
	public String getResourceDirectory() throws IOException{
		return ProjectProperties.getInstance().getResourceDirectory();
	}
	public String getOutlierDirectory() throws IOException{
		return getWorkingDir()+dataProperties.getProperty(NSConstants.OUTLIERS);
	}
	
	public String getMetaDataFilename() throws IOException{
		return getResourceDirectory()+dataProperties.getProperty(NSConstants.META_DATABASE_FILENAME);
	}

	public String getSwcDir() throws IOException{
		return dataProperties.getProperty(NSConstants.SWC_LOCATION)+"/";
	}
	public String getSwcDir(String dataset) throws IOException{
		return dataProperties.getProperty(NSConstants.SWC_LOCATION)+"/"+dataset+"/";
	}
	public String getFastaDirByClassification(String dataset, String classification) throws IOException{
		return dataProperties.getProperty(NSConstants.FASTA_LOCATION)+"/"+dataset+"/"+classification+"/";
	}

	public String getGlobalFastaDir() throws IOException{
		return dataProperties.getProperty(NSConstants.FASTA_LOCATION)+"/";
	}
	public String getFastaDir() throws IOException{
		return dataProperties.getProperty(NSConstants.FASTA_LOCATION)+"/"+NSConstants.DEFAULT_DATASET+"/";
	}
	public String getFastaDir(String dataset) throws IOException{
		return dataProperties.getProperty(NSConstants.FASTA_LOCATION)+"/"+dataset+"/";
	}
	public String getFastaDirByClassification(String classification) throws IOException{
		return dataProperties.getProperty(NSConstants.FASTA_LOCATION)+"/"+NSConstants.DEFAULT_DATASET+"/"+classification+"/";
	}
	public String getFastaArborDir(String dataset) throws IOException{
		return getFastaDir(dataset)+"Arborization/";
	}
	public String getFastaArborDir() throws IOException{
		return getFastaDir()+"Arborization/";
	}
	public String getFastaDir(String dataset, int mapping, 
			String classification)
	throws IOException{
		if (classification.equals("Arborization")){
			classification = "Arbor";
		}
		return DataProperties.getInstance().getFastaArborDir(dataset)+
			"/Mapping"+mapping+"/Size/";
	}

	public String getFastaFilename()
	throws IOException{
		return getFastaFilename(NSConstants.DEFAULT_DATASET, NSConstants.DEFAULT_MAPPING_MODE, 
				NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, 
				NSConstants.DEFAULT_ORDER);
	}
	public String getFastaFilename(int mapping)
	throws Exception{
		return getFastaFilename(NSConstants.DEFAULT_DATASET, mapping, 
				NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, 
				NSConstants.DEFAULT_ORDER);
	}
	public String getFastaFilename(String dataset, int mapping)
	throws IOException{
		return getFastaFilename(dataset, mapping, NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, NSConstants.DEFAULT_ORDER);
	}
	public String getFastaFilename(String dataset, int mapping, String order)
	throws IOException{
		return getFastaFilename(dataset, mapping, NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, order);
	}
	public String getFastaFilename(int mapping, String order)
	throws IOException{
		return getFastaFilename(NSConstants.DEFAULT_DATASET, mapping, NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, order);
	}
	public String getFastaFilename(String dataset, int mapping, 
			NeuronTreeClassification classification, String traversal, String order)
	throws IOException{
		String classStr = classification.toString();
		if (classification == NeuronTreeClassification.ARBORIZATION){
			classStr = "Arbor";
		}else if (classification == NeuronTreeClassification.SUB_ARBORIZATION){
			classStr = "SubArbor";
		}
		return DataProperties.getInstance().getFastaArborDir(dataset)+
			"/Mapping"+mapping+"/Size/"+dataset+classStr+traversal+order+"Map"+mapping+".fasta";
	}
	public String getQueryFastaFilename(int mapping, String order, String queryName, String lab)
	throws IOException{
		return getQueryFastaFilename(mapping, NSConstants.DEFAULT_TRAVERSAL, order, queryName, lab);
	}
	public String getQueryFastaFilename(int mapping, String traversal, String order, String queryName, String lab)
	throws IOException{
		return DataProperties.getInstance().getFastaArborDir()+"/Mapping"+mapping+"/"+traversal+"/"+lab+"/"+queryName
			+"."+order.toLowerCase()+".fasta";
	}
	
	public String getNewSequencesFastaFilename(String dataset, int mapping)
	throws IOException{
		return getNewSequencesFastaFilename(dataset, mapping, NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, NSConstants.DEFAULT_ORDER);
	}
	public String getNewSequencesFastaFilename(String dataset, int mapping, String order)
	throws IOException{
		return getNewSequencesFastaFilename(dataset, mapping, NSConstants.DEFAULT_CLASSIFICATION, NSConstants.DEFAULT_TRAVERSAL, order);
	}
	public String getNewSequencesFastaFilename(String dataset, int mapping, 
			NeuronTreeClassification classification, String traversal, String order)
	throws IOException{
		String classStr = classification.toString();
		if (classification == NeuronTreeClassification.ARBORIZATION){
			classStr = "Arbor";
		}else if (classification == NeuronTreeClassification.SUB_ARBORIZATION){
			classStr = "SubArbor";
		}
		return DataProperties.getInstance().getFastaArborDir()+
			"/Mapping"+mapping+"/Size/"+dataset+classStr+traversal+order+"Map"+mapping+"-New.fasta";
	}
		
	public String getFastaRandomsDir(String dataset) throws IOException{
		return getFastaDir(dataset)+"/Random/";
	}
	public String getFastaRandomsDir() throws IOException{
		return getFastaDir()+"/Random/";
	}
	public String getResultsDir() throws IOException{
		return dataProperties.getProperty(NSConstants.RESULTS_LOCATION);
	}
	public String getResultsLookupDir() throws IOException{
		return dataProperties.getProperty(NSConstants.RESULTS_LOOKUP_LOCATION);
	}
	public String getFlaggedNeuronsFile() throws IOException{
		return dataProperties.getProperty(NSConstants.FLAGGED_NEURONS_FILE);
	}
	
	public String getOutlierFile(int mapping) throws IOException{
		return getOutlierFile(mapping, NSConstants.DEFAULT_ORDER, NSConstants.DEFAULT_BASELINE, 0);
	}
	public String getOutlierFile(int mapping, String order) throws IOException{
		return getOutlierFile(mapping, order, NSConstants.DEFAULT_BASELINE, 0);
	}
	public String getOutlierFile(int mapping, String order, int method) throws IOException{
		return getOutlierFile(mapping, order, method, 0);
	}
	public String getOutlierFile(int mapping, String order, int method, float threshold) throws IOException{
		return getOutlierFile(mapping, order, method, threshold, "");
	}
	public String getOutlierFile(int mapping, String order, int method, double threshold, String thresholdModifier) 
	throws IOException{
		return getOutlierDirectory()+"Map"+mapping+order+"Method"+method+"Thresh"+threshold+thresholdModifier+"Outliers.txt";
	}

	public String getDimerTooUnlikelyFile(int mapping, int method) throws IOException{
		return getResourceDirectory()
		+dataProperties.getProperty(NSConstants.DIMER_TOO_UNLIKELY+".map"+mapping+"method"+method);
	}
	public String getTrimerTooUnlikelyFile(int mapping, int method) throws IOException{
		return getResourceDirectory()+dataProperties.getProperty(NSConstants.TRIMER_TOO_UNLIKELY+".map"+mapping+"method"+method);
	}

	public double getMedianLengthToBranch() throws IOException{
		return Double.parseDouble(
				dataProperties.getProperty(NSConstants.GEOMETRY_MEDIAN_LENGTH));
	}

	public double getMedianBranchAngle() throws IOException{
		return Double.parseDouble(
				dataProperties.getProperty(NSConstants.GEOMETRY_MEDIAN_ANGLE));
	}
	
	public String getSequenceLengthsFile(int mapping) throws IOException{
		return getSequenceLengthsFile(mapping, "arborization", NSConstants.DEFAULT_DATASET);
	}
	public String getSequenceLengthsFile(int mapping, String arborType) throws IOException{
		return getSequenceLengthsFile(mapping, arborType, NSConstants.DEFAULT_DATASET);
	}
	public String getSequenceLengthsFile(int mapping, String arborType, String dataset) throws IOException{
		return dataProperties.getProperty(
				NSConstants.LENGTH_DATABASE+"."+dataset+"."+arborType.toLowerCase()+
				".type"+NSConstants.mappingModeSequenceType[mapping]);
	}
	public String getSequenceLengthsFileBySeqType(int seqType, String sequenceType) throws IOException{
		return getSequenceLengthsFileBySeqType(seqType, sequenceType, NSConstants.DEFAULT_DATASET);
	}
	public String getSequenceLengthsFileBySeqType(int seqType, String arborType, String dataset) throws IOException{
		return dataProperties.getProperty(
				NSConstants.LENGTH_DATABASE+"."+dataset+"."+arborType.toLowerCase()+
				".type"+seqType);
	}

	public String getWellRepresentedNeuriteClassesFilename() throws IOException{
		return "resources/WellRepresentedNeuriteClasses.txt";
	}

	public int getCharactersPerMapping(int mapping){
		switch (mapping){
		case NSConstants.MAPPING_MODE_TOPO_CA:
			return 2;
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			return 3;
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			return 4;
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			return 20;
		case NSConstants.MAPPING_MODE_TOPO_LTB_AOB_SEPARATE:
			return 4;
		default:
			return 0;
		}
	}

	public int getDatasetTypeId(String dataset){
		return Integer.parseInt(dataProperties.getProperty("data.datasetTypeId."+dataset));
	}
	
	/**
	 * @param queryName
	 * @return File if it exists (using default dataset), otherwise null
	 * @throws Exception
	 */
	public File getSwcFileByName(String queryName) throws Exception{
		return getSwcFileByName(NSConstants.DEFAULT_DATASET, queryName);
	}
	/**
	 * @param dataset
	 * @param queryName
	 * @return File if it exists, otherwise null
	 * @throws Exception
	 */
	public File getSwcFileByName(String dataset, String queryName) throws Exception{
		String neuronName = NeuriteSequenceStringUtils.getNeuronNameFromSequence(queryName);
		if (neuronName == null) neuronName = queryName;
		String archive = ClassificationUtils.getArchiveByNeuron(neuronName);
		String fileBase = getSwcDir(dataset)+"/"+archive.toLowerCase()+"/CNG version/";
		File file = new File(fileBase);
		if (!file.exists()){
			fileBase = getSwcDir(dataset)+"/"+archive.toLowerCase();
			file = new File(fileBase);
			if (!file.exists()){
				System.err.println("File "+file.getAbsolutePath()+" does not exist");
				return null;
			}
		}
		String filename = fileBase + neuronName + ".CNG.swc";
		file = new File(filename);
		if (!file.exists()){
			filename = fileBase + neuronName + ".swc";
			file = new File(filename);
			if (!file.exists()){
				return null;
			}
		}
		return file;
	}

	public File getNumCConstrainedTreeShapesFile() throws Exception{
		return new File("resources/NumCConstrainedTreeShapes.bin");
	}

	public String getNeuromorphoSequenceFile(int mapping, String order) throws Exception{
		return ProjectProperties.getInstance()
			.getResourceDirectory()+"fullMap"+mapping+order+"DatabaseSequence.txt";
	}

}
