package org.cn3.ns.scripts;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.data.DataConverter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.data.persistence.GACTCompressor;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class EqualSubtreesFinder {

	private static final String PARAM_THRESHOLD = "threshold";
	private static final String PARAM_SET_NUM = "set-num";
	private static final String PARAM_METHOD = "method";
	
	private static final int mapping = NSConstants.MAPPING_MODE_TOPO_ACT;	
	private static int method = NSConstants.RANDOM_UNIFORM_DIST;
	private static float threshold = 0;
	private static int setNum = 0;
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			method = config.getInt(PARAM_METHOD);
			threshold = config.getFloat(PARAM_THRESHOLD);
			setNum = config.getInt(PARAM_SET_NUM);
			
			FastaReader fr = new FastaReader(DataProperties.getInstance().getFastaArborDir()+"/Mapping2/Size/NMArborSizeLTSMap2.fasta");
			BinaryTreeNode tree, node;
			Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
			boolean check;
			Set<String> foundTrees = new HashSet<String>();
			
			while (fr.next()){
				tree = DataConverter.fastaACTStringToTree(fr.getSequence());
				stack.push(tree);
				check = false;
				while (!stack.isEmpty()){
					node = stack.pop();
					if (node.getRight().hasChildren()){
						stack.push(node.getRight());
						check = true;
					}
					if (node.getLeft().hasChildren()){
						stack.push(node.getLeft());
						check = true;
					}
					else{
						check = false;
					}
					if (check && 
							node.getRight().getDegree() == node.getLeft().getDegree() &&
							node.getRight().getDegree() > 3 &&
							node.getRight().getShapeNum() != node.getLeft().getShapeNum()){
						foundTrees.add(fr.getSequenceName());
						System.out.println(fr.getSequenceName()+"; Total Degree: "+tree.getDegree()+"; Equal degree: "+node.getRight().getDegree());
						break;
					}
				}
				while (!stack.isEmpty()){
					stack.pop();
				}
			}
			
			if (setNum > 0){
				removeFromResults(foundTrees);
				removeFromOutliers(foundTrees);
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static void removeFromOutliers(Set<String> equalSubtreesSet) throws Exception{
		FileReader freader = null;
		BufferedReader reader = null;
		
		StringBuffer sb = new StringBuffer();
		File file = new File(DataProperties.getInstance().getOutlierFile(mapping));
		try{
			freader = new FileReader(file);
		    reader = new BufferedReader(freader);

			String dataString = "";
			String[] fields;
			
			while ( (dataString=reader.readLine()) != null ) {
		        fields = dataString.split(",");
	        	if (!equalSubtreesSet.contains(fields[0])){
	        		sb.append(dataString+"\r\n");
		        }
		    }
			
		} catch (IOException ioe){
			throw new Exception(ioe);
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}
		DataWriter.writeStringToFile(file, sb.toString());
	}
	private static void removeFromResults(Set<String> equalSubtreesSet) throws Exception{
		FileReader freader = null;
		BufferedReader reader = null;
		
		// Get sequence names and map them to unique ids
		List<String> sequenceNames = NSDataReader.loadSequenceList(mapping);
		Map<String,Integer> sequenceIdMap = new HashMap<String,Integer>();
		for (int j = 0; j < sequenceNames.size(); j++){
			sequenceIdMap.put(sequenceNames.get(j), j);
		}
		
		int kept = 0, removed = 0;
		StringBuffer sb = new StringBuffer();
		String sequenceName, sequence;
		File file = new File(getConstrainedTreesFilename());
		if (file.exists()){
			try{
				freader = new FileReader(file);
			    reader = new BufferedReader(freader);
	
				String dataString = "";
				
				Pattern p = Pattern.compile("(\\d+)\\|(\\d)\\-(\\d)");
				Matcher m;
				int id;
				
				while ( (dataString=reader.readLine()) != null ) {
			        m = p.matcher(dataString);
			        if (m.matches()){
			        	id = Integer.parseInt(m.group(1));
			        	sequenceName = (String)sequenceNames.get(id);
			        	sequence = GACTCompressor.decompressSequence(reader);
			        	if (!equalSubtreesSet.contains(sequenceName)){
			        		sb.append(dataString+"\r\n");
			        		sb.append(GACTCompressor.compressSequenceIntoString(sequence)+"\r\n");
			        		kept++;
			        	}
			        	else{
			        		removed++;
			        		System.out.println("Removed "+sequenceName);
			        	}
			        }
			    }
				
				
			} catch (IOException ioe){
				throw new Exception(ioe);
			} finally{
				if (reader != null){
					reader.close();
				}
				if (freader != null){
					freader.close();
				}
			}
			DataWriter.writeStringToFile(file, sb.toString());
			System.out.println("Kept: "+kept);
			System.out.println("Removed: "+removed);
		}
		
	}
	
	private static String getConstrainedTreesFilename() throws Exception{
		return DataProperties.getInstance().getFastaRandomsDir()+"/Method"+method+
			"/Mapping2/constrainedTrees-set"+setNum+"-threshold-"+threshold+".dat";
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_THRESHOLD)
            .setStringParser(JSAP.FLOAT_PARSER)
            .setDefault("0")
            .setRequired(false)
            .setShortFlag('T') 
            .setLongFlag(PARAM_THRESHOLD);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_SET_NUM)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag('s') 
	        .setLongFlag(PARAM_SET_NUM);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_METHOD)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+NSConstants.DEFAULT_BASELINE)
	        .setShortFlag('m') 
	        .setLongFlag(PARAM_METHOD);
	    jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + EqualSubtreesFinder.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

	
}
