package org.cn3.ns.scripts;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadataComplete;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Creates a csv file containing sequence names and their length
 * 
 * java org.cn3.ns.scripts.GenerateSequenceLengthFile -d <dataset> -O <STL|LTS> -o <outputDir> -m <mapping>
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.GenerateSequenceLengthFile -d NM_SpurLen2D -o "/home/tgillett/results/" -O STL
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.GenerateSequenceLengthFile -d NM -o "/home/tgillett/results/" -O STL
 *
 */
public class GenerateSequenceLengthFile {

	private static final String PARAM_OUTPUT_DIR = "outputDir";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String dataset = config.getString(NSConstants.PARAM_DATASET);
			String outputDir = config.getString(PARAM_OUTPUT_DIR);
			int mapping = config.getInt(NSConstants.PARAM_MAPPING_MODE);
			String travOrder = config.getString(NSConstants.PARAM_ORDER);
			
			List<String> arborizations = NSDataReader.loadSequenceListFromFastaFile(mapping, dataset, travOrder, false);
			Map<String,NeuronMetadataComplete> metaData = MetaDataManager.getExtendedCellMetaData();
			Map<String,Integer> lenMap = NSDataReader.loadLengthDatabaseFromFastaFile(mapping, dataset, travOrder);
			String querySequence;
			String output = outputDir+"/SequenceLengths-"+dataset+"-MapType"+NSConstants.mappingModeSequenceType[mapping]+".csv";
			NeuronMetadataComplete neuron;
			int i;
			
			StringBuffer sb = new StringBuffer();
			// col labels
			//sb.append("Sequence,ArborType,CellType,BrainRegion,Animal,Archive,SeqLen,Region2,Region3,Class1,Class2,Class3,Gender,Protocol,Strain\n");
			sb.append("Sequence,SeqLen\n");
			DataWriter.writeStringToFile(output, sb.toString());
			
			// Last result set is irrelevent
			Set<String> used = new TreeSet<String>();
			for (i = 0; i < arborizations.size(); i++){
				querySequence = arborizations.get(i);
				neuron = metaData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(querySequence));
				if (used.contains(querySequence)){
					System.out.println(i+" " +querySequence);
				}
				if (neuron != null){
					sb = new StringBuffer();
					sb.append(querySequence);
					sb.append(","+lenMap.get(querySequence));
					sb.append("\n");
					DataWriter.appendStringToFile(output, sb.toString());
					used.add(querySequence);
				}
			}
			System.out.println("Done generating sequence length csv file: "+output);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(NSConstants.PARAM_DATASET)
			.setStringParser(JSAP.STRING_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.DEFAULT_DATASET)
			.setShortFlag('d') 
			.setLongFlag(NSConstants.PARAM_DATASET);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_OUTPUT_DIR)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(false)
		.setDefault("C:/")
		.setShortFlag('o') 
		.setLongFlag(PARAM_OUTPUT_DIR);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(NSConstants.PARAM_ORDER)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(false)
		.setDefault(NSConstants.DEFAULT_ORDER)
		.setShortFlag('O') 
		.setLongFlag(NSConstants.PARAM_ORDER);
        jsap.registerParameter(opt);

    	opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.DEFAULT_MAPPING_MODE)
			.setShortFlag('M') 
			.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);
        	    
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + GenerateSequenceLengthFile.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
