package org.cn3.ns.scripts;

import java.text.NumberFormat;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.Asymmetry;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.generation.BinaryTreeGenerator;
import org.cn3.morpho.utils.BinaryTreeUtils;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.generation.NodeSequenceBinaryTreeGenerator;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappingUtils;

/**
 * 
 * @author gillette
 * Generates random trees produces with terminal and segmental growth models over various tree sizes 
 * and measures asymmetry, caulescence, %C, and dimer percentile ranks.
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.GrowthModelKmers
 *
 */
public class GrowthModelKmers {

	private static final int TREES_PER_SIZE = 50;
	//private static final String OUTPUT_DIRECTORY = "C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/";
	private static final String OUTPUT_DIRECTORY = "/home/tgillett/results/MotifAnalysis/";

	private static String[] trimersToTestArr = {"ACC","ACT","TCT","TCA","TCC","CTA","CAT","AAT","ATA","CAC"};
	//private static String[] dimersToTestArr = {"AT","CT","AA","CC","AC"};
	private static String[] dimersToTestArr = {"AA","AC","AT","CA","CC","CT","TA","TC","TT"};
	private static NumberFormat nf = NumberFormat.getInstance();
	public static void main(String[] args){
		try{
			BinaryTreeNode tree;
			String sequence;
			int termDegree, size;
			Map<String,Number> motifs;
			/*StringBuffer terminalFasta = new StringBuffer();
			StringBuffer segmentalFasta = new StringBuffer();
			StringBuffer apicalModelFasta = new StringBuffer();
			StringBuffer axonModelFasta = new StringBuffer();*/
			StringBuffer biasedTerminalFasta = new StringBuffer();
			List<String> trimersToTest = Arrays.asList(trimersToTestArr);
			List<String> dimersToTest = Arrays.asList(dimersToTestArr);
			
			StringBuffer datSb = new StringBuffer();
			datSb.append("Growth,Length,Asymmetry,Caulescence,PercentC");
			/*
			for (String dimer : dimersToTest){
				datSb.append(","+dimer);
			}
			for (String trimer : trimersToTest){
				datSb.append(","+trimer);
			}*/
			datSb.append("\n");
			
			nf.setMaximumFractionDigits(4);
			
			Map<String,Integer> lengthMap = NSDataReader.loadLengthDatabaseFromFastaFile(
					NSConstants.MAPPING_MODE_TOPO_ACT, "NM_SpurLen2D", "STL");
			//List<Integer> lengths = NSDataReader.loadUniqueSequenceLengths(lengthMap);

			NodeSequenceBinaryTreeGenerator.debugLevel = 0;
			
			// Loop through set lengths and generate set of trees
			for (int length = 18; length < 200; length++){
			//for (int length : lengths){
				if (length < 200){
					System.out.println("Sequence Length "+length);
					termDegree = length+1;
					size = termDegree + length; // size = number of branches
					for (int treeNum = 1; treeNum <= TREES_PER_SIZE; treeNum++){
						System.out.println("Tree "+treeNum);
						/*
						// Generate terminal growth
						tree = BinaryTreeGenerator.generateTerminalGrowthBinaryTreeByDegree(termDegree);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");

						//terminalFasta.append(">TerminalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						//System.out.println(motifs.keySet());
						datSb.append("Terminal,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");
						
						// Generate segmental growth
						tree = BinaryTreeGenerator.generateSegmentalGrowthBinaryTreeByDegree(termDegree);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Segmental,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");


						// Q0.7S0 - Biased towards interstitial
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0.7, 0);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0.7S0,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");
*/
						// Q0S0.4 - Terminal with low-order bias
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0, 0.4);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						biasedTerminalFasta.append(">TerminalS0.4_L"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0S0.4,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						//datSb.append(addDimers(termDegree,motifs));
						//datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");
/*
						// Q0S0.4
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0, 0.4);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0S0.4,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");

						// Q0S-0.4
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0, -0.4);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0S-0.4,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");


						// Q0.35S-0.4
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0.35, -0.4);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0.35S-0.4,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");

						
						// Q0.35S0.5
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0.35, 0.5);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0.35S0.5,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");

						// Q0.35S0.3
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, 0.35, 0.3);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,3);
						datSb.append("Q0.35S0.3,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addTrimers(termDegree,motifs,trimersToTest)+"\n");
*/
					}
				}
			}
			
			// Output results
			//String filename = OUTPUT_DIRECTORY+"TargetQSModelsTrimerPRs.csv";
			String filename = OUTPUT_DIRECTORY+"SelectQSModelsLargeKmerPRs.csv";
			//DataWriter.writeStringToFile(filename, datSb.toString());
			//DataWriter.appendStringToFile(filename, datSb.toString());
//			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"TerminalGrowthSequences.fasta", axonModelFasta.toString());
			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"BiasedTerminalGrowthSequences.fasta", biasedTerminalFasta.toString());
			System.out.println("Wrote to "+filename);
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	private static DistributionBean[] trimerDists = new DistributionBean[trimersToTestArr.length];
	private static String addTrimers(int termDegree, Map<String,Number> treeMotifs, List<String> trimersToTest) throws Exception{
		StringBuffer sb = new StringBuffer();
		
		BinaryTreeNode tree;
		String sequence;
		Map<String,Number> surrogateMotifs;
		for (int trimerNum = 0; trimerNum < trimersToTest.size(); trimerNum++){
			trimerDists[trimerNum] = new DistributionBean();
		}

		// Generate monomer constrained surrogates
		//System.out.println(termDegree+" " +treeMotifs.get("C").intValue());
		for (int i = 0; i < 60; i++){
			tree = NodeSequenceBinaryTreeGenerator.generateRandomDimerConstrainedNonuniformTree(treeMotifs,"STL");
			sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			surrogateMotifs = MotifFinder.findMotifs(sequence,3);
			for (int trimerNum = 0; trimerNum < trimersToTest.size(); trimerNum++){
				trimerDists[trimerNum].addData(surrogateMotifs.get(trimersToTest.get(trimerNum)));
			}
		}
		
		for (int trimerNum = 0; trimerNum < trimersToTest.size(); trimerNum++){
			sb.append(","+nf.format(trimerDists[trimerNum].getPercentileRankForValue(treeMotifs.get(trimersToTest.get(trimerNum)).doubleValue())));
		}
		
		return(sb.toString());
	}
	
	private static DistributionBean[] dimerDists = new DistributionBean[dimersToTestArr.length];
	private static String addDimers(int termDegree, Map<String,Number> treeMotifs) throws Exception{
		StringBuffer sb = new StringBuffer();
		BinaryTreeNode tree;
		String sequence;
		Map<String,Number> surrogateMotifs;
		for (int dimerNum = 0; dimerNum < dimersToTestArr.length; dimerNum++){
			dimerDists[dimerNum] = new DistributionBean();
		}

		// Generate monomer constrained surrogates
		//System.out.println(termDegree+" " +treeMotifs.get("C").intValue());
		for (int i = 0; i < 60; i++){
			tree = NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(termDegree, treeMotifs.get("C").intValue());
			sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			surrogateMotifs = MotifFinder.findMotifs(sequence,2);
			for (int dimerNum = 0; dimerNum < dimersToTestArr.length; dimerNum++){
				dimerDists[dimerNum].addData(surrogateMotifs.get(dimersToTestArr[dimerNum]));
			}
		}
		
		for (int dimerNum = 0; dimerNum < dimersToTestArr.length; dimerNum++){
			sb.append(","+nf.format(dimerDists[dimerNum].getPercentileRankForValue(treeMotifs.get(dimersToTestArr[dimerNum]).doubleValue())));
		}
		
		return(sb.toString());
	}
	
}
