package org.cn3.ns.scripts;

import java.util.LinkedList;
import java.util.List;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class RandomizeFastaSequenceOrder {

	private static final String PARAM_INPUT_FILENAME = "input";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String inputFilename = config.getString(PARAM_INPUT_FILENAME);

			String outputFilename = inputFilename.substring(0,inputFilename.length()-6)+".reordered.fasta";
			
			List sequences = new LinkedList();
			FastaReader fr = new FastaReader(inputFilename);
			while (fr.next()){
				sequences.add(fr.getHeader()+"\r\n"+
						NeuriteSequenceStringUtils.insertBreaksAtLength(fr.getSequence(), 80)+"\r\n");
			}
			
			StringBuffer sb = new StringBuffer();
			String sequenceString;
			while (sequences.size() > 0){
				sequenceString = (String)sequences.remove((int)(sequences.size()*Math.random()));
				sb.append(sequenceString);
			}
			
			DataWriter.appendStringToFile(outputFilename, sb.toString());

		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(PARAM_INPUT_FILENAME)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('I') 
        .setLongFlag(PARAM_INPUT_FILENAME);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + RandomizeFastaSequenceOrder.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
	
}
