package org.cn3.ns.scripts;

import java.io.File;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * STL
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.SplitFastaByArborTypes -f /home/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeSTLMap2.fasta
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.SplitFastaByArborTypes -f /home/tgillett/fastaFromSwc/NM_SpurLen5/Arborization/Mapping2/Size/NMArborSizeSTLMap2_SpurLen5.fasta
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.SplitFastaByArborTypes -f /home/tgillett/fastaFromSwc/NM_SpurLen5/Arborization/Mapping2/Size/NM_SpurLen5ArborSizeSTLMap2.fasta
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.SplitFastaByArborTypes -f /home/tgillett/fastaFromSwc/NM_SpurLen2D/Arborization/Mapping2/Size/NM_SpurLen2DArborSizeSTLMap2.fasta
 * 
 * LTS
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.SplitFastaByArborTypes -f /home/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.fasta
 */
public class SplitFastaByArborTypes {

	private static final String PARAM_FASTA_FILE = "fastaFile";
	
	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			File fastaFile = new File(config.getString(PARAM_FASTA_FILE));
			
			StringBuffer axons = new StringBuffer();
			StringBuffer dendrites = new StringBuffer();
			StringBuffer apicals = new StringBuffer();
			FastaReader fr = new FastaReader(fastaFile);
			String arborType;
			
			while (fr.next()){
				arborType = NeuriteSequenceStringUtils.getArborizationType(fr.getSequenceName());
				if (arborType.equals("Axon")){
					axons.append(fr.getHeader()+"\n"+fr.getSequence()+"\n");
				}
				else if (arborType.equals("ApicalDendrite")){
					apicals.append(fr.getHeader()+"\n"+fr.getSequence()+"\n");
				}
				else{
					dendrites.append(fr.getHeader()+"\n"+fr.getSequence()+"\n");
				}
			}
			fr.close();
			
			String dir = fastaFile.getParent();
			String baseFilename = fastaFile.getName().replace("ArborSize", "Arbor");
			
			DataWriter.writeStringToFile(dir+"/"+baseFilename.replace("Arbor", "Axon"),axons);
			DataWriter.writeStringToFile(dir+"/"+baseFilename.replace("Arbor", "Apical"),apicals);
			DataWriter.writeStringToFile(dir+"/"+baseFilename.replace("Arbor", "Dendrite"),dendrites);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_FASTA_FILE)
			.setStringParser(JSAP.STRING_PARSER)
			.setRequired(true)
			.setShortFlag('f') 
			.setLongFlag(PARAM_FASTA_FILE);
        jsap.registerParameter(opt);
        
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + SplitFastaByArborTypes.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
