package org.cn3.ns.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.LinkedQueue;
import org.cn3.morpho.utils.BinaryTreeUtils;
import org.cn3.ns.domain.CNode;
import org.cn3.ns.nodeencoding.NodeEncoder;
import org.cn3.ns.nodeencoding.NodeEncoderFactory;
import org.cn3.ns.resources.EncodingConstants;
import org.cn3.ns.resources.NSConstants;

/**
 * @author gillette
 *
 */
public class MappedBinaryTreeUtils extends BinaryTreeUtils {

	public static double getCNodeFraction(BinaryTreeNode tree){
		int size = 0;
		int cCount = 0;
		LinkedQueue queue = new LinkedQueue();
		queue.enqueue(tree);
		BinaryTreeNode node;
		while (!queue.isEmpty()){
			node = (BinaryTreeNode)queue.dequeue();
			size++;
			if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
					|| (!node.getLeft().hasChildren() && node.getRight().hasChildren())){
				cCount++;
			}
			if (node.getLeft().hasChildren()){
				queue.enqueue(node.getLeft());
			}
			if (node.getRight().hasChildren()){
				queue.enqueue(node.getRight());
			}
		}
		
		return (double)cCount/size;
	}
	
	public static BinaryTreeNode createATT(){
		BinaryTreeNode node = new BinaryTreeNode();
		
		node.setLeft(new BinaryTreeNode());
		node.getLeft().setLeft(new BinaryTreeNode());
		node.getLeft().setRight(new BinaryTreeNode());

		node.setRight(new BinaryTreeNode());
		node.getRight().setLeft(new BinaryTreeNode());
		node.getRight().setRight(new BinaryTreeNode());
		
		return node;
	}
	
	public static BinaryTreeNode createCT(){
		CNode cNode = new CNode();
		
		cNode.setChild(new BinaryTreeNode());
		cNode.getChild().setLeft(new BinaryTreeNode());
		cNode.getChild().setRight(new BinaryTreeNode());

		return cNode;
	}

	public static BinaryTreeNode createT(){
		BinaryTreeNode node = new BinaryTreeNode();
		
		node.setLeft(new BinaryTreeNode());
		node.setRight(new BinaryTreeNode());

		return node;
	}

	public static void setCChild(BinaryTreeNode parent, BinaryTreeNode child){
		parent.setRight(child);
	}
	
	public static boolean isTNode(BinaryTreeNode node){
		return (node.hasChildren() && node.getLeft().isLeaf() && node.getRight().isLeaf());
	}

	public static boolean isANode(BinaryTreeNode node){
		return (node.hasBothChildren() && node.getLeft().hasChildren() && node.getRight().hasChildren());
	}

	public static boolean isCNode(BinaryTreeNode node){
		return (node.hasBothChildren() && 
				(node.getLeft().isLeaf() ^ node.getRight().isLeaf()));
	}
	
	public static int getACTMonomerType(BinaryTreeNode node){
		if (isANode(node)){
			return EncodingConstants.A;
		}
		if (isCNode(node)){
			return EncodingConstants.C;
		}
		if (isTNode(node)){
			return EncodingConstants.T;
		}
		return EncodingConstants.UNDEFINED;
	}

	public static int getDegreeFromAC(int aNodes, int cNodes){
		return cNodes + 2*aNodes + 2;
	}

	public static int getAFromCDegree(int degree, int cNodes){
		return (degree - cNodes)/2 - 1;
	}
	
	public static String reorderSequence(String originalSequence, String order) throws Exception{
		return reorderSequence(originalSequence, order, NSConstants.MAPPING_MODE_TOPO_ACT);
	}
	public static String reorderSequence(String originalSequence, String order, int mapping) throws Exception{
		NodeEncoder nodeEncoder = NodeEncoderFactory.createNewNodeEncoder(mapping);
		BinaryTreeNode tree = nodeEncoder.sequenceToTree(originalSequence);
		return nodeEncoder.encodeTree(tree, order);
		//BinaryTreeNode tree = DataConverter.fastaACTStringToTree(originalSequence);
		//return MappingUtils.traverseTreeToStringWithMappingMode2(tree,order);
	}

	public static List<BinaryTreeNode> getBifurcationList(BinaryTreeNode root, String travOrder)
	throws Exception{
		List<BinaryTreeNode> bifurcationList = new ArrayList<BinaryTreeNode>(root.getDegree()-1);
    	Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
    	BinaryTreeNode current;
    	
    	stack.push(root);
    	while (!stack.isEmpty()){
    		current = stack.pop();
    		//System.out.println("Check: "+mapNodeMode2(current)+"");
    		if (current.hasChildren()) {
    			bifurcationList.add(current);
    			MappingUtils.addChildrenToStack(stack, current, travOrder);
    		}
    	}

		return bifurcationList;
	}
	
}
