package org.cn3.ns.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.utils.StringUtils;

public class NeuriteSequenceStringUtils extends StringUtils {

	public static int getLengthFromName(String name){
		Pattern p = Pattern.compile("LEN(\\d+)NUM(\\d+)");
		Matcher m = p.matcher(name);
		if (m.matches()){
			return (new Integer(m.group(1))).intValue();
		}
		return 0;
	}

	public static int getNumFromName(String name){
		Pattern p = Pattern.compile("LEN(\\d+)NUM(\\d+)");
		Matcher m = p.matcher(name);
		if (m.matches()){
			return (new Integer(m.group(2))).intValue();
		}
		return 0;
	}

	public static Set<String> generatePossibleACTMotifsSet(int length){
		return new TreeSet<String>(generatePossibleACTMotifs(length));
	}
	private static Map<Object,List<String>> actNMers = new HashMap<Object,List<String>>();
	public static List<String> generatePossibleACTMotifs(int length){
		List<String> list = new ArrayList<String>();
		if (length < 1){
			return list;
		}
		Integer lengthObj = new Integer(length);
		if (actNMers.containsKey(lengthObj)){
			return actNMers.get(lengthObj);
		}
		String curr;
		int i, j, num, power, currNum;
		for (i = 0; i < Math.pow(3, length); i++){
			curr = "";
			num = i;
			//System.out.println(i);
			for (j = 1; j <= length; j++){
				power = (int)Math.pow(3,length-j);
				currNum = num / power;
				num = num - currNum * power;
				switch (currNum){
				case 0:
					curr = curr.concat("A");
					break;
				case 1:
					curr = curr.concat("C");
					break;
				case 2:
					curr = curr.concat("T");
					break;
				}
			}
			list.add(curr);
		}
		actNMers.put(lengthObj, list);
		return list;
	}
	
	public static List<String> generatePossibleACTKmers(int length, String order){
		boolean isLTS = order.equalsIgnoreCase("LTS");
		List<String> list = new ArrayList<String>();
		if (length < 1){
			return list;
		}
		if (actNMers.containsKey(length+"LTS")){
			return actNMers.get(length+"LTS");
		}
		Stack<NodeCounter> counterStack;
		List<NodeCounter> counters = new ArrayList<NodeCounter>();
		String curr;
		NodeCounter counter;
		int i, j, k, num, power, currNum;
		boolean keep;
		// Each kmer possibility, bound by 3^length (3 characters, 'length' positions)
		for (i = 0; i < Math.pow(3, length); i++){
			curr = "";
			num = i;
			counter = null;
			// Initialize counter stack and list
			counterStack = new Stack<NodeCounter>();
			counters.clear();
			keep = true; // Assume this kmer is to be kept unless otherwise determined
			// Always assume the case of a parent node to the current kmer
			counter = new NodeCounter(isLTS);
			counter.setAncestryUnknown(true);
			//System.out.println(i);
			// Generate kmer by looping through positions
			for (j = 1; j <= length; j++){
				// Determine character given i and j (num changes at each j)
				//System.out.println(j);
				power = (int)Math.pow(3,length-j);
				currNum = num / power;
				num = num - currNum * power;
				switch (currNum){
				case 0:
					curr = curr.concat("A");
					if (counter != null){
						// Put the existing counter on the stack
						counterStack.push(counter);
					}
					// Signal each existing counter that an A has been added
					for (k = 0; k < counters.size(); k++){
						counters.get(k).a();
					}
					// Create a new counter for the new A
					counter = new NodeCounter(isLTS);
					counters.add(counter);
					break;
				case 1:
					curr = curr.concat("C");
					// Signal each existing counter than a C has been added
					for (k = 0; k < counters.size(); k++){
						counters.get(k).c();
					}
					break;
				case 2:
					curr = curr.concat("T");
					// Signal each existing counter than a T has been added
					for (k = 0; k < counters.size(); k++){
						counters.get(k).t();
					}
					if (counter != null){
						// Remove completed subtrees from the stack and list until an incomplete subtree remains or no trees remain
						boolean keepGoing = true;
						do{
							//System.out.println("StackCheck; isUp: "+counter.isUp());
							// If counter is up, the 1st subtree has just reached its end
							if (counter.isUp()){
								// Flip the counter so counting on the 2nd subtree will begin
								counter.flip();
								// Go on to the next character with this counter, no need to keepGoing (remove more counters)
								keepGoing = false;
							}
							// The counter is down, so this completes a subtree
							else{
								// Check to see whether the first and second subtree counts satisfy traversal order
								if (isLTS){
									if (counter.getCount() < 0){
										//System.out.println("isLTS count < 0 and not a legitimate kmer");
										keep = false;
										j = length;
									}else if (counter.isAncestryUnknown()){
										// When in LTS mode and counters/parent trees are exhausted, then 
										// just continue on with the current counter pretending it is a further ancestor
										//System.out.println("isLTS ancestry unknown");
										keepGoing = false;
									}else {
										// The kmer is still good at this point, so remove the completed subtree counter and access the next
										counters.remove(counter);
										counter = counterStack.pop();
										//System.out.println("isLTS count >= 0");
									}
								}else{
									if (counter.getCount() > 0){
										keep = false;
										j = length;
										//System.out.println("is NOT LTS count > 0 and not a legitimate kmer");
									}else{
										// The kmer is still good at this point
										if (counter.isAncestryUnknown()){
											// unknown ancestor counter needs to be used and its count set to running length
											keepGoing = false;
											counter.setCount(j);
										}else{
											// remove the completed subtree counter and access the next
											counters.remove(counter);
											counter = counterStack.pop();
											
										}
									}
								}
							}
						} while (keep && keepGoing && counter != null);
					}
					break;
				}
			}

			//System.out.println(curr);
			//StringBuffer sb = new StringBuffer();
			if (isLTS){
				for (j = 0; j < counters.size(); j++){
					counter = counters.get(j);
					//sb.append(" "+counter.getCount()+" "+counter.isDown());
					if (counter.getCount() != 0 && counter.isDown() && counter.getCount() < 0){
						keep = false;
					}
				}
			}
			//System.out.println(sb.toString());
			if (keep){
				list.add(curr);
			}
			
		}
		actNMers.put(length+"LTS", list);
		return list;
	}
	
	private static final class NodeCounter{
		private boolean up = true;
		private boolean down = false;
		private boolean lts = true;
		private int aCount = 2;
		private int tCount = 0;
		private int count = 0;
		private boolean ancestryUnknown = false;
		public NodeCounter(boolean isLTS){ 
			setLTS(isLTS);
		}
		private void setLTS(boolean isLTS){
			lts = isLTS;
			if (!lts){
				aCount = 1;
				tCount = 1;
			}
		}
		public void c(){
			count += up ? 1 : -1;
			if (!up) down = true;
		}
		public void a(){
			count += aCount * (up ? 1 : -1);
			if (!up) down = true;
		}
		public void t(){
			count += tCount * (up ? 1 : -1);
			if (!up) down = true;
			//System.out.println("T; "+up+" " +down);
		}
		public void flip(){
			up = false;
		}
		public int getCount(){
			return count;
		}
		public boolean isUp(){
			return up;
		}
		public boolean isDown(){
			return down;
		}
		public boolean isAncestryUnknown(){
			return ancestryUnknown;
		}
		public void setAncestryUnknown(boolean ancestryUnknown){
			this.ancestryUnknown = ancestryUnknown;
		}
		public void setCount(int count){
			this.count = count;
		}
	}

	public static boolean isACTString(String str){
		Pattern p = Pattern.compile("^[ACT]+$", Pattern.CASE_INSENSITIVE);
		Matcher m = p.matcher(str);
		return m.matches();
	}

	public static boolean isResidueString(String str){
		Pattern p = Pattern.compile("^[ARNDCQEGHILKMFPSTWYVBJZX]+$", Pattern.CASE_INSENSITIVE);
		Matcher m = p.matcher(str);
		return m.matches();
	}

	public static void main(String args[]){
		//String str = "AAACCTTcCSTTACTT";
		//System.out.println(str);
		//System.out.println(isACTString(str));
		//System.out.println(isResidueString(str));

		List<String> tmp = generatePossibleACTKmers(5,"LTS");
		for (int i = 0; i < tmp.size(); i++)
			System.out.println(tmp.get(i));
		System.out.println(tmp.size());
	}
	
}
